#!/usr/bin/env python
# coding: utf-8
# 
# author: Ruthard Baudach, <rthrd@web.de>
# Version 1, 22.06.2007
#----------------------------------------------------
# module simulating an interface to the GNUmed database
# storing intermediate data

import wx
import os

#=======================================================================
class cFormDBInterface:
	"""
	Supposed to simulate a middleware and temporary storage.
		
	"""
	def __init__(self):
		"""
		Create backend_connector as database surrogate.
		"""
		self.backend_connector = wx.FileConfig(localFilename=os.path.join(os.getcwd(),'backend.fake'))
		# first group of config is default current_ctrl
		self.backend_connector.SetPath('/')
		# load default current form = first in database
		cont,self.default_current_form,next = self.backend_connector.GetFirstGroup()
		# initialize self.form_defs to prepare the next step
		self.form_defs = {'name':''}
		# initialize self.form_list, just in case someone calls getFormList without loadFormlist
		self.loadFormList()
		
		
	#--------------------------------------------------------
	def loadFormDefinition(self,current_form = ''):
		"""
		Loads form definition from Config - later from GNUmed database.
		
		"""
		# load default, if current_form is empty
		if current_form == '':
			current_form = self.default_current_form
		# set path
		self.backend_connector.SetPath('/'+current_form)
		# define new containers
		self.form_defs = {}
		self.ctrl_defs = {}
		# read definitions
		# self.form_defs['image'] should hold the actual image - wx.Config can't do this, hence the hack with "image" and "image_path"
		self.form_defs['image'] = self.backend_connector.Read('image')
		self.form_defs['image_path'] = self.backend_connector.Read('image_path')
		self.form_defs['width'] = self.backend_connector.Read('width')	
		self.form_defs['height'] = self.backend_connector.Read('height')	
		self.form_defs['soap'] = self.backend_connector.Read('soap')		
		self.form_defs['name'] = current_form
						
		# control definition
		Gcont,Ggroup,Gnext = self.backend_connector.GetFirstGroup()
		while Gcont:
			self.ctrl_defs[Ggroup]={}
			self.ctrl_defs[Ggroup]['type'] = self.backend_connector.Read(Ggroup+'/type')
			self.ctrl_defs[Ggroup]['xpos'] = self.backend_connector.Read(Ggroup+'/xpos')
			self.ctrl_defs[Ggroup]['ypos'] = self.backend_connector.Read(Ggroup+'/ypos')
			self.ctrl_defs[Ggroup]['width'] = self.backend_connector.Read(Ggroup+'/width')
			self.ctrl_defs[Ggroup]['height'] = self.backend_connector.Read(Ggroup+'/height')
			self.ctrl_defs[Ggroup]['default'] = self.backend_connector.Read(Ggroup+'/default')
			self.ctrl_defs[Ggroup]['style'] = self.backend_connector.Read(Ggroup+'/style')
			Gcont,Ggroup,Gnext = self.backend_connector.GetNextGroup(Gnext)
		return self.form_defs, self.ctrl_defs
			
	#---------------------------------------------------------------	
	def loadFormList(self):
		"""		
		Get list of forms - later from GNUmed database.
		"""
		self.form_list = []
		self.backend_connector.SetPath('/')
		cont,group,next = self.backend_connector.GetFirstGroup()
		while cont:
			self.form_list.append(group)
			cont,group,next = self.backend_connector.GetNextGroup(next)
		return self.form_list
				
	#---------------------------------------------------------------	
	def saveFormDefinition(self,form_defs,ctrl_defs):
		
		self.backend_connector.SetPath('/'+form_defs['name'])
		# config can`t store wx.Image objects, thus changing it to the corresponding path
		form_defs['image'] = form_defs['image_path']
		for item in form_defs.keys():
			self.backend_connector.Write(item, self.form_defs[item])
		for group in self.ctrl_defs.keys():
			for item in ctrl_defs[group].keys():
				self.backend_connector.Write(group+'/'+item, ctrl_defs[group][item])
				
	def delFormDefinition(self,form_name):
		self.backend_connector.DelGroup('/'+form_name)
		
	def renForm(old_name,new_name):
		"""
		ToDo: write me.
		"""
		pass 
		
	def renCtrl(old_name,new_name):
		"""
		ToDo: write me.
		"""
		pass
	