#!/usr/bin/env python
# coding: utf-8

import wx
import wx.lib.scrolledpanel
import gmFormDBInterface
import gmFormBase


#========================================================================


#===============================================================================	
class cFormEditor(wx.Panel):
	"""
	
	"""
	def __init__(self,parent,name = 'au'):
		wx.Panel.__init__(self,parent,-1)
		# instanciate the fake backend connector	
		self.formdef = gmFormDBInterface.cFormDBInterface()
		# hand down ancestors
		self.parent = parent
		
		
		# intermediate storage of form editing data - maybe use a list here to enable undo and redo funktions
		self.form_tmp = {'form': {}, 'ctrls': {}, 'current_form': '', 'current_ctrl': ''}
		
		# load initial form definition
		self.form_tmp['form'], self.form_tmp['ctrls'] = self.formdef.loadFormDefinition()
		
		# import intermediate storage variable, define form and control definition containers - split in distinct variables just for typing convenience
		self.form_defs = self.form_tmp['form']
		self.ctrl_defs = self.form_tmp['ctrls']
		self.current_form = self.form_tmp['current_form']
		self.current_ctrl = self.form_tmp['current_ctrl']
		
		
		
		# do layout
		self.sizer = wx.BoxSizer(wx.HORIZONTAL)
		self.properties = cFormPropertyPanel(self)
		self.form_editor_display = cFormEditorDisplay(self)
		self.sizer.Add(self.properties,0,wx.ALL,0)
		self.sizer.Add(self.form_editor_display,1,wx.EXPAND|wx.ALL,0)
		self.SetSizer(self.sizer)
		
		
		
#============================================================================
class cFormPropertyPanel(wx.Panel):
	"""
	Displaying and changing the properties of the form controls.
	"""
	def __init__(self,parent):
		wx.Panel.__init__(self,parent,-1,style=wx.BORDER_SUNKEN)
		# hand down ancestors
		self.parent = parent
		
		# import fake database connector from parent
		self.formdef = parent.formdef
		
		# start to work
		self._create_toolbar()
		self._define_form_variables()
		self._do_layout()
		self._set_values()
				
	def _create_toolbar(self):
		
		toolbar = self.parent.parent.parent.CreateToolBar()
		
		# ---------------------- ID's ----------------------------
		ID_NEW_FORM = wx.NewId()
		ID_RESTORE_FORM = wx.NewId()
		ID_SAVE_FORM = wx.NewId()
		ID_DELETE_FORM = wx.NewId()
		ID_IMAGE_PATH = wx.NewId()
		ID_IMAGE_SCAN = wx.NewId()
		ID_DELETE_IMAGE = wx.NewId()
		ID_NEW_CTRL = wx.NewId()
		ID_LAST_CTRL = wx.NewId()
		ID_NEXT_CTRL = wx.NewId()
		ID_DELETE_CTRL = wx.NewId()
		
		# -------------------------- tools ------------------------
		toolbar.AddControl(wx.StaticText(toolbar,-1,'form:'))
		toolbar.AddSeparator()
		toolbar.AddSimpleTool(ID_NEW_FORM,wx.ArtProvider.GetBitmap(wx.ART_NEW,wx.ART_BUTTON,(16,16)),'new','create new form')
		toolbar.AddSeparator()
		toolbar.AddSimpleTool(ID_RESTORE_FORM,wx.ArtProvider.GetBitmap(wx.ART_UNDO,wx.ART_BUTTON,(16,16)),'restore','restore form to last saved state')
		toolbar.AddSeparator()
		toolbar.AddSimpleTool(ID_SAVE_FORM,wx.ArtProvider.GetBitmap(wx.ART_FILE_SAVE,wx.ART_BUTTON,(16,16)),'save','save current form')
		toolbar.AddSeparator()
		toolbar.AddSimpleTool(ID_DELETE_FORM,wx.ArtProvider.GetBitmap(wx.ART_DELETE,wx.ART_BUTTON,(16,16)),'delete','delete current form')
		toolbar.AddSeparator()
		toolbar.AddSeparator()
		toolbar.AddControl(wx.StaticText(toolbar,-1,'background:'))
		toolbar.AddSeparator()
		toolbar.AddSimpleTool(ID_IMAGE_PATH,wx.ArtProvider.GetBitmap(wx.ART_FILE_OPEN,wx.ART_BUTTON,(16,16)),'image','open background image file')
		toolbar.AddSeparator()
		# todo - get another Bitmap
		toolbar.AddSimpleTool(ID_IMAGE_SCAN,wx.ArtProvider.GetBitmap(wx.ART_NEW_DIR,wx.ART_BUTTON,(16,16)),'scan','scan background image form')
		toolbar.AddSeparator()
		toolbar.AddSimpleTool(ID_DELETE_IMAGE,wx.ArtProvider.GetBitmap(wx.ART_DELETE,wx.ART_BUTTON,(16,16)),'delete','delete background image')
		toolbar.AddSeparator()
		toolbar.AddSeparator()
		toolbar.AddControl(wx.StaticText(toolbar,-1,'control:'))
		toolbar.AddSimpleTool(ID_NEW_CTRL,wx.ArtProvider.GetBitmap(wx.ART_NEW,wx.ART_BUTTON,(16,16)),'new','create new control')
		toolbar.AddSeparator()
		toolbar.AddSimpleTool(ID_LAST_CTRL,wx.ArtProvider.GetBitmap(wx.ART_GO_BACK,wx.ART_BUTTON,(16,16)),'back','go back to last control')
		toolbar.AddSimpleTool(ID_NEXT_CTRL,wx.ArtProvider.GetBitmap(wx.ART_GO_FORWARD,wx.ART_BUTTON,(16,16)),'next','go to next control')
		toolbar.AddSeparator()
		toolbar.AddSimpleTool(ID_DELETE_CTRL,wx.ArtProvider.GetBitmap(wx.ART_DELETE,wx.ART_BUTTON,(16,16)),'delete','delete current control')
		#self.parent.parent.parent.SetToolBar(toolbar)
		#toolbar.Realize()	
		# --------------------- events ----------------------------------
		self.parent.parent.parent.Bind(wx.EVT_TOOL,self._on_new_form,id=ID_NEW_FORM)
		self.parent.parent.parent.Bind(wx.EVT_TOOL,self._on_restore_form,id=ID_RESTORE_FORM)
		self.parent.parent.parent.Bind(wx.EVT_TOOL,self._on_save_form,id=ID_SAVE_FORM)
		self.parent.parent.parent.Bind(wx.EVT_TOOL,self._on_delete_form,id=ID_DELETE_FORM)
		self.parent.parent.parent.Bind(wx.EVT_TOOL,self._on_image_path,id=ID_IMAGE_PATH)
		self.parent.parent.parent.Bind(wx.EVT_TOOL,self._on_scan_image,id=ID_IMAGE_SCAN)
		self.parent.parent.parent.Bind(wx.EVT_TOOL,self._on_delete_image,id=ID_DELETE_IMAGE)
		self.parent.parent.parent.Bind(wx.EVT_TOOL,self._on_new_ctrl,id=ID_NEW_CTRL)
		self.parent.parent.parent.Bind(wx.EVT_TOOL,self._on_last_ctrl,id=ID_LAST_CTRL)
		self.parent.parent.parent.Bind(wx.EVT_TOOL,self._on_next_ctrl,id=ID_NEXT_CTRL)
		self.parent.parent.parent.Bind(wx.EVT_TOOL,self._on_delete_ctrl,id=ID_DELETE_CTRL)
		
	def _do_layout(self):	
		
		#-------------- Controls ----------------------
		# form editing
		# Combobox to choose form from
		self.cb_FORM_NAME = wx.ComboBox(self,-1,'',size=(160,-1),style=wx.CB_DROPDOWN)
		self.cb_FORM_NAME.SetToolTip(wx.ToolTip('name of form'))
		# page setup
		btn_PAGE_SETUP = wx.Button(self,-1,'page setup')
		
		# control editing
		# control name
		self.cb_CTRL_NAME = wx.ComboBox(self,-1,'',size=(160,-1))
		self.cb_CTRL_NAME.SetToolTip(wx.ToolTip('name of current control'))
		# control type
		self.cb_TYPE = wx.ComboBox(self,-1,'',choices=self.type_list,size=(160,-1))
		self.cb_TYPE.SetToolTip(wx.ToolTip('type of current control'))
		lbl_TYPE = wx.StaticText(self,-1,'type')
		#  control position
		self.tc_POS_X = wx.TextCtrl(self,-1,'',size=(40,-1),style=wx.TE_PROCESS_ENTER)
		self.tc_POS_X.SetToolTip(wx.ToolTip('X-position of current control'))
		self.tc_POS_Y = wx.TextCtrl(self,-1,'',size=(40,-1),style=wx.TE_PROCESS_ENTER)
		self.tc_POS_Y.SetToolTip(wx.ToolTip('Y-position of current control'))
		lbl_POS = wx.StaticText(self,-1,'position')
		# Control Size
		self.tc_WIDTH = wx.TextCtrl(self,-1,'',size=(40,-1),style=wx.TE_PROCESS_ENTER)
		self.tc_WIDTH.SetToolTip(wx.ToolTip('width of current control'))
		self.tc_HEIGHT = wx.TextCtrl(self,-1,'',size=(40,-1),style=wx.TE_PROCESS_ENTER)
		self.tc_HEIGHT.SetToolTip(wx.ToolTip('height of current control'))
		lbl_SIZE_X = wx.StaticText(self,-1,'x')
		lbl_SIZE  = wx.StaticText(self,-1,'size')
		# Control's default value
		self.tc_DEFAULT = wx.TextCtrl(self,-1,'',size=(160,-1),style=wx.TE_PROCESS_ENTER)
		self.tc_DEFAULT.SetToolTip(wx.ToolTip('default value of current control'))
		lbl_DEFAULT = wx.StaticText(self,-1,'default value')
										
		# buttons for dialog navigation
		btn_CANCEL = wx.BitmapButton(self,wx.ID_CANCEL,wx.ArtProvider.GetBitmap(wx.ART_CROSS_MARK,wx.ART_BUTTON, (16,16)))
		btn_CANCEL.SetToolTip(wx.ToolTip('cancel'))
		btn_OK = wx.BitmapButton(self,wx.ID_OK,wx.ArtProvider.GetBitmap(wx.ART_TICK_MARK,wx.ART_BUTTON,(16,16)))
		btn_OK.SetToolTip(wx.ToolTip('save and exit'))
		
		# --------------------- layout sizers --------------------------------------
		# horizontal sizers
		# control position sizer
		possizer = wx.BoxSizer(wx.HORIZONTAL)
		possizer.Add(self.tc_POS_X,0,wx.ALL,0)
		possizer.Add((5,5),0,wx.ALIGN_CENTER|wx.LEFT|wx.RIGHT,5)
		possizer.Add(self.tc_POS_Y,0,wx.ALL,0)
		# control size sizer
		sizesizer = wx.BoxSizer(wx.HORIZONTAL)
		sizesizer.Add(self.tc_WIDTH,0,wx.ALL,0)
		sizesizer.Add(lbl_SIZE_X,0,wx.ALIGN_CENTER|wx.LEFT|wx.RIGHT,5)
		sizesizer.Add(self.tc_HEIGHT,0,wx.ALL,0)
		# button sizer for form editor navigation
		btnsizer = wx.BoxSizer(wx.HORIZONTAL)
		btnsizer.Add(btn_CANCEL,0,wx.LEFT,5)
		btnsizer.Add((16,16),1,wx.EXPAND,0)
		btnsizer.Add(btn_OK,0,wx.ALIGN_RIGHT|wx.RIGHT,5)
		# form box
		formbox = wx.StaticBox(self,-1,'form')
		formbxsizer = wx.StaticBoxSizer(formbox,wx.VERTICAL)
		formbxsizer.Add(self.cb_FORM_NAME,0,wx.ALL,5)
		formbxsizer.Add(btn_PAGE_SETUP,0,wx.ALL,5)
		# ctrl box
		ctrlbox = wx.StaticBox(self,-1,'control')
		ctrlbxsizer = wx.StaticBoxSizer(ctrlbox,wx.VERTICAL)
		ctrlbxsizer.Add(self.cb_CTRL_NAME,0,wx.ALL,5)
		ctrlbxsizer.Add(lbl_TYPE,0,wx.RIGHT|wx.TOP|wx.LEFT,5)
		ctrlbxsizer.Add(self.cb_TYPE,0,wx.RIGHT|wx.BOTTOM|wx.LEFT,5)
		ctrlbxsizer.Add(lbl_POS,0,wx.RIGHT|wx.TOP|wx.LEFT,5)
		ctrlbxsizer.Add(possizer,0,wx.RIGHT|wx.BOTTOM|wx.LEFT,5)
		ctrlbxsizer.Add(lbl_SIZE,0,wx.RIGHT|wx.TOP|wx.LEFT,5)
		ctrlbxsizer.Add(sizesizer,0,wx.RIGHT|wx.BOTTOM|wx.LEFT,5)
		ctrlbxsizer.Add(lbl_DEFAULT,0,wx.RIGHT|wx.TOP|wx.LEFT,5,)
		ctrlbxsizer.Add(self.tc_DEFAULT,0,wx.RIGHT|wx.LEFT,5)
		# Main vertical sizer	
		sizer = wx.BoxSizer(wx.VERTICAL)
		sizer.Add(formbxsizer,0,wx.ALL,5)
		sizer.Add((10,10),0,wx.ALL,0)
		sizer.Add(ctrlbxsizer,0,wx.ALL,5)
		sizer.Add((10,10),1,wx.ALL|wx.EXPAND,0)
		sizer.Add(btnsizer,0,wx.ALL,5)
				
		self.SetSizer(sizer)
				
		# ------------------ events -----------------------------
		
		self.Bind(wx.EVT_COMBOBOX,self._on_form_selected,self.cb_FORM_NAME)
		self.Bind(wx.EVT_TEXT_ENTER,self._on_form_entered,self.cb_FORM_NAME)
		self.Bind(wx.EVT_BUTTON,self._on_page_setup,btn_PAGE_SETUP)
		self.Bind(wx.EVT_COMBOBOX,self._on_ctrl_selected,self.cb_CTRL_NAME)
		self.Bind(wx.EVT_TEXT_ENTER,self._on_ctrl_entered,self.cb_CTRL_NAME)
		self.Bind(wx.EVT_COMBOBOX,self._on_ctrl_changed,self.cb_TYPE)
		self.Bind(wx.EVT_TEXT_ENTER,self._on_type_entered,self.cb_TYPE)
		self.Bind(wx.EVT_TEXT_ENTER,self._on_ctrl_changed,self.tc_POS_X)
		self.Bind(wx.EVT_TEXT_ENTER,self._on_ctrl_changed,self.tc_POS_Y)
		self.Bind(wx.EVT_TEXT_ENTER,self._on_ctrl_changed,self.tc_WIDTH)
		self.Bind(wx.EVT_TEXT_ENTER,self._on_ctrl_changed,self.tc_HEIGHT)
		self.Bind(wx.EVT_TEXT_ENTER,self._on_ctrl_changed,self.tc_DEFAULT)
		self.Bind(wx.EVT_BUTTON,self._on_cancel,id=wx.ID_CANCEL)
		self.Bind(wx.EVT_BUTTON,self._on_ok,id=wx.ID_OK)
	
	# ----------------------------- toolbar functions -------------------------------------------
	def _on_new_form(self,evt):
		"""
		Create a new, empty form.
		"""
		print 'new form'
		
		new_form = 'form_' + str(len(self.formdef.loadFormList()))
		self.parent.form_defs = { 
				'image': '',
				'image_path': '',
				'width': '',
				'height': '',
				'soap': '',
				'name': new_form
				}
		self.parent.ctrl_defs = {
				'ctrl_1': {	
					'type': 'TextCtrl',
					'xpos': '',
					'ypos': '',
					'width': '100',
					'height': '',
					'default': '',
					'style': ''
					}
				}
		#self.parent.current_ctrl = 'ctrl_1'
		# and now let's display this form - it's not saved yet.
		self._change_form()
		
	def _on_restore_form(self,evt):
		print self.parent.ctrl_defs[self.parent.current_ctrl]['xpos']
		self.parent.form_defs,self.parent.ctrl_defs = self.formdef.loadFormDefinition(self.parent.current_form )
		
		print self.parent.ctrl_defs[self.parent.current_ctrl]['xpos']
		self._define_form_variables()
		self._set_values()
		self._change_form()
		
	def _on_save_form(self,evt):
		self.formdef.saveFormDefinition(self.parent.form_defs,self.parent.ctrl_defs)
		
	def _on_delete_form(self,evt):
		pass
	
	def _on_image_path(self,evt):
		
		dlg = wx.FileDialog(self,'choose a background image',wildcard='*.jpeg')
		if dlg.ShowModal() == wx.ID_OK:
			self.parent.form_defs['image_path'] = dlg.GetPath()
			self.parent.form_defs['image'] = self.parent.form_defs['image_path']
			self.parent.form_defs['width'] = str(wx.Image(self.parent.form_defs['image_path'],wx.BITMAP_TYPE_JPEG).ConvertToBitmap().GetWidth() + 20)
			self.parent.form_defs['height'] = str(wx.Image(self.parent.form_defs['image_path'],wx.BITMAP_TYPE_JPEG).ConvertToBitmap().GetHeight() + 20)
		dlg.Destroy()
		self._set_values()
		self.parent.form_editor_display._refresh_form()
		
		
	def _on_scan_image(self,evt):
		"""
		ToDo: put a scan image here to feed self.parent.form_defs[image] and [image_path]
		"""
		wx.MessageBox('sorry, not implemented yet')
		
	def _on_delete_image(self,evt):
		"""
		ToDo: write me
		"""
		wx.MessageBox('sorry, not implemented yet')
	
	def _on_new_ctrl(self,evt,new_ctrl=''):
		if new_ctrl == '':
			new_ctrl = 'control_' + str(len(self.ctrl_list))
		else:
			self.parent.ctrl_defs[new_ctrl]= {
						
						'type': 'TextCtrl',
						'xpos': '0',
						'ypos': '0',
						'width': '100',
						'height': '',
						'default': '',
						'style': ''
						}
					
			# and now let's make this ctrl the current one and display it - it's not saved yet.
			self.parent.current_ctrl = self.cb_FORM_NAME.GetValue()
			self._change_form()
			
	def _on_last_ctrl(self,evt):
		"""
		ToDo: write me
		"""
		wx.MessageBox('sorry, not implemented yet')
	
	def _on_next_ctrl(self,evt):
		"""
		ToDo: write me
		"""
		wx.MessageBox('sorry, not implemented yet')
	
	def _on_delete_ctrl(self,evt):
		"""
		ToDo: Fix me
		"""
		wx.MessageBox('sorry, doesn\'t work yet')
		del self.parent.ctrl_defs[self.parent.current_ctrl]
		# calculate new default ctrl_list and current control to prepare _change_form
		self.parent.ctrl_list = self.parent.ctrl_defs.keys()
		self.parent.ctrl_list.reverse()
		self.parent.current_ctrl = self.parent.ctrl_list[0]
		self._change_form()	
		
	# ----------------------------- property panel functions ------------------------------------
	def _on_form_selected(self,evt):
		"""
		Loads a form definition and changes the form.
		
		In order to do so we set the current form in the fake database and load the form definition, that is stored in self.parent.form_defs and self.parent.ctrl_defs for use in this class, and in the fake backend's form_defs and ctrl_defs variables for use for the form_editor_display.
		"""
		self.parent.current_form = self.cb_FORM_NAME.GetValue()
		self.parent.form_defs,self.parent.ctrl_defs = self.formdef.loadFormDefinition(self.parent.current_form )
		self._change_form()
		
	def _on_form_entered(self,evt):
		"""
		Selects the entered form or renames the current, if the name is new
		
		Does yet NOT store the name change.
		ToDo: implement formdef.renCtrl - or we'll get two Controls next time as wx.Config doesn't know about the change
		"""
		print 'formentered'
		new_name = self.cb_FORM_NAME.GetValue()
		print new_name
		# do we know that name already?
		if new_name in self.formdef.loadFormList():
			self.cb_FORM_NAME.SetValue(new_name)
			self._on_form_selected(evt)
		# obviously not, let's rename the form
		else:
			dlg = cFormRenameDlg(self,'We\'re about to rename the current form\n\nname change will not be saved\n','Information')
			if dlg.ShowModal() == wx.ID_OK:
				self.parent.form_defs['name'] = new_name
				# refresh everything (especially form_list)
				self._change_form()
			else:
				self.cb_FORM_NAME.SetValue(self.parent.form_defs['name'])
			dlg.Destroy()
			
	def _on_page_setup(self,evt):
		"""
		Setup page data of form.
		
		The best way to store the data is probably to store the wx.PageSetupDialogData object directly in the database.
		"""
		data = wx.PageSetupDialogData()
		data.SetPaperId(wx.PAPER_LETTER)
	
		dlg = wx.PageSetupDialog(self, data)

		if dlg.ShowModal() == wx.ID_OK:
			data = dlg.GetPageSetupData()
			# here should be something stored
			# ToDo - how can data.GetPaperSize help us with setting the background image to the correct size?
		dlg.Destroy()
			
	def _on_ctrl_selected(self,evt):
		self.parent.current_ctrl = self.cb_CTRL_NAME.GetValue()
		self._set_values()
	
	def _on_ctrl_entered(self,evt):
		"""
		Selects the entered control or renames the current, if the name is new.
		
		Does yet NOT store the name change.
		ToDo: implement formdef.renCtrl - or we'll get two Controls next time as wx.Config doesn't know about the change
		"""
		new_ctrl_name = self.cb_CTRL_NAME.GetValue()
		# do we know that name already?
		if new_ctrl_name in self.ctrl_list:
			wx.MessageBox('control '+new_ctrl+'already exists')
			self.cb_CTRL_NAME.SetValue(self.parent.current_ctrl)
		# no => rename control - or, to be exact: create a new one with the same settings and delete the old one
		else:
			# create new control definition with the current one as sample
			dlg = cFormRenameDlg(self,'We\'re about to rename the current control\n\nname change will not be saved\n','Information')
			if dlg.ShowModal() == wx.ID_OK:
				self.parent.ctrl_defs[new_ctrl_name] = self.parent.ctrl_defs[self.parent.current_ctrl]
				# delete  current control
				del self.parent.ctrl_defs[self.parent.current_ctrl]
				# pretend the entered control name was selected
				self._on_ctrl_selected(evt)
				
			else:
				self.cb_CTRL_NAME.SetValue(self.parent.current_ctrl)
			dlg.Destroy()
		
	def _on_type_entered(self,evt):
		new_type = self.cb_TYPE.GetValue()
		# do we know that name already?
		if new_type in self.type_list:
			self.parent.ctrl_defs(self.parent.current_ctrl)['type'] = new_type
			self._on_change_form(evt)
		else:
			wx.MessageBox('I don\'t know that type of control\nPlease define it\nor contact the author')
	
	def _on_ctrl_changed(self,evt):
		self.parent.ctrl_defs[self.parent.current_ctrl]['type'] = self.cb_TYPE.GetValue()
		self.parent.ctrl_defs[self.parent.current_ctrl]['xpos'] = self.tc_POS_X.GetValue()
		self.parent.ctrl_defs[self.parent.current_ctrl]['ypos'] = self.tc_POS_Y.GetValue()
		self.parent.ctrl_defs[self.parent.current_ctrl]['width'] = self.tc_WIDTH.GetValue()
		self.parent.ctrl_defs[self.parent.current_ctrl]['height'] = self.tc_HEIGHT.GetValue()
		self.parent.ctrl_defs[self.parent.current_ctrl]['default'] = self.tc_DEFAULT.GetValue()
		# define self.variables new
		self._define_form_variables()
		# pretend current control was selected.
		self._on_ctrl_selected(evt)
		# refresh
		self.parent.form_editor_display._refresh_form()
	
	def _on_cancel(self,evt):
		"""
		ToDo: Fix me
		"""
		wx.MessageBox('sorry, doesn\'t work yet')
		
	def _on_ok(self,evt):
		"""
		ToDo: Fix me
		"""
		wx.MessageBox('sorry, doesn\'t work yet')
		self._on_save_form(evt)
		
		
	# ----------------------------------- other functions -----------------------------------------
	
	def _define_form_variables(self):
		"""
		Calculates some usefull variables.
		
		self.ctrl_list: list of controls in this form
		self.parent.current_control: currently in property_panel displayed control, is set by default to first control of the form
		self.type_list: list of known control types for display in control type combobox
		self.form_list: list of available forms
		"""
		# hack to simulate the storage of an image in the backend
		if self.parent.form_defs['image_path'][-5:] == '.jpeg':
			self.parent.form_defs['image'] =  wx.Image(self.parent.form_defs['image_path'],wx.BITMAP_TYPE_JPEG).ConvertToBitmap()
		# list of controls in this form
		self.ctrl_list = self.parent.ctrl_defs.keys()
		# call reverse() as .keys() returns the keys backwards ?
		self.ctrl_list.reverse()
		# by default in property_panel displayed control
		self.parent.current_ctrl = self.ctrl_list[0]
		# list of known control types for display in control type combobox
		type_dir = dir(gmFormBase)
		self.type_list = []
		for t in type_dir:
			if t[0:4] == 'ctrl':# get type list - all control classes in gmFormCtrls should start with a "c"
				self.type_list.append(t[4:])
		# list of available forms
		self.form_list = self.formdef.loadFormList()

	def _set_values(self):
		"""
		Sets the values from self.parent.form_defs and self.parent.ctrl_defs[self.parent.current_ctrl] in the property controls.
		"""
				
		# set the ComboBox choices
		self.cb_FORM_NAME.Clear()
		for i in range(len(self.form_list)):
			self.cb_FORM_NAME.Append(self.form_list[i])
		self.cb_FORM_NAME.SetValue(self.parent.form_defs['name'])
		# set the ComboBox choices
		self.cb_CTRL_NAME.Clear()
		for i in range (len(self.ctrl_list)):
			self.cb_CTRL_NAME.Append(self.ctrl_list[i])
		self.cb_CTRL_NAME.SetValue(self.parent.current_ctrl)
		self.cb_TYPE.SetValue(self.parent.ctrl_defs[self.parent.current_ctrl]['type'])
		self.tc_POS_X.SetValue(self.parent.ctrl_defs[self.parent.current_ctrl]['xpos'])
		self.tc_POS_Y.SetValue(self.parent.ctrl_defs[self.parent.current_ctrl]['ypos'])
		self.tc_WIDTH.SetValue(self.parent.ctrl_defs[self.parent.current_ctrl]['width'])
		self.tc_HEIGHT.SetValue(self.parent.ctrl_defs[self.parent.current_ctrl]['height'])
		self.tc_DEFAULT.SetValue(self.parent.ctrl_defs[self.parent.current_ctrl]['default'])
		
	def _change_form(self):
		"""
		Sets the current form definition and displays it in property_panel and form_editor_display.
		"""
		# take care of property panel
		self._define_form_variables()
		self._set_values()
		self.parent.form_editor_display._refresh_form()
		
#============================================================================
class cFormRenameDlg(wx.Dialog):
	def __init__(self,parent,message,title):
		wx.Dialog.__init__(self,parent,-1,title=title)
		lbl_MESSAGE = wx.StaticText(self,-1,message)
		btns = self.CreateButtonSizer(wx.OK|wx.CANCEL)
		sizer = wx.BoxSizer(wx.VERTICAL)
		sizer.Add(lbl_MESSAGE,1,wx.ALL|wx.EXPAND,5)
		sizer.Add(btns,0,wx.ALL,5)
		self.SetSizerAndFit(sizer)
		
#============================================================================
class cFormEditorDisplay(gmFormBase.cFormDisplay):
	""" 
	At the moment this class makes no sense, as it does nothing else than it's base class.
	But hopefully one day we'll catch a mouse to do graphic layouting...
	""" 
	def __init__(self, parent):
		gmFormBase.cFormDisplay.__init__(self, parent)
		
#===========================================================================

		

		