#!/usr/bin/env python
# coding: utf-8
# 
# author: Ruthard Baudach, <rthrd@web.de>
# Version 1, 22.06.2007
#----------------------------------------------------
# module collecting controls for use with GNUmed forms
# for the moment just custom controls
# classes named ctrlXyz will be automatically integrated in FormEditors PropertyPanel.
# add an "elif" - clause in cFormDisplay for every new control class
import wx
import wx.lib.scrolledpanel
import wx.html # just because of fancy ctrl definition!
		

#============================================================================
class cFormDisplay(wx.lib.scrolledpanel.ScrolledPanel):

	def __init__(self, parent):
		wx.wx.lib.scrolledpanel.ScrolledPanel.__init__(self, parent, -1, style=wx.SUNKEN_BORDER)
		
		# hand down ancestors
		self.parent = parent
		
		# import fake database connector from parent
		self.formdef = self.parent.formdef
				
		# initiate container for the actual form control instances to avoid an error we paint the form the first time
		self.controls = {}
		
		# paint form
		self._refresh_form()
		self.SetScrollRate(20,20)
		
		self.Bind(wx.EVT_PAINT, self.OnPaint)
		
	def OnPaint(self, event):
		self.dc = wx.PaintDC(self)
		self.PrepareDC(self.dc)
		
		# only draw a background, if we have a jpeg!
		if self.parent.form_defs['image_path'][-5:] == '.jpeg':
			self.parent.form_defs['image'] =  wx.Image(self.parent.form_defs['image_path'],wx.BITMAP_TYPE_JPEG).ConvertToBitmap()	
			# FIXME - how can the image be drawn to a sensible scale independent of it's resolution?
			self.dc.DrawBitmap(self.parent.form_defs['image'],10,10)
		
	def _refresh_form(self):
		#self.form_defs, self.ctrl_defs  = self.formdef.loadFormDefinition(self.parent.current_form )
		# do we have valid form size? - when size is entered not directly but rather via page select or background image, probably not longer necessary
		try:
			width = int(self.parent.form_defs['width'])
		except ValueError:
			width = 400
		try:
			height = int(self.parent.form_defs['height'])
		except ValueError:
			height = 400
		self.SetVirtualSize((width, height))
		self.OnPaint(self)
		
		for i in self.controls.keys():
			self.controls[i].Destroy()
		
		
		
		self.controls = {}
		
		for c in self.parent.ctrl_defs.keys():
			
			# move the dictionary values to variables just for the convenience of adding more controls
			typ = self.parent.ctrl_defs[c]['type']
			default = self.parent.ctrl_defs[c]['default']
			# '' = default - just to avoid to much MessageBoxes to be removed when the backend works
			# do we have valid ctrl pos and size? ToDo - add dialog to ask for valid input. might be lateron moved to Validators of the gmFormEditor's cPropertyPanel, when the backend gives us integers
			if self.parent.ctrl_defs[c]['xpos'] == '':
				xpos = 10
			else:
				try:
					xpos = int(self.parent.ctrl_defs[c]['xpos'])
				except ValueError:
					wx.MessageBox('control '+c+' has invalid x-position\nusing default')
					xpos = -1
			if self.parent.ctrl_defs[c]['ypos'] == '':
				ypos = 10
			else:
				try:
					ypos = int(self.parent.ctrl_defs[c]['ypos'])
				except ValueError:
					wx.MessageBox('control '+c+' has invalid y-position\nusing default')
					ypos = -1
			if self.parent.ctrl_defs[c]['width'] == '':
				width = -1
			else:
				try:
					width = int(self.parent.ctrl_defs[c]['width'])
				except ValueError:
					wx.MessageBox('control '+c+' has invalid width\nusing default')
					width = -1
			if self.parent.ctrl_defs[c]['height'] == '':
				height = -1
			else:
				try:
					
					height = int(self.parent.ctrl_defs[c]['height'])
				except ValueError:
					wx.MessageBox('control '+c+' has invalid height\nusing default')
					height = -1
			
			# building controls according to ctrl_defs
			# to do - how can the style be integrated (a number is exsectes instead of a unicode string - has to be translated somehow  ,style=self.ctrl_defs[c]['style'])
			
			if typ == 'TextCtrl':
				self.controls[c] = ctrlTextCtrl(self,-1,default,pos=(xpos,ypos),size=(width,height))
			elif typ == 'DPC':
				# hack to make wx.DatePickerCtrl happy receiving a wx.DateTime object as default
				default = wx.DateTime.Today()
				self.controls[c] = ctrlDPC(self,-1,default,pos=(xpos,ypos),size=(width,height),style=wx.DP_DROPDOWN) 
			elif typ == 'Label':
				self.controls[c] = ctrlTextCtrl(self,-1,default,pos=(xpos,ypos),size=(width,height))
			elif typ == 'GospelAccordingToMatthew':
				self.controls[c] = ctrlGospelAccordingToMatthew(self,-1,default,pos=(xpos,ypos),size=(width,height))
			else:
				wx.MessageBox('        Sorry,\nType: '+self.ctrl_defs[c]['type']+' of Control "'+c+'"\nis not known to cFormDisplay class','Error parsing the form definition')
	
	
	
	
#===========================================================================
class ctrlTextCtrl(wx.TextCtrl):
	def __init__(self,parent,id,default,pos,size):#,style):
		wx.TextCtrl.__init__(self,parent,id,default,pos,size)#,style)
		
#===========================================================================
class ctrlDPC(wx.DatePickerCtrl):
	def __init__(self,parent,id,default,pos,size,style):#,style):
		wx.DatePickerCtrl.__init__(self,parent,id,default,pos,size,style)
		
#===========================================================================
class ctrlLabel(wx.StaticText):
	def __init__(self,parent,id,default,pos,size):
		wx.StaticText.__init__(self,parent,id,default,pos,size)
		
#===========================================================================
class ctrlGospelAccordingToMatthew(wx.html.HtmlWindow):
	"""
	Just as a demonstration of possibilities...
	
	If a fancy ctrlType needs any imported modules, the have to be added in the import list of this module!
	"""
	def __init__(self,parent,id,default,pos,size):
		wx.html.HtmlWindow.__init__(self,parent,id,pos,size)
		if default == '':
			default = """
			</P><P ALIGN= "CENTER"><A NAME=40N><H1 >Matthäus</H1></P><P></P><P ALIGN= "CENTER"><A NAME=40N_1><H2>Matthäus<BR>Kapitel 1</H2></P><P ALIGN= "LEFT"><A NAME=40N_1_3><B> 1,3 </B>ιουδας δε εγεννησεν τον φαρες και τον ζαρα εκ της θαμαρ φαρες δε εγεννησεν τον εσρωμ εσρωμ δε εγεννησεν τον αραμ
			<A NAME=40N_1_4><B> 1,4 </B>αραμ δε εγεννησεν τον αμιναδαβ αμιναδαβ δε εγεννησεν τον ναασσων ναασσων δε εγεννησεν τον σαλμων
			<A NAME=40N_1_5><B> 1,5 </B>σαλμων δε εγεννησεν τον βοες εκ της ραχαβ βοες δε εγεννησεν τον ιωβηδ εκ της ρουθ ιωβηδ δε εγεννησεν τον ιεσσαι
			<A NAME=40N_1_6><B> 1,6 </B>ιεσσαι δε εγεννησεν τον δαυειδ τον βασιλεα δαυειδ δε εγεννησεν τον σολομωνα εκ της του ουριου
			<A NAME=40N_1_7><B> 1,7 </B>σολομων δε εγεννησεν τον ροβοαμ ροβοαμ δε εγεννησεν τον αβια αβια δε εγεννησεν τον ασαφ
			<A NAME=40N_1_8><B> 1,8 </B>ασαφ δε εγεννησεν τον ιωσαφατ ιωσαφατ δε εγεννησεν τον ιωραμ ιωραμ δε εγεννησεν τον οζειαν
			<A NAME=40N_1_9><B> 1,9 </B>οζειας δε εγεννησεν τον ιωαθαμ ιωαθαμ δε εγεννησεν τον αχαζ αχαζ δε εγεννησεν τον εζεκιαν
			<A NAME=40N_1_10><B> 1,10 </B>εζεκιας δε εγεννησεν τον μανασση μανασσης δε εγεννησεν τον αμως αμως δε εγεννησεν τον ιωσειαν
			<A NAME=40N_1_11><B> 1,11 </B>ιωσειας δε εγεννησεν τον ιεχονιαν και τους αδελφους αυτου επι της μετοικεσιας βαβυλωνος
			<A NAME=40N_1_12><B> 1,12 </B>μετα δε την μετοικεσιαν βαβυλωνος ιεχονιας εγεννησεν τον σαλαθιηλ σαλαθιηλ δε εγεννησεν τον ζοροβαβελ
			<A NAME=40N_1_13><B> 1,13 </B>ζοροβαβελ δε εγεννησεν τον αβιουδ αβιουδ δε εγεννησεν τον ελιακειμ ελιακειμ δε εγεννησεν τον αζωρ
			<A NAME=40N_1_14><B> 1,14 </B>αζωρ δε εγεννησεν τον σαδωκ σαδωκ δε εγεννησεν τον αχειμ αχειμ δε εγεννησεν τον ελιουδ
			<A NAME=40N_1_15><B> 1,15 </B>ελιουδ δε εγεννησεν τον ελεαζαρ ελεαζαρ δε εγεννησεν τον μαθθαν μαθθαν δε εγεννησεν τον ιακωβ
			<A NAME=40N_1_16><B> 1,16 </B>ιακωβ δε εγεννησεν τον ιωσηφ τον ανδρα μαριας εξ ης εγεννηθη ιησους ο λεγομενος χριστος
			<A NAME=40N_1_17><B> 1,17 </B>πασαι ουν αι γενεαι απο αβρααμ εως δαυειδ γενεαι δεκατεσσαρες και απο δαυειδ εως της μετοικεσιας βαβυλωνος γενεαι δεκατεσσαρες και απο της μετοικεσιας βαβυλωνος εως του χριστου γενεαι δεκατεσσαρες
			<A NAME=40N_1_18><B> 1,18 </B>του δε ιησου χριστου η γενεσις ουτως ην μνηστευθεισης της μητρος αυτου μαριας τω ιωσηφ πριν η συνελθειν αυτους ευρεθη εν γαστρι εχουσα εκ πνευματος αγιου
			<A NAME=40N_1_19><B> 1,19 </B>ιωσηφ δε ο ανηρ αυτης δικαιος ων και μη θελων αυτην δειγματισαι εβουληθη λαθρα απολυσαι αυτην
			<A NAME=40N_1_20><B> 1,20 </B>ταυτα δε αυτου ενθυμηθεντος ιδου αγγελος κυριου κατ οναρ εφανη αυτω λεγων ιωσηφ υιος δαυειδ μη φοβηθης παραλαβειν μαριαμ την γυναικα σου το γαρ εν αυτη γεννηθεν εκ πνευματος εστιν αγιου
			<A NAME=40N_1_21><B> 1,21 </B>τεξεται δε υιον και καλεσεις το ονομα αυτου ιησουν αυτος γαρ σωσει τον λαον αυτου απο των αμαρτιων αυτων
			<A NAME=40N_1_22><B> 1,22 </B>τουτο δε ολον γεγονεν ινα πληρωθη το ρηθεν υπο κυριου δια του προφητου λεγοντος
			<A NAME=40N_1_23><B> 1,23 </B>ιδου η παρθενος εν γαστρι εξει και τεξεται υιον και καλεσουσιν το ονομα αυτου εμμανουηλ ο εστιν μεθερμηνευομενον μεθ ημων ο θεος
			<A NAME=40N_1_24><B> 1,24 </B>εγερθεις δε ιωσηφ απο του υπνου εποιησεν ως προσεταξεν αυτω ο αγγελος κυριου και παρελαβεν την γυναικα αυτου
			<A NAME=40N_1_25><B> 1,25 </B>και ουκ εγινωσκεν αυτην εως ου ετεκεν υιον και εκαλεσεν το ονομα αυτου ιησουν
			</P><P ALIGN= "CENTER"><A NAME=40N_2><H2>Matthäus<BR>Kapitel 2</H2></P><P ALIGN= "LEFT"><A NAME=40N_2_2><B> 2,2 </B>λεγοντες που εστιν ο τεχθεις βασιλευς των ιουδαιων ειδομεν γαρ αυτου τον αστερα εν τη ανατολη και ηλθομεν προσκυνησαι αυτω
			<A NAME=40N_2_3><B> 2,3 </B>ακουσας δε ο βασιλευς ηρωδης εταραχθη και πασα ιεροσολυμα μετ αυτου
			<A NAME=40N_2_4><B> 2,4 </B>και συναγαγων παντας τους αρχιερεις και γραμματεις του λαου επυνθανετο παρ αυτων που ο χριστος γενναται
			<A NAME=40N_2_5><B> 2,5 </B>οι δε ειπαν αυτω εν βηθλεεμ της ιουδαιας ουτως γαρ γεγραπται δια του προφητου
			<A NAME=40N_2_6><B> 2,6 </B>και συ βηθλεεμ γη ιουδα ουδαμως ελαχιστη ει εν τοις ηγεμοσιν ιουδα εκ σου γαρ εξελευσεται ηγουμενος οστις ποιμανει τον λαον μου τον ισραηλ
			<A NAME=40N_2_7><B> 2,7 </B>τοτε ηρωδης λαθρα καλεσας τους μαγους ηκριβωσεν παρ αυτων τον χρονον του φαινομενου αστερος
			<A NAME=40N_2_8><B> 2,8 </B>και πεμψας αυτους εις βηθλεεμ ειπεν πορευθεντες εξετασατε ακριβως περι του παιδιου επαν δε ευρητε απαγγειλατε μοι οπως καγω ελθων προσκυνησω αυτω
			<A NAME=40N_2_9><B> 2,9 </B>οι δε ακουσαντες του βασιλεως επορευθησαν και ιδου ο αστηρ ον ειδον εν τη ανατολη προηγεν αυτους εως ελθων εσταθη επανω ου ην το παιδιον
			<A NAME=40N_2_10><B> 2,10 </B>ιδοντες δε τον αστερα εχαρησαν χαραν μεγαλην σφοδρα
			<A NAME=40N_2_11><B> 2,11 </B>και ελθοντες εις την οικιαν ειδον το παιδιον μετα μαριας της μητρος αυτου και πεσοντες προσεκυνησαν αυτω και ανοιξαντες τους θησαυρους αυτων προσηνεγκαν αυτω δωρα χρυσον και λιβανον και σμυρναν
			<A NAME=40N_2_12><B> 2,12 </B>και χρηματισθεντες κατ οναρ μη ανακαμψαι προς ηρωδην δι αλλης οδου ανεχωρησαν εις την χωραν αυτων
			<A NAME=40N_2_13><B> 2,13 </B>αναχωρησαντων δε αυτων ιδου αγγελος κυριου φαινεται κατ οναρ τω ιωσηφ λεγων εγερθεις παραλαβε το παιδιον και την μητερα αυτου και φευγε εις αιγυπτον και ισθι εκει εως αν ειπω σοι μελλει γαρ ηρωδης ζητειν το παιδιον του απολεσαι αυτο
			<A NAME=40N_2_14><B> 2,14 </B>ο δε εγερθεις παρελαβεν το παιδιον και την μητερα αυτου νυκτος και ανεχωρησεν εις αιγυπτον
			<A NAME=40N_2_15><B> 2,15 </B>και ην εκει εως της τελευτης ηρωδου ινα πληρωθη το ρηθεν υπο κυριου δια του προφητου λεγοντος εξ αιγυπτου εκαλεσα τον υιον μου
			<A NAME=40N_2_16><B> 2,16 </B>τοτε ηρωδης ιδων οτι ενεπαιχθη υπο των μαγων εθυμωθη λιαν και αποστειλας ανειλεν παντας τους παιδας τους εν βηθλεεμ και εν πασι τοις οριοις αυτης απο διετους και κατωτερω κατα τον χρονον ον ηκριβωσεν παρα των μαγων
			<A NAME=40N_2_17><B> 2,17 </B>τοτε επληρωθη το ρηθεν δια ιερεμιου του προφητου λεγοντος
			<A NAME=40N_2_18><B> 2,18 </B>φωνη εν ραμα ηκουσθη κλαυθμος και οδυρμος πολυς ραχηλ κλαιουσα τα τεκνα αυτης και ουκ ηθελεν παρακληθηναι οτι ουκ εισιν
			<A NAME=40N_2_19><B> 2,19 </B>τελευτησαντος δε του ηρωδου ιδου αγγελος κυριου φαινεται κατ οναρ τω ιωσηφ εν αιγυπτω
			<A NAME=40N_2_20><B> 2,20 </B>λεγων εγερθεις παραλαβε το παιδιον και την μητερα αυτου και πορευου εις γην ισραηλ τεθνηκασιν γαρ οι ζητουντες την ψυχην του παιδιου
			<A NAME=40N_2_21><B> 2,21 </B>ο δε εγερθεις παρελαβεν το παιδιον και την μητερα αυτου και εισηλθεν εις γην ισραηλ
			<A NAME=40N_2_22><B> 2,22 </B>ακουσας δε οτι αρχελαος βασιλευει της ιουδαιας αντι του πατρος αυτου ηρωδου εφοβηθη εκει απελθειν χρηματισθεις δε κατ οναρ ανεχωρησεν εις τα μερη της γαλιλαιας
			<A NAME=40N_2_23><B> 2,23 </B>και ελθων κατωκησεν εις πολιν λεγομενην ναζαρεθ οπως πληρωθη το ρηθεν δια των προφητων οτι ναζωραιος κληθησεται
			</P><P ALIGN= "CENTER"><A NAME=40N_3><H2>Matthäus<BR>Kapitel 3</H2></P><P ALIGN= "LEFT"><A NAME=40N_3_2><B> 3,2 </B>λεγων μετανοειτε ηγγικεν γαρ η βασιλεια των ουρανων
			<A NAME=40N_3_3><B> 3,3 </B>ουτος γαρ εστιν ο ρηθεις δια ησαιου του προφητου λεγοντος φωνη βοωντος εν τη ερημω ετοιμασατε την οδον κυριου ευθειας ποιειτε τας τριβους αυτου
			<A NAME=40N_3_4><B> 3,4 </B>αυτος δε ο ιωαννης ειχεν το ενδυμα αυτου απο τριχων καμηλου και ζωνην δερματινην περι την οσφυν αυτου η δε τροφη ην αυτου ακριδες και μελι αγριον
			<A NAME=40N_3_5><B> 3,5 </B>τοτε εξεπορευετο προς αυτον ιεροσολυμα και πασα η ιουδαια και πασα η περιχωρος του ιορδανου
			<A NAME=40N_3_6><B> 3,6 </B>και εβαπτιζοντο εν τω ιορδανη ποταμω υπ αυτου εξομολογουμενοι τας αμαρτιας αυτων
			<A NAME=40N_3_7><B> 3,7 </B>ιδων δε πολλους των φαρισαιων και σαδδουκαιων ερχομενους επι το βαπτισμα ειπεν αυτοις γεννηματα εχιδνων τις υπεδειξεν υμιν φυγειν απο της μελλουσης οργης
			<A NAME=40N_3_8><B> 3,8 </B>ποιησατε ουν καρπον αξιον της μετανοιας
			<A NAME=40N_3_9><B> 3,9 </B>και μη δοξητε λεγειν εν εαυτοις πατερα εχομεν τον αβρααμ λεγω γαρ υμιν οτι δυναται ο θεος εκ των λιθων τουτων εγειραι τεκνα τω αβρααμ
			<A NAME=40N_3_10><B> 3,10 </B>ηδη δε η αξινη προς την ριζαν των δενδρων κειται παν ουν δενδρον μη ποιουν καρπον καλον εκκοπτεται και εις πυρ βαλλεται
			<A NAME=40N_3_11><B> 3,11 </B>εγω μεν υμας βαπτιζω εν υδατι εις μετανοιαν ο δε οπισω μου ερχομενος ισχυροτερος μου εστιν ου ουκ ειμι ικανος τα υποδηματα βαστασαι αυτος υμας βαπτισει εν πνευματι αγιω και πυρι
			<A NAME=40N_3_12><B> 3,12 </B>ου το πτυον εν τη χειρι αυτου και διακαθαριει την αλωνα αυτου και συναξει τον σιτον αυτου εις την αποθηκην το δε αχυρον κατακαυσει πυρι ασβεστω
			<A NAME=40N_3_13><B> 3,13 </B>τοτε παραγινεται ο ιησους απο της γαλιλαιας επι τον ιορδανην προς τον ιωαννην του βαπτισθηναι υπ αυτου
			<A NAME=40N_3_14><B> 3,14 </B>ο δε διεκωλυεν αυτον λεγων εγω χρειαν εχω υπο σου βαπτισθηναι και συ ερχη προς με
			<A NAME=40N_3_15><B> 3,15 </B>αποκριθεις δε ο ιησους ειπεν προς αυτον αφες αρτι ουτως γαρ πρεπον εστιν ημιν πληρωσαι πασαν δικαιοσυνην τοτε αφιησιν αυτον
			<A NAME=40N_3_16><B> 3,16 </B>βαπτισθεις δε ο ιησους ευθυς ανεβη απο του υδατος και ιδου ανεωχθησαν οι ουρανοι και ειδεν πνευμα θεου καταβαινον ωσει περιστεραν ερχομενον επ αυτον
			<A NAME=40N_3_17><B> 3,17 </B>και ιδου φωνη εκ των ουρανων λεγουσα ουτος εστιν ο υιος μου ο αγαπητος εν ω ευδοκησα
			</P><P ALIGN= "CENTER"><A NAME=40N_4><H2>Matthäus<BR>Kapitel 4</H2></P><P ALIGN= "LEFT"><A NAME=40N_4_2><B> 4,2 </B>και νηστευσας ημερας τεσσερακοντα και τεσσερακοντα νυκτας υστερον επεινασεν
			<A NAME=40N_4_3><B> 4,3 </B>και προσελθων ο πειραζων ειπεν αυτω ει υιος ει του θεου ειπε ινα οι λιθοι ουτοι αρτοι γενωνται
			<A NAME=40N_4_4><B> 4,4 </B>ο δε αποκριθεις ειπεν γεγραπται ουκ επ αρτω μονω ζησεται ο ανθρωπος αλλ επι παντι ρηματι εκπορευομενω δια στοματος θεου
			<A NAME=40N_4_5><B> 4,5 </B>τοτε παραλαμβανει αυτον ο διαβολος εις την αγιαν πολιν και εστησεν αυτον επι το πτερυγιον του ιερου
			<A NAME=40N_4_6><B> 4,6 </B>και λεγει αυτω ει υιος ει του θεου βαλε σεαυτον κατω γεγραπται γαρ οτι τοις αγγελοις αυτου εντελειται περι σου και επι χειρων αρουσιν σε μηποτε προσκοψης προς λιθον τον ποδα σου
			<A NAME=40N_4_7><B> 4,7 </B>εφη αυτω ο ιησους παλιν γεγραπται ουκ εκπειρασεις κυριον τον θεον σου
			<A NAME=40N_4_8><B> 4,8 </B>παλιν παραλαμβανει αυτον ο διαβολος εις ορος υψηλον λιαν και δεικνυσιν αυτω πασας τας βασιλειας του κοσμου και την δοξαν αυτων
			<A NAME=40N_4_9><B> 4,9 </B>και ειπεν αυτω ταυτα σοι παντα δωσω εαν πεσων προσκυνησης μοι
			<A NAME=40N_4_10><B> 4,10 </B>τοτε λεγει αυτω ο ιησους υπαγε σατανα γεγραπται γαρ κυριον τον θεον σου προσκυνησεις και αυτω μονω λατρευσεις
			<A NAME=40N_4_11><B> 4,11 </B>τοτε αφιησιν αυτον ο διαβολος και ιδου αγγελοι προσηλθον και διηκονουν αυτω
			<A NAME=40N_4_12><B> 4,12 </B>ακουσας δε οτι ιωαννης παρεδοθη ανεχωρησεν εις την γαλιλαιαν
			<A NAME=40N_4_13><B> 4,13 </B>και καταλιπων την ναζαρα ελθων κατωκησεν εις καφαρναουμ την παραθαλασσιαν εν οριοις ζαβουλων και νεφθαλειμ
			<A NAME=40N_4_14><B> 4,14 </B>ινα πληρωθη το ρηθεν δια ησαιου του προφητου λεγοντος
			<A NAME=40N_4_15><B> 4,15 </B>γη ζαβουλων και γη νεφθαλειμ οδον θαλασσης περαν του ιορδανου γαλιλαια των εθνων
			<A NAME=40N_4_16><B> 4,16 </B>ο λαος ο καθημενος εν σκοτει φως ειδεν μεγα και τοις καθημενοις εν χωρα και σκια θανατου φως ανετειλεν αυτοις
			<A NAME=40N_4_17><B> 4,17 </B>απο τοτε ηρξατο ο ιησους κηρυσσειν και λεγειν μετανοειτε ηγγικεν γαρ η βασιλεια των ουρανων
			<A NAME=40N_4_18><B> 4,18 </B>περιπατων δε παρα την θαλασσαν της γαλιλαιας ειδεν δυο αδελφους σιμωνα τον λεγομενον πετρον και ανδρεαν τον αδελφον αυτου βαλλοντας αμφιβληστρον εις την θαλασσαν ησαν γαρ αλεεις
			<A NAME=40N_4_19><B> 4,19 </B>και λεγει αυτοις δευτε οπισω μου και ποιησω υμας αλεεις ανθρωπων
			<A NAME=40N_4_20><B> 4,20 </B>οι δε ευθεως αφεντες τα δικτυα ηκολουθησαν αυτω
			<A NAME=40N_4_21><B> 4,21 </B>και προβας εκειθεν ειδεν αλλους δυο αδελφους ιακωβον τον του ζεβεδαιου και ιωαννην τον αδελφον αυτου εν τω πλοιω μετα ζεβεδαιου του πατρος αυτων καταρτιζοντας τα δικτυα αυτων και εκαλεσεν αυτους
			<A NAME=40N_4_22><B> 4,22 </B>οι δε ευθεως αφεντες το πλοιον και τον πατερα αυτων ηκολουθησαν αυτω
			<A NAME=40N_4_23><B> 4,23 </B>και περιηγεν εν ολη τη γαλιλαια διδασκων εν ταις συναγωγαις αυτων και κηρυσσων το ευαγγελιον της βασιλειας και θεραπευων πασαν νοσον και πασαν μαλακιαν εν τω λαω
			<A NAME=40N_4_24><B> 4,24 </B>και απηλθεν η ακοη αυτου εις ολην την συριαν και προσηνεγκαν αυτω παντας τους κακως εχοντας ποικιλαις νοσοις και βασανοις συνεχομενους και δαιμονιζομενους και σεληνιαζομενους και παραλυτικους και εθεραπευσεν αυτους
			<A NAME=40N_4_25><B> 4,25 </B>και ηκολουθησαν αυτω οχλοι πολλοι απο της γαλιλαιας και δεκαπολεως και ιεροσολυμων και ιουδαιας και περαν του ιορδανου
			</P><P ALIGN= "CENTER"><A NAME=40N_5><H2>Matthäus<BR>Kapitel 5</H2></P><P ALIGN= "LEFT"><A NAME=40N_5_2><B> 5,2 </B>και ανοιξας το στομα αυτου εδιδασκεν αυτους λεγων
			<A NAME=40N_5_3><B> 5,3 </B>μακαριοι οι πτωχοι τω πνευματι οτι αυτων εστιν η βασιλεια των ουρανων
			<A NAME=40N_5_4><B> 5,4 </B>μακαριοι οι πραεις οτι αυτοι κληρονομησουσιν την γην
			<A NAME=40N_5_5><B> 5,5 </B>μακαριοι οι πενθουντες οτι αυτοι παρακληθησονται
			<A NAME=40N_5_6><B> 5,6 </B>μακαριοι οι πεινωντες και διψωντες την δικαιοσυνην οτι αυτοι χορτασθησονται
			<A NAME=40N_5_7><B> 5,7 </B>μακαριοι οι ελεημονες οτι αυτοι ελεηθησονται
			<A NAME=40N_5_8><B> 5,8 </B>μακαριοι οι καθαροι τη καρδια οτι αυτοι τον θεον οψονται
			<A NAME=40N_5_9><B> 5,9 </B>μακαριοι οι ειρηνοποιοι οτι αυτοι υιοι θεου κληθησονται
			<A NAME=40N_5_10><B> 5,10 </B>μακαριοι οι δεδιωγμενοι ενεκεν δικαιοσυνης οτι αυτων εστιν η βασιλεια των ουρανων
			<A NAME=40N_5_11><B> 5,11 </B>μακαριοι εστε οταν ονειδισωσιν υμας και διωξωσιν και ειπωσιν παν πονηρον καθ υμων ψευδομενοι ενεκεν εμου
			<A NAME=40N_5_12><B> 5,12 </B>χαιρετε και αγαλλιασθε οτι ο μισθος υμων πολυς εν τοις ουρανοις ουτως γαρ εδιωξαν τους προφητας τους προ υμων
			<A NAME=40N_5_13><B> 5,13 </B>υμεις εστε το αλας της γης εαν δε το αλας μωρανθη εν τινι αλισθησεται εις ουδεν ισχυει ετι ει μη βληθεν εξω καταπατεισθαι υπο των ανθρωπων
			<A NAME=40N_5_14><B> 5,14 </B>υμεις εστε το φως του κοσμου ου δυναται πολις κρυβηναι επανω ορους κειμενη
			<A NAME=40N_5_15><B> 5,15 </B>ουδε καιουσιν λυχνον και τιθεασιν αυτον υπο τον μοδιον αλλ επι την λυχνιαν και λαμπει πασιν τοις εν τη οικια
			<A NAME=40N_5_16><B> 5,16 </B>ουτως λαμψατω το φως υμων εμπροσθεν των ανθρωπων οπως ιδωσιν υμων τα καλα εργα και δοξασωσιν τον πατερα υμων τον εν τοις ουρανοις
			<A NAME=40N_5_17><B> 5,17 </B>μη νομισητε οτι ηλθον καταλυσαι τον νομον η τους προφητας ουκ ηλθον καταλυσαι αλλα πληρωσαι
			<A NAME=40N_5_18><B> 5,18 </B>αμην γαρ λεγω υμιν εως αν παρελθη ο ουρανος και η γη ιωτα εν η μια κεραια ου μη παρελθη απο του νομου εως αν παντα γενηται
			<A NAME=40N_5_19><B> 5,19 </B>ος εαν ουν λυση μιαν των εντολων τουτων των ελαχιστων και διδαξη ουτως τους ανθρωπους ελαχιστος κληθησεται εν τη βασιλεια των ουρανων ος δ αν ποιηση και διδαξη ουτος μεγας κληθησεται εν τη βασιλεια των ουρανων
			<A NAME=40N_5_20><B> 5,20 </B>λεγω γαρ υμιν οτι εαν μη περισσευση υμων η δικαιοσυνη πλειον των γραμματεων και φαρισαιων ου μη εισελθητε εις την βασιλειαν των ουρανων
			<A NAME=40N_5_21><B> 5,21 </B>ηκουσατε οτι ερρεθη τοις αρχαιοις ου φονευσεις ος δ αν φονευση ενοχος εσται τη κρισει
			<A NAME=40N_5_22><B> 5,22 </B>εγω δε λεγω υμιν οτι πας ο οργιζομενος τω αδελφω αυτου ενοχος εσται τη κρισει ος δ αν ειπη τω αδελφω αυτου ρακα ενοχος εσται τω συνεδριω ος δ αν ειπη μωρε ενοχος εσται εις την γεενναν του πυρος
			<A NAME=40N_5_23><B> 5,23 </B>εαν ουν προσφερης το δωρον σου επι το θυσιαστηριον κακει μνησθης οτι ο αδελφος σου εχει τι κατα σου
			<A NAME=40N_5_24><B> 5,24 </B>αφες εκει το δωρον σου εμπροσθεν του θυσιαστηριου και υπαγε πρωτον διαλλαγηθι τω αδελφω σου και τοτε ελθων προσφερε το δωρον σου
			<A NAME=40N_5_25><B> 5,25 </B>ισθι ευνοων τω αντιδικω σου ταχυ εως οτου ει μετ αυτου εν τη οδω μηποτε σε παραδω ο αντιδικος τω κριτη και ο κριτης τω υπηρετη και εις φυλακην βληθηση
			<A NAME=40N_5_26><B> 5,26 </B>αμην λεγω σοι ου μη εξελθης εκειθεν εως αν αποδως τον εσχατον κοδραντην
			<A NAME=40N_5_27><B> 5,27 </B>ηκουσατε οτι ερρεθη ου μοιχευσεις
			<A NAME=40N_5_28><B> 5,28 </B>εγω δε λεγω υμιν οτι πας ο βλεπων γυναικα προς το επιθυμησαι ηδη εμοιχευσεν αυτην εν τη καρδια αυτου
			<A NAME=40N_5_29><B> 5,29 </B>ει δε ο οφθαλμος σου ο δεξιος σκανδαλιζει σε εξελε αυτον και βαλε απο σου συμφερει γαρ σοι ινα αποληται εν των μελων σου και μη ολον το σωμα σου βληθη εις γεενναν
			<A NAME=40N_5_30><B> 5,30 </B>και ει η δεξια σου χειρ σκανδαλιζει σε εκκοψον αυτην και βαλε απο σου συμφερει γαρ σοι ινα αποληται εν των μελων σου και μη ολον το σωμα σου εις γεενναν απελθη
			<A NAME=40N_5_31><B> 5,31 </B>ερρεθη δε ος αν απολυση την γυναικα αυτου δοτω αυτη αποστασιον
			<A NAME=40N_5_32><B> 5,32 </B>εγω δε λεγω υμιν οτι πας ο απολυων την γυναικα αυτου παρεκτος λογου πορνειας ποιει αυτην μοιχευθηναι και ος εαν απολελυμενην γαμηση μοιχαται
			<A NAME=40N_5_33><B> 5,33 </B>παλιν ηκουσατε οτι ερρεθη τοις αρχαιοις ουκ επιορκησεις αποδωσεις δε τω κυριω τους ορκους σου
			<A NAME=40N_5_34><B> 5,34 </B>εγω δε λεγω υμιν μη ομοσαι ολως μητε εν τω ουρανω οτι θρονος εστιν του θεου
			<A NAME=40N_5_35><B> 5,35 </B>μητε εν τη γη οτι υποποδιον εστιν των ποδων αυτου μητε εις ιεροσολυμα οτι πολις εστιν του μεγαλου βασιλεως
			<A NAME=40N_5_36><B> 5,36 </B>μητε εν τη κεφαλη σου ομοσης οτι ου δυνασαι μιαν τριχα λευκην ποιησαι η μελαιναν
			<A NAME=40N_5_37><B> 5,37 </B>εστω δε ο λογος υμων ναι ναι ου ου το δε περισσον τουτων εκ του πονηρου εστιν
			<A NAME=40N_5_38><B> 5,38 </B>ηκουσατε οτι ερρεθη οφθαλμον αντι οφθαλμου και οδοντα αντι οδοντος
			<A NAME=40N_5_39><B> 5,39 </B>εγω δε λεγω υμιν μη αντιστηναι τω πονηρω αλλ οστις σε ραπιζει εις την δεξιαν σιαγονα στρεψον αυτω και την αλλην
			<A NAME=40N_5_40><B> 5,40 </B>και τω θελοντι σοι κριθηναι και τον χιτωνα σου λαβειν αφες αυτω και το ιματιον
			<A NAME=40N_5_41><B> 5,41 </B>και οστις σε αγγαρευσει μιλιον εν υπαγε μετ αυτου δυο
			<A NAME=40N_5_42><B> 5,42 </B>τω αιτουντι σε δος και τον θελοντα απο σου δανισασθαι μη αποστραφης
			<A NAME=40N_5_43><B> 5,43 </B>ηκουσατε οτι ερρεθη αγαπησεις τον πλησιον σου και μισησεις τον εχθρον σου
			<A NAME=40N_5_44><B> 5,44 </B>εγω δε λεγω υμιν αγαπατε τους εχθρους υμων και προσευχεσθε υπερ των διωκοντων υμας
			<A NAME=40N_5_45><B> 5,45 </B>οπως γενησθε υιοι του πατρος υμων του εν ουρανοις οτι τον ηλιον αυτου ανατελλει επι πονηρους και αγαθους και βρεχει επι δικαιους και αδικους
			<A NAME=40N_5_46><B> 5,46 </B>εαν γαρ αγαπησητε τους αγαπωντας υμας τινα μισθον εχετε ουχι και οι τελωναι το αυτο ποιουσιν
			<A NAME=40N_5_47><B> 5,47 </B>και εαν ασπασησθε τους αδελφους υμων μονον τι περισσον ποιειτε ουχι και οι εθνικοι το αυτο ποιουσιν
			<A NAME=40N_5_48><B> 5,48 </B>εσεσθε ουν υμεις τελειοι ως ο πατηρ υμων ο ουρανιος τελειος εστιν
			</P><P ALIGN= "CENTER"><A NAME=40N_6><H2>Matthäus<BR>Kapitel 6</H2></P><P ALIGN= "LEFT"><A NAME=40N_6_2><B> 6,2 </B>οταν ουν ποιης ελεημοσυνην μη σαλπισης εμπροσθεν σου ωσπερ οι υποκριται ποιουσιν εν ταις συναγωγαις και εν ταις ρυμαις οπως δοξασθωσιν υπο των ανθρωπων αμην λεγω υμιν απεχουσιν τον μισθον αυτων
			<A NAME=40N_6_3><B> 6,3 </B>σου δε ποιουντος ελεημοσυνην μη γνωτω η αριστερα σου τι ποιει η δεξια σου
			<A NAME=40N_6_4><B> 6,4 </B>οπως η σου ελεημοσυνη η εν τω κρυπτω και ο πατηρ σου ο βλεπων εν τω κρυπτω αποδωσει σοι
			<A NAME=40N_6_5><B> 6,5 </B>και οταν προσευχησθε ουκ εσεσθε ως οι υποκριται οτι φιλουσιν εν ταις συναγωγαις και εν ταις γωνιαις των πλατειων εστωτες προσευχεσθαι οπως φανωσιν τοις ανθρωποις αμην λεγω υμιν απεχουσιν τον μισθον αυτων
			<A NAME=40N_6_6><B> 6,6 </B>συ δε οταν προσευχη εισελθε εις το ταμειον σου και κλεισας την θυραν σου προσευξαι τω πατρι σου τω εν τω κρυπτω και ο πατηρ σου ο βλεπων εν τω κρυπτω αποδωσει σοι
			<A NAME=40N_6_7><B> 6,7 </B>προσευχομενοι δε μη βατταλογησητε ωσπερ οι εθνικοι δοκουσιν γαρ οτι εν τη πολυλογια αυτων εισακουσθησονται
			<A NAME=40N_6_8><B> 6,8 </B>μη ουν ομοιωθητε αυτοις οιδεν γαρ ο πατηρ υμων ων χρειαν εχετε προ του υμας αιτησαι αυτον
			<A NAME=40N_6_9><B> 6,9 </B>ουτως ουν προσευχεσθε υμεις πατερ ημων ο εν τοις ουρανοις αγιασθητω το ονομα σου
			<A NAME=40N_6_10><B> 6,10 </B>ελθατω η βασιλεια σου γενηθητω το θελημα σου ως εν ουρανω και επι γης
			<A NAME=40N_6_11><B> 6,11 </B>τον αρτον ημων τον επιουσιον δος ημιν σημερον
			<A NAME=40N_6_12><B> 6,12 </B>και αφες ημιν τα οφειληματα ημων ως και ημεις αφηκαμεν τοις οφειλεταις ημων
			<A NAME=40N_6_13><B> 6,13 </B>και μη εισενεγκης ημας εις πειρασμον αλλα ρυσαι ημας απο του πονηρου
			<A NAME=40N_6_14><B> 6,14 </B>εαν γαρ αφητε τοις ανθρωποις τα παραπτωματα αυτων αφησει και υμιν ο πατηρ υμων ο ουρανιος
			<A NAME=40N_6_15><B> 6,15 </B>εαν δε μη αφητε τοις ανθρωποις ουδε ο πατηρ υμων αφησει τα παραπτωματα υμων
			<A NAME=40N_6_16><B> 6,16 </B>οταν δε νηστευητε μη γινεσθε ως οι υποκριται σκυθρωποι αφανιζουσιν γαρ τα προσωπα αυτων οπως φανωσιν τοις ανθρωποις νηστευοντες αμην λεγω υμιν απεχουσιν τον μισθον αυτων
			<A NAME=40N_6_17><B> 6,17 </B>συ δε νηστευων αλειψαι σου την κεφαλην και το προσωπον σου νιψαι
			<A NAME=40N_6_18><B> 6,18 </B>οπως μη φανης τοις ανθρωποις νηστευων αλλα τω πατρι σου τω εν τω κρυφαιω και ο πατηρ σου ο βλεπων εν τω κρυφαιω αποδωσει σοι
			<A NAME=40N_6_19><B> 6,19 </B>μη θησαυριζετε υμιν θησαυρους επι της γης οπου σης και βρωσις αφανιζει και οπου κλεπται διορυσσουσιν και κλεπτουσιν
			<A NAME=40N_6_20><B> 6,20 </B>θησαυριζετε δε υμιν θησαυρους εν ουρανω οπου ουτε σης ουτε βρωσις αφανιζει και οπου κλεπται ου διορυσσουσιν ουδε κλεπτουσιν
			<A NAME=40N_6_21><B> 6,21 </B>οπου γαρ εστιν ο θησαυρος σου εκει εσται και η καρδια σου
			<A NAME=40N_6_22><B> 6,22 </B>ο λυχνος του σωματος εστιν ο οφθαλμος εαν η ο οφθαλμος σου απλους ολον το σωμα σου φωτεινον εσται
			<A NAME=40N_6_23><B> 6,23 </B>εαν δε ο οφθαλμος σου πονηρος η ολον το σωμα σου σκοτεινον εσται ει ουν το φως το εν σοι σκοτος εστιν το σκοτος ποσον
			<A NAME=40N_6_24><B> 6,24 </B>ουδεις δυναται δυσι κυριοις δουλευειν η γαρ τον ενα μισησει και τον ετερον αγαπησει η ενος ανθεξεται και του ετερου καταφρονησει ου δυνασθε θεω δουλευειν και μαμωνα
			<A NAME=40N_6_25><B> 6,25 </B>δια τουτο λεγω υμιν μη μεριμνατε τη ψυχη υμων τι φαγητε μηδε τω σωματι υμων τι ενδυσησθε ουχι η ψυχη πλειον εστιν της τροφης και το σωμα του ενδυματος
			<A NAME=40N_6_26><B> 6,26 </B>εμβλεψατε εις τα πετεινα του ουρανου οτι ου σπειρουσιν ουδε θεριζουσιν ουδε συναγουσιν εις αποθηκας και ο πατηρ υμων ο ουρανιος τρεφει αυτα ουχ υμεις μαλλον διαφερετε αυτων
			<A NAME=40N_6_27><B> 6,27 </B>τις δε εξ υμων μεριμνων δυναται προσθειναι επι την ηλικιαν αυτου πηχυν ενα
			<A NAME=40N_6_28><B> 6,28 </B>και περι ενδυματος τι μεριμνατε καταμαθετε τα κρινα του αγρου πως αυξανουσιν ου κοπιωσιν ουδε νηθουσιν
			<A NAME=40N_6_29><B> 6,29 </B>λεγω δε υμιν οτι ουδε σολομων εν παση τη δοξη αυτου περιεβαλετο ως εν τουτων
			<A NAME=40N_6_30><B> 6,30 </B>ει δε τον χορτον του αγρου σημερον οντα και αυριον εις κλιβανον βαλλομενον ο θεος ουτως αμφιεννυσιν ου πολλω μαλλον υμας ολιγοπιστοι
			<A NAME=40N_6_31><B> 6,31 </B>μη ουν μεριμνησητε λεγοντες τι φαγωμεν η τι πιωμεν η τι περιβαλωμεθα
			<A NAME=40N_6_32><B> 6,32 </B>παντα γαρ ταυτα τα εθνη επιζητουσιν οιδεν γαρ ο πατηρ υμων ο ουρανιος οτι χρηζετε τουτων απαντων
			<A NAME=40N_6_33><B> 6,33 </B>ζητειτε δε πρωτον την βασιλειαν και την δικαιοσυνην αυτου και ταυτα παντα προστεθησεται υμιν
			<A NAME=40N_6_34><B> 6,34 </B>μη ουν μεριμνησητε εις την αυριον η γαρ αυριον μεριμνησει εαυτης αρκετον τη ημερα η κακια αυτης
			</P><P ALIGN= "CENTER"><A NAME=40N_7><H2>Matthäus<BR>Kapitel 7</H2></P><P ALIGN= "LEFT"><A NAME=40N_7_2><B> 7,2 </B>εν ω γαρ κριματι κρινετε κριθησεσθε και εν ω μετρω μετρειτε μετρηθησεται υμιν
			<A NAME=40N_7_3><B> 7,3 </B>τι δε βλεπεις το καρφος το εν τω οφθαλμω του αδελφου σου την δε εν τω σω οφθαλμω δοκον ου κατανοεις
			<A NAME=40N_7_4><B> 7,4 </B>η πως ερεις τω αδελφω σου αφες εκβαλω το καρφος εκ του οφθαλμου σου και ιδου η δοκος εν τω οφθαλμω σου
			<A NAME=40N_7_5><B> 7,5 </B>υποκριτα εκβαλε πρωτον εκ του οφθαλμου σου την δοκον και τοτε διαβλεψεις εκβαλειν το καρφος εκ του οφθαλμου του αδελφου σου
			<A NAME=40N_7_6><B> 7,6 </B>μη δωτε το αγιον τοις κυσιν μηδε βαλητε τους μαργαριτας υμων εμπροσθεν των χοιρων μηποτε καταπατησουσιν αυτους εν τοις ποσιν αυτων και στραφεντες ρηξωσιν υμας
			<A NAME=40N_7_7><B> 7,7 </B>αιτειτε και δοθησεται υμιν ζητειτε και ευρησετε κρουετε και ανοιγησεται υμιν
			<A NAME=40N_7_8><B> 7,8 </B>πας γαρ ο αιτων λαμβανει και ο ζητων ευρισκει και τω κρουοντι ανοιγησεται
			<A NAME=40N_7_9><B> 7,9 </B>η τις εστιν εξ υμων ανθρωπος ον αιτησει ο υιος αυτου αρτον μη λιθον επιδωσει αυτω
			<A NAME=40N_7_10><B> 7,10 </B>η και ιχθυν αιτησει μη οφιν επιδωσει αυτω
			<A NAME=40N_7_11><B> 7,11 </B>ει ουν υμεις πονηροι οντες οιδατε δοματα αγαθα διδοναι τοις τεκνοις υμων ποσω μαλλον ο πατηρ υμων ο εν τοις ουρανοις δωσει αγαθα τοις αιτουσιν αυτον
			<A NAME=40N_7_12><B> 7,12 </B>παντα ουν οσα εαν θελητε ινα ποιωσιν υμιν οι ανθρωποι ουτως και υμεις ποιειτε αυτοις ουτος γαρ εστιν ο νομος και οι προφηται
			<A NAME=40N_7_13><B> 7,13 </B>εισελθατε δια της στενης πυλης οτι πλατεια [η πυλη] και ευρυχωρος η οδος η απαγουσα εις την απωλειαν και πολλοι εισιν οι εισερχομενοι δι αυτης
			<A NAME=40N_7_14><B> 7,14 </B>οτι στενη [η πυλη] και τεθλιμμενη η οδος η απαγουσα εις την ζωην και ολιγοι εισιν οι ευρισκοντες αυτην
			<A NAME=40N_7_15><B> 7,15 </B>προσεχετε απο των ψευδοπροφητων οιτινες ερχονται προς υμας εν ενδυμασιν προβατων εσωθεν δε εισιν λυκοι αρπαγες
			<A NAME=40N_7_16><B> 7,16 </B>απο των καρπων αυτων επιγνωσεσθε αυτους μητι συλλεγουσιν απο ακανθων σταφυλας η απο τριβολων συκα
			<A NAME=40N_7_17><B> 7,17 </B>ουτως παν δενδρον αγαθον καρπους καλους ποιει το δε σαπρον δενδρον καρπους πονηρους ποιει
			<A NAME=40N_7_18><B> 7,18 </B>ου δυναται δενδρον αγαθον καρπους πονηρους ενεγκειν ουδε δενδρον σαπρον καρπους καλους ενεγκειν
			<A NAME=40N_7_19><B> 7,19 </B>παν δενδρον μη ποιουν καρπον καλον εκκοπτεται και εις πυρ βαλλεται
			<A NAME=40N_7_20><B> 7,20 </B>αραγε απο των καρπων αυτων επιγνωσεσθε αυτους
			<A NAME=40N_7_21><B> 7,21 </B>ου πας ο λεγων μοι κυριε κυριε εισελευσεται εις την βασιλειαν των ουρανων αλλ ο ποιων το θελημα του πατρος μου του εν τοις ουρανοις
			<A NAME=40N_7_22><B> 7,22 </B>πολλοι ερουσιν μοι εν εκεινη τη ημερα κυριε κυριε ου τω σω ονοματι επροφητευσαμεν και τω σω ονοματι δαιμονια εξεβαλομεν και τω σω ονοματι δυναμεις πολλας εποιησαμεν
			<A NAME=40N_7_23><B> 7,23 </B>και τοτε ομολογησω αυτοις οτι ουδεποτε εγνων υμας αποχωρειτε απ εμου οι εργαζομενοι την ανομιαν
			<A NAME=40N_7_24><B> 7,24 </B>πας ουν οστις ακουει μου τους λογους τουτους και ποιει αυτους ομοιωθησεται ανδρι φρονιμω οστις ωκοδομησεν αυτου την οικιαν επι την πετραν
			<A NAME=40N_7_25><B> 7,25 </B>και κατεβη η βροχη και ηλθον οι ποταμοι και επνευσαν οι ανεμοι και προσεπεσαν τη οικια εκεινη και ουκ επεσεν τεθεμελιωτο γαρ επι την πετραν
			<A NAME=40N_7_26><B> 7,26 </B>και πας ο ακουων μου τους λογους τουτους και μη ποιων αυτους ομοιωθησεται ανδρι μωρω οστις ωκοδομησεν αυτου την οικιαν επι την αμμον
			<A NAME=40N_7_27><B> 7,27 </B>και κατεβη η βροχη και ηλθον οι ποταμοι και επνευσαν οι ανεμοι και προσεκοψαν τη οικια εκεινη και επεσεν και ην η πτωσις αυτης μεγαλη
			<A NAME=40N_7_28><B> 7,28 </B>και εγενετο οτε ετελεσεν ο ιησους τους λογους τουτους εξεπλησσοντο οι οχλοι επι τη διδαχη αυτου
			<A NAME=40N_7_29><B> 7,29 </B>ην γαρ διδασκων αυτους ως εξουσιαν εχων και ουχ ως οι γραμματεις αυτων
			</P><P ALIGN= "CENTER"><A NAME=40N_8><H2>Matthäus<BR>Kapitel 8</H2></P><P ALIGN= "LEFT"><A NAME=40N_8_2><B> 8,2 </B>και ιδου λεπρος προσελθων προσεκυνει αυτω λεγων κυριε εαν θελης δυνασαι με καθαρισαι
			<A NAME=40N_8_3><B> 8,3 </B>και εκτεινας την χειρα ηψατο αυτου λεγων θελω καθαρισθητι και ευθεως εκαθερισθη αυτου η λεπρα
			<A NAME=40N_8_4><B> 8,4 </B>και λεγει αυτω ο ιησους ορα μηδενι ειπης αλλα υπαγε σεαυτον δειξον τω ιερει και προσενεγκον το δωρον ο προσεταξεν μωυσης εις μαρτυριον αυτοις
			<A NAME=40N_8_5><B> 8,5 </B>εισελθοντος δε αυτου εις καφαρναουμ προσηλθεν αυτω εκατονταρχης παρακαλων αυτον
			<A NAME=40N_8_6><B> 8,6 </B>και λεγων κυριε ο παις μου βεβληται εν τη οικια παραλυτικος δεινως βασανιζομενος
			<A NAME=40N_8_7><B> 8,7 </B>λεγει αυτω εγω ελθων θεραπευσω αυτον
			<A NAME=40N_8_8><B> 8,8 </B>αποκριθεις δε ο εκατονταρχης εφη κυριε ουκ ειμι ικανος ινα μου υπο την στεγην εισελθης αλλα μονον ειπε λογω και ιαθησεται ο παις μου
			<A NAME=40N_8_9><B> 8,9 </B>και γαρ εγω ανθρωπος ειμι υπο εξουσιαν εχων υπ εμαυτον στρατιωτας και λεγω τουτω πορευθητι και πορευεται και αλλω ερχου και ερχεται και τω δουλω μου ποιησον τουτο και ποιει
			<A NAME=40N_8_10><B> 8,10 </B>ακουσας δε ο ιησους εθαυμασεν και ειπεν τοις ακολουθουσιν αμην λεγω υμιν ουδε εν τω ισραηλ τοσαυτηνν πιστιν ευρον
			<A NAME=40N_8_11><B> 8,11 </B>λεγω δε υμιν οτι πολλοι απο ανατολων και δυσμων ηξουσιν και ανακλιθησονται μετα αβρααμ και ισαακ και ιακωβ εν τη βασιλεια των ουρανων
			<A NAME=40N_8_12><B> 8,12 </B>οι δε υιοι της βασιλειας εξελευσονται εις το σκοτος το εξωτερον εκει εσται ο κλαυθμος και ο βρυγμος των οδοντων
			<A NAME=40N_8_13><B> 8,13 </B>και ειπεν ο ιησους τω εκατονταρχη υπαγε ως επιστευσας γενηθητω σοι και ιαθη ο παις εν τη ωρα εκεινη
			<A NAME=40N_8_14><B> 8,14 </B>και ελθων ο ιησους εις την οικιαν πετρου ειδεν την πενθεραν αυτου βεβλημενην και πυρεσσουσαν
			<A NAME=40N_8_15><B> 8,15 </B>και ηψατο της χειρος αυτης και αφηκεν αυτην ο πυρετος και ηγερθη και διηκονει αυτω
			<A NAME=40N_8_16><B> 8,16 </B>οψιας δε γενομενης προσηνεγκαν αυτω δαιμονιζομενους πολλους και εξεβαλεν τα πνευματα λογω και παντας τους κακως εχοντας εθεραπευσεν
			<A NAME=40N_8_17><B> 8,17 </B>οπως πληρωθη το ρηθεν δια ησαιου του προφητου λεγοντος αυτος τας ασθενειας ημων ελαβεν και τας νοσους εβαστασεν
			<A NAME=40N_8_18><B> 8,18 </B>ιδων δε ο ιησους πολλους οχλους περι αυτον εκελευσεν απελθειν εις το περαν
			<A NAME=40N_8_19><B> 8,19 </B>και προσελθων εις γραμματευς ειπεν αυτω διδασκαλε ακολουθησω σοι οπου εαν απερχη
			<A NAME=40N_8_20><B> 8,20 </B>και λεγει αυτω ο ιησους αι αλωπεκες φωλεους εχουσιν και τα πετεινα του ουρανου κατασκηνωσεις ο δε υιος του ανθρωπου ουκ εχει που την κεφαλην κλινη
			<A NAME=40N_8_21><B> 8,21 </B>ετερος δε των μαθητων ειπεν αυτω κυριε επιτρεψον μοι πρωτον απελθειν και θαψαι τον πατερα μου
			<A NAME=40N_8_22><B> 8,22 </B>ο δε λεγει αυτω ακολουθει μοι και αφες τους νεκρους θαψαι τους εαυτων νεκρους
			<A NAME=40N_8_23><B> 8,23 </B>και εμβαντι αυτω εις το πλοιον ηκολουθησαν αυτω οι μαθηται αυτου
			<A NAME=40N_8_24><B> 8,24 </B>και ιδου σεισμος μεγας εγενετο εν τη θαλασση ωστε το πλοιον καλυπτεσθαι υπο των κυματων αυτος δε εκαθευδεν
			<A NAME=40N_8_25><B> 8,25 </B>και προσελθοντες ηγειραν αυτον λεγοντες κυριε σωσον απολλυμεθα
			<A NAME=40N_8_26><B> 8,26 </B>και λεγει αυτοις τι δειλοι εστε ολιγοπιστοι τοτε εγερθεις επετιμησεν τοις ανεμοις και τη θαλασση και εγενετο γαληνη μεγαλη
			<A NAME=40N_8_27><B> 8,27 </B>οι δε ανθρωποι εθαυμασαν λεγοντες ποταπος εστιν ουτος οτι και οι ανεμοι και η θαλασσα αυτω υπακουουσιν
			<A NAME=40N_8_28><B> 8,28 </B>και ελθοντος αυτου εις το περαν εις την χωραν των γαδαρηνων υπηντησαν αυτω δυο δαιμονιζομενοι εκ των μνημειων εξερχομενοι χαλεποι λιαν ωστε μη ισχυειν τινα παρελθειν δια της οδου εκεινης
			<A NAME=40N_8_29><B> 8,29 </B>και ιδου εκραξαν λεγοντες τι ημιν και σοι υιε του θεου ηλθες ωδε προ καιρου βασανισαι ημας
			<A NAME=40N_8_30><B> 8,30 </B>ην δε μακραν απ αυτων αγελη χοιρων πολλων βοσκομενη
			<A NAME=40N_8_31><B> 8,31 </B>οι δε δαιμονες παρεκαλουν αυτον λεγοντες ει εκβαλλεις ημας αποστειλον ημας εις την αγελην των χοιρων
			<A NAME=40N_8_32><B> 8,32 </B>και ειπεν αυτοις υπαγετε οι δε εξελθοντες απηλθον εις τους χοιρους και ιδου ωρμησεν πασα η αγελη κατα του κρημνου εις την θαλασσαν και απεθανον εν τοις υδασιν
			<A NAME=40N_8_33><B> 8,33 </B>οι δε βοσκοντες εφυγον και απελθοντες εις την πολιν απηγγειλαν παντα και τα των δαιμονιζομενων
			<A NAME=40N_8_34><B> 8,34 </B>και ιδου πασα η πολις εξηλθεν εις υπαντησιν του ιησου και ιδοντες αυτον παρεκαλεσαν οπως μεταβη απο των οριων αυτων
			</P><P ALIGN= "CENTER"><A NAME=40N_9><H2>Matthäus<BR>Kapitel 9</H2></P><P ALIGN= "LEFT"><A NAME=40N_9_2><B> 9,2 </B>και ιδου προσεφερον αυτω παραλυτικον επι κλινης βεβλημενον και ιδων ο ιησους την πιστιν αυτων ειπεν τω παραλυτικω θαρσει τεκνον αφιενται σου αι αμαρτιαι
			<A NAME=40N_9_3><B> 9,3 </B>και ιδου τινες των γραμματεων ειπον εν εαυτοις ουτος βλασφημει
			<A NAME=40N_9_4><B> 9,4 </B>και ιδων ο ιησους τας ενθυμησεις αυτων ειπεν ινατι ενθυμεισθε πονηρα εν ταις καρδιαις υμων
			<A NAME=40N_9_5><B> 9,5 </B>τι γαρ εστιν ευκοπωτερον ειπειν αφιενται σου αι αμαρτιαι η ειπειν εγειρε και περιπατει
			<A NAME=40N_9_6><B> 9,6 </B>ινα δε ειδητε οτι εξουσιαν εχει ο υιος του ανθρωπου επι της γης αφιεναι αμαρτιας τοτε λεγει τω παραλυτικω εγερθεις αρον σου την κλινην και υπαγε εις τον οικον σου
			<A NAME=40N_9_7><B> 9,7 </B>και εγερθεις απηλθεν εις τον οικον αυτου
			<A NAME=40N_9_8><B> 9,8 </B>ιδοντες δε οι οχλοι εφοβηθησαν και εδοξασαν τον θεον τον δοντα εξουσιαν τοιαυτην τοις ανθρωποις
			<A NAME=40N_9_9><B> 9,9 </B>και παραγων ο ιησους εκειθεν ειδεν ανθρωπον καθημενον επι το τελωνιον μαθθαιον λεγομενον και λεγει αυτω ακολουθει μοι και αναστας ηκολουθησει αυτω
			<A NAME=40N_9_10><B> 9,10 </B>και εγενετο αυτου ανακειμενου εν τη οικια ιδου πολλοι τελωναι και αμαρτωλοι ελθοντες συνανεκειντο τω ιησου και τοις μαθηταις αυτου
			<A NAME=40N_9_11><B> 9,11 </B>και ιδοντες οι φαρισαιοι ελεγον τοις μαθηταις αυτου διατι μετα των τελωνων και αμαρτωλων εσθιει ο διδασκαλος υμων
			<A NAME=40N_9_12><B> 9,12 </B>ο δε ακουσας ειπεν ου χρειαν εχουσιν οι ισχυοντες ιατρου αλλ οι κακως εχοντες
			<A NAME=40N_9_13><B> 9,13 </B>πορευθεντες δε μαθετε τι εστιν ελεος θελω και ου θυσιαν ου γαρ ηλθον καλεσαι δικαιους αλλα αμαρτωλους
			<A NAME=40N_9_14><B> 9,14 </B>τοτε προσερχονται αυτω οι μαθηται ιωαννου λεγοντες διατι ημεις και οι φαρισαιοι νηστευομεν οι δε μαθηται σου ου νηστευουσιν
			<A NAME=40N_9_15><B> 9,15 </B>και ειπεν αυτοις ο ιησους μη δυνανται οι υιοι του νυμφωνος πενθειν εφ οσον μετ αυτων εστιν ο νυμφιος ελευσονται δε ημεραι οταν απαρθη απ αυτων ο νυμφιος και τοτε νηστευσουσιν
			<A NAME=40N_9_16><B> 9,16 </B>ουδεις δε επιβαλλει επιβλημα ρακους αγναφου επι ιματιω παλαιω αιρει γαρ το πληρωμα αυτου απο του ιματιου και χειρον σχισμα γινεται
			<A NAME=40N_9_17><B> 9,17 </B>ουδε βαλλουσιν οινον νεον εις ασκους παλαιους ει δε μηγε ρηγνυνται οι ασκοι και ο οινος εκχειται και οι ασκοι απολλυνται αλλα βαλλουσιν οινον νεον εις ασκους καινους και αμφοτεροι συντηρουνται
			<A NAME=40N_9_18><B> 9,18 </B>ταυτα αυτου λαλουντος αυτοις ιδου αρχων εισελθων προσεκυνει αυτω λεγων η θυγατηρ μου αρτι ετελευτησεν αλλα ελθων επιθες την χειρα σου επ αυτην και ζησεται
			<A NAME=40N_9_19><B> 9,19 </B>και εγερθεις ο ιησους ηκολουθει αυτω και οι μαθηται αυτου
			<A NAME=40N_9_20><B> 9,20 </B>και ιδου γυνη αιμορροουσα δωδεκα ετη προσελθουσα οπισθεν ηψατο του κρασπεδου του ιματιου αυτου
			<A NAME=40N_9_21><B> 9,21 </B>ελεγεν γαρ εν εαυτη εαν μονον αψωμαι του ιματιου αυτου σωθησομαι
			<A NAME=40N_9_22><B> 9,22 </B>ο δε στραφεις και ιδων αυτην ειπεν θαρσει θυγατερ η πιστις σου σεσωκεν σε και εσωθη η γυνη απο της ωρας εκεινης
			<A NAME=40N_9_23><B> 9,23 </B>και ελθων ο ιησους εις την οικιαν του αρχοντος και ιδων τους αυλητας και τον οχλον θορυβουμενον ελεγεν
			<A NAME=40N_9_24><B> 9,24 </B>αναχωρειτε ου γαρ απεθανεν το κορασιον αλλα καθευδει και κατεγελων αυτου
			<A NAME=40N_9_25><B> 9,25 </B>οτε δε εξεβληθη ο οχλος εισελθων εκρατησεν της χειρος αυτης και ηγερθη το κορασιον
			<A NAME=40N_9_26><B> 9,26 </B>και εξηλθεν η φημη αυτη εις ολην την γην εκεινην
			<A NAME=40N_9_27><B> 9,27 </B>και παραγοντι εκειθεν τω ιησου ηκολουθησαν αυτω δυο τυφλοι κραζοντες και λεγοντες ελεησον ημας υιος δαυειδ
			<A NAME=40N_9_28><B> 9,28 </B>ελθοντι δε εις την οικιαν προσηλθον αυτω οι τυφλοι και λεγει αυτοις ο ιησους πιστευετε οτι δυναμαι τουτο ποιησαι λεγουσιν αυτω ναι κυριε
			<A NAME=40N_9_29><B> 9,29 </B>τοτε ηψατο των οφθαλμων αυτων λεγων κατα την πιστιν υμων γενηθητω υμιν
			<A NAME=40N_9_30><B> 9,30 </B>και ανεωχθησαν αυτων οι οφθαλμοι και ενεβριμηθη αυτοις ο ιησους λεγων ορατε μηδεις γινωσκετω
			<A NAME=40N_9_31><B> 9,31 </B>οι δε εξελθοντες διεφημισαν αυτον εν ολη τη γη εκεινη
			<A NAME=40N_9_32><B> 9,32 </B>αυτων δε εξερχομενων ιδου προσηνεγκαν αυτω ανθρωπον κωφον δαιμονιζομενον
			<A NAME=40N_9_33><B> 9,33 </B>και εκβληθεντος του δαιμονιου ελαλησεν ο κωφος και εθαυμασαν οι οχλοι λεγοντες ουδεποτε εφανη ουτως εν τω ισραηλ
			<A NAME=40N_9_34><B> 9,34 </B>οι δε φαρισαιοι ελεγον εν τω αρχοντι των δαιμονιων εκβαλλει τα δαιμονια
			<A NAME=40N_9_35><B> 9,35 </B>και περιηγεν ο ιησους τας πολεις πασας και τας κωμας διδασκων εν ταις συναγωγαις αυτων και κηρυσσων το ευαγγελιον της βασιλειας και θεραπευων πασαν νοσον και πασαν μαλακιαν
			<A NAME=40N_9_36><B> 9,36 </B>ιδων δε τους οχλους εσπλαγχνισθη περι αυτων οτι ησαν εσκυλμενοι και ερριμμενοι ωσει προβατα μη εχοντα ποιμενα
			<A NAME=40N_9_37><B> 9,37 </B>τοτε λεγει τοις μαθηταις αυτου ο μεν θερισμος πολυς οι δε εργαται ολιγοι
			<A NAME=40N_9_38><B> 9,38 </B>δεηθητε ουν του κυριου του θερισμου οπως εκβαλη εργατας εις τον θερισμον αυτου
			</P><P ALIGN= "CENTER"><A NAME=40N_10><H2>Matthäus<BR>Kapitel 10</H2></P><P ALIGN= "LEFT"><A NAME=40N_10_2><B> 10,2 </B>των δε δωδεκα αποστολων τα ονοματα εστιν ταυτα πρωτος σιμων ο λεγομενος πετρος και ανδρεας ο αδελφος αυτου και ιακωβος ο του ζεβεδαιου και ιωαννης ο αδελφος αυτου
			<A NAME=40N_10_3><B> 10,3 </B>φιλιππος και βαρθολομαιος θωμας και μαθθαιος ο τελωνης ιακωβος ο του αλφαιου και λεββαιοσ
			<A NAME=40N_10_4><B> 10,4 </B>σιμων ο καναναιος και ιουδας ο ισκαριωτης ο και παραδους αυτον
			<A NAME=40N_10_5><B> 10,5 </B>τουτους τους δωδεκα απεστειλεν ο ιησους παραγγειλας αυτοις λεγων εις οδον εθνων μη απελθητε και εις πολιν σαμαριτων μη εισελθητε
			<A NAME=40N_10_6><B> 10,6 </B>πορευεσθε δε μαλλον προς τα προβατα τα απολωλοτα οικου ισραηλ
			<A NAME=40N_10_7><B> 10,7 </B>πορευομενοι δε κηρυσσετε λεγοντες οτι ηγγικεν η βασιλεια των ουρανων
			<A NAME=40N_10_8><B> 10,8 </B>ασθενουντας θεραπευετε νεκρους εγειρετε λεπρους καθαριζετε δαιμονια εκβαλλετε δωρεαν ελαβετε δωρεαν δοτε
			<A NAME=40N_10_9><B> 10,9 </B>μη κτησησθε χρυσον μηδε αργυρον μηδε χαλκον εις τας ζωνας υμων
			<A NAME=40N_10_10><B> 10,10 </B>μη πηραν εις οδον μηδε δυο χιτωνας μηδε υποδηματα μηδε ραβδον αξιος γαρ ο εργατης της τροφης αυτου
			<A NAME=40N_10_11><B> 10,11 </B>εις ην δ αν πολιν η κωμην εισελθητε εξετασατε τις εν αυτη αξιος εστιν κακει μεινατε εως αν εξελθητε
			<A NAME=40N_10_12><B> 10,12 </B>εισερχομενοι δε εις την οικιαν ασπασασθε αυτην
			<A NAME=40N_10_13><B> 10,13 </B>και εαν μεν η η οικια αξια ελθατω η ειρηνη υμων επ αυτην εαν δε μη η αξια η ειρηνη υμων προς υμας επιστραφητω
			<A NAME=40N_10_14><B> 10,14 </B>και ος αν μη δεξηται υμας μηδε ακουση τους λογους υμων εξερχομενοι εξω της οικιας η της πολεως εκεινης εκτιναξατε τον κονιορτον εκ των ποδων υμων
			<A NAME=40N_10_15><B> 10,15 </B>αμην λεγω υμιν ανεκτοτερον εσται γη σοδομων και γομορρων εν ημερα κρισεως η τη πολει εκεινη
			<A NAME=40N_10_16><B> 10,16 </B>ιδου εγω αποστελλω υμας ως προβατα εν μεσω λυκων γινεσθε ουν φρονιμοι ως οι οφεις και ακεραιοι ως αι περιστεραι
			<A NAME=40N_10_17><B> 10,17 </B>προσεχετε δε απο των ανθρωπων παραδωσουσιν γαρ υμας εις συνεδρια και εν ταις συναγωγαις αυτων μαστιγωσουσιν υμας
			<A NAME=40N_10_18><B> 10,18 </B>και επι ηγεμονας δε και βασιλεις αχθησεσθε ενεκεν εμου εις μαρτυριον αυτοις και τοις εθνεσιν
			<A NAME=40N_10_19><B> 10,19 </B>οταν δε παραδωσιν υμας μη μεριμνησητε πως η τι λαλησητε δοθησεται γαρ υμιν εν εκεινη τη ωρα τι λαλησητε
			<A NAME=40N_10_20><B> 10,20 </B>ου γαρ υμεις εστε οι λαλουντες αλλα το πνευμα του πατρος υμων το λαλουν εν υμιν
			<A NAME=40N_10_21><B> 10,21 </B>παραδωσει δε αδελφος αδελφον εις θανατον και πατηρ τεκνον και επαναστησονται τεκνα επι γονεις και θανατωσουσιν αυτους
			<A NAME=40N_10_22><B> 10,22 </B>και εσεσθε μισουμενοι υπο παντων δια το ονομα μου ο δε υπομεινας εις τελος ουτος σωθησεται
			<A NAME=40N_10_23><B> 10,23 </B>οταν δε διωκωσιν υμας εν τη πολει ταυτη φευγετε εις την ετεραν αμην γαρ λεγω υμιν ου μη τελεσητε τας πολεις του ισραηλ εως ελθη ο υιος του ανθρωπου
			<A NAME=40N_10_24><B> 10,24 </B>ουκ εστιν μαθητης υπερ τον διδασκαλον ουδε δουλος υπερ τον κυριον αυτου
			<A NAME=40N_10_25><B> 10,25 </B>αρκετον τω μαθητη ινα γενηται ως ο διδασκαλος αυτου και ο δουλος ως ο κυριος αυτου ει τον οικοδεσποτην βεελζεβουλ επεκαλεσαν ποσω μαλλον τους οικιακους αυτου
			<A NAME=40N_10_26><B> 10,26 </B>μη ουν φοβηθητε αυτους ουδεν γαρ εστιν κεκαλυμμενον ο ουκ αποκαλυφθησεται και κρυπτον ο ου γνωσθησεται
			<A NAME=40N_10_27><B> 10,27 </B>ο λεγω υμιν εν τη σκοτια ειπατε εν τω φωτι και ο εις το ους ακουετε κηρυξατε επι των δωματων
			<A NAME=40N_10_28><B> 10,28 </B>και μη φοβεισθε απο των αποκτεννοντων το σωμα την δε ψυχην μη δυναμενων αποκτειναι φοβεισθε δε μαλλον τον δυναμενον και ψυχην και σωμα απολεσαι εν γεεννη
			<A NAME=40N_10_29><B> 10,29 </B>ουχι δυο στρουθια ασσαριου πωλειται και εν εξ αυτων ου πεσειται επι την γην ανευ του πατρος υμων
			<A NAME=40N_10_30><B> 10,30 </B>υμων δε και αι τριχες της κεφαλης πασαι ηριθμημεναι εισιν
			<A NAME=40N_10_31><B> 10,31 </B>μη ουν φοβεισθε πολλων στρουθιων διαφερετε υμεις
			<A NAME=40N_10_32><B> 10,32 </B>πας ουν οστις ομολογησει εν εμοι εμπροσθεν των ανθρωπων ομολογησω καγω εν αυτω εμπροσθεν του πατρος μου του εν ουρανοις
			<A NAME=40N_10_33><B> 10,33 </B>οστις δ αν αρνησηται με εμπροσθεν των ανθρωπων αρνησομαι καγω αυτον εμπροσθεν του πατρος μου του εν ουρανοις
			<A NAME=40N_10_34><B> 10,34 </B>μη νομισητε οτι ηλθον βαλειν ειρηνην επι την γην ουκ ηλθον βαλειν ειρηνην αλλα μαχαιραν
			<A NAME=40N_10_35><B> 10,35 </B>ηλθον γαρ διχασαι ανθρωπον κατα του πατρος αυτου και θυγατερα κατα της μητρος αυτης και νυμφην κατα της πενθερας αυτης
			<A NAME=40N_10_36><B> 10,36 </B>και εχθροι του ανθρωπου οι οικιακοι αυτου
			<A NAME=40N_10_37><B> 10,37 </B>ο φιλων πατερα η μητερα υπερ εμε ουκ εστιν μου αξιος και ο φιλων υιον η θυγατερα υπερ εμε ουκ εστιν μου αξιος
			<A NAME=40N_10_38><B> 10,38 </B>και ος ου λαμβανει τον σταυρον αυτου και ακολουθει οπισω μου ουκ εστιν μου αξιος
			<A NAME=40N_10_39><B> 10,39 </B>ο ευρων την ψυχην αυτου απολεσει αυτην και ο απολεσας την ψυχην αυτου ενεκεν εμου ευρησει αυτην
			<A NAME=40N_10_40><B> 10,40 </B>ο δεχομενος υμας εμε δεχεται και ο εμε δεχομενος δεχεται τον αποστειλαντα με
			<A NAME=40N_10_41><B> 10,41 </B>ο δεχομενος προφητην εις ονομα προφητου μισθον προφητου λημψεται και ο δεχομενος δικαιον εις ονομα δικαιου μισθον δικαιου λημψεται
			<A NAME=40N_10_42><B> 10,42 </B>και ος αν ποτιση ενα των μικρων τουτων ποτηριον ψυχρου μονον εις ονομα μαθητου αμην λεγω υμιν ου μη απολεση τον μισθον αυτου
			</P><P ALIGN= "CENTER"><A NAME=40N_11><H2>Matthäus<BR>Kapitel 11</H2></P><P ALIGN= "LEFT"><A NAME=40N_11_2><B> 11,2 </B>ο δε ιωαννης ακουσας εν τω δεσμωτηριω τα εργα του χριστου πεμψας δια των μαθητων αυτου
			<A NAME=40N_11_3><B> 11,3 </B>ειπεν αυτω συ ει ο ερχομενος η ετερον προσδοκωμεν
			<A NAME=40N_11_4><B> 11,4 </B>και αποκριθεις ο ιησους ειπεν αυτοις πορευθεντες απαγγειλατε ιωαννη α ακουετε και βλεπετε
			<A NAME=40N_11_5><B> 11,5 </B>τυφλοι αναβλεπουσιν και χωλοι περιπατουσιν λεπροι καθαριζονται και κωφοι ακουουσιν και νεκροι εγειρονται και πτωχοι ευαγγελιζονται
			<A NAME=40N_11_6><B> 11,6 </B>και μακαριος εστιν ος εαν μη σκανδαλισθη εν εμοι
			<A NAME=40N_11_7><B> 11,7 </B>τουτων δε πορευομενων ηρξατο ο ιησους λεγειν τοις οχλοις περι ιωαννου τι εξηλθατε εις την ερημον θεασασθαι καλαμον υπο ανεμου σαλευομενον
			<A NAME=40N_11_8><B> 11,8 </B>αλλα τι εξηλθατε ιδειν ανθρωπον εν μαλακοις ημφιεσμενον ιδου οι τα μαλακα φορουντες εν τοις οικοις των βασιλεων
			<A NAME=40N_11_9><B> 11,9 </B>αλλα τι εξηλθατε προφητην ιδειν ναι λεγω υμιν και περισσοτερον προφητου
			<A NAME=40N_11_10><B> 11,10 </B>ουτος εστιν περι ου γεγραπται ιδου εγω αποστελλω τον αγγελον μου προ προσωπου σου ος κατασκευασει την οδον σου εμπροσθεν σου
			<A NAME=40N_11_11><B> 11,11 </B>αμην λεγω υμιν ουκ εγηγερται εν γεννητοις γυναικων μειζων ιωαννου του βαπτιστου ο δε μικροτερος εν τη βασιλεια των ουρανων μειζων αυτου εστιν
			<A NAME=40N_11_12><B> 11,12 </B>απο δε των ημερων ιωαννου του βαπτιστου εως αρτι η βασιλεια των ουρανων βιαζεται και βιασται αρπαζουσιν αυτην
			<A NAME=40N_11_13><B> 11,13 </B>παντες γαρ οι προφηται και ο νομος εως ιωαννου επροφητευσαν
			<A NAME=40N_11_14><B> 11,14 </B>και ει θελετε δεξασθαι αυτος εστιν ηλειας ο μελλων ερχεσθαι
			<A NAME=40N_11_15><B> 11,15 </B>ο εχων ωτα ακουετω
			<A NAME=40N_11_16><B> 11,16 </B>τινι δε ομοιωσω την γενεαν ταυτην ομοια εστιν παιδιοις καθημενοις εν ταις αγοραις α προσφωνουντα τοις ετεροις
			<A NAME=40N_11_17><B> 11,17 </B>λεγουσιν ηυλησαμεν υμιν και ουκ ωρχησασθε εθρηνησαμεν και ουκ εκοψασθε
			<A NAME=40N_11_18><B> 11,18 </B>ηλθεν γαρ ιωαννης μητε εσθιων μητε πινων και λεγουσιν δαιμονιον εχει
			<A NAME=40N_11_19><B> 11,19 </B>ηλθεν ο υιος του ανθρωπου εσθιων και πινων και λεγουσιν ιδου ανθρωπος φαγος και οινοποτης τελωνων φιλος και αμαρτωλων και εδικαιωθη η σοφια απο των εργων αυτης
			<A NAME=40N_11_20><B> 11,20 </B>τοτε ηρξατο ονειδιζειν τας πολεις εν αις εγενοντο αι πλεισται δυναμεις αυτου οτι ου μετενοησαν
			<A NAME=40N_11_21><B> 11,21 </B>ουαι σοι χοραζιν ουαι σοι βηθσαιδαν οτι ει εν τυρω και σιδωνι εγενοντο αι δυναμεις αι γενομεναι εν υμιν παλαι αν εν σακκω και σποδω μετενοησαν
			<A NAME=40N_11_22><B> 11,22 </B>πλην λεγω υμιν τυρω και σιδωνι ανεκτοτερον εσται εν ημερα κρισεως η υμιν
			<A NAME=40N_11_23><B> 11,23 </B>και συ καφαρναουμ μη εως ουρανου υψωθηση εως αδου καταβιβασθηση οτι ει εν σοδομοις εγενηθησαν αι δυναμεις αι γενομεναι εν σοι εμεινεν αν μεχρι της σημερον
			<A NAME=40N_11_24><B> 11,24 </B>πλην λεγω υμιν οτι γη σοδομων ανεκτοτερον εσται εν ημερα κρισεως η σοι
			<A NAME=40N_11_25><B> 11,25 </B>εν εκεινω τω καιρω αποκριθεις ο ιησους ειπεν εξομολογουμαι σοι πατερ κυριε του ουρανου και της γης οτι εκρυψας ταυτα απο σοφων και συνετων και απεκαλυψας αυτα νηπιοις
			<A NAME=40N_11_26><B> 11,26 </B>ναι ο πατηρ οτι ουτως ευδοκια εγενετο εμπροσθεν σου
			<A NAME=40N_11_27><B> 11,27 </B>παντα μοι παρεδοθη υπο του πατρος μου και ουδεις επιγινωσκει τον υιον ει μη ο πατηρ ουδε τον πατερα τις επιγινωσκει ει μη ο υιος και ω εαν βουληται ο υιος αποκαλυψαι
			<A NAME=40N_11_28><B> 11,28 </B>δευτε προς με παντες οι κοπιωντες και πεφορτισμενοι καγω αναπαυσω υμας
			<A NAME=40N_11_29><B> 11,29 </B>αρατε τον ζυγον μου εφ υμας και μαθετε απ εμου οτι πραυς ειμι και ταπεινος τη καρδια και ευρησετε αναπαυσιν ταις ψυχαις υμων
			<A NAME=40N_11_30><B> 11,30 </B>ο γαρ ζυγος μου χρηστος και το φορτιον μου ελαφρον εστιν
			</P><P ALIGN= "CENTER"><A NAME=40N_12><H2>Matthäus<BR>Kapitel 12</H2></P><P ALIGN= "LEFT"><A NAME=40N_12_2><B> 12,2 </B>οι δε φαρισαιοι ιδοντες ειπαν αυτω ιδου οι μαθηται σου ποιουσιν ο ουκ εξεστιν ποιειν εν σαββατω
			<A NAME=40N_12_3><B> 12,3 </B>ο δε ειπεν αυτοις ουκ ανεγνωτε τι εποιησεν δαυειδ οτε επεινασεν και οι μετ αυτου
			<A NAME=40N_12_4><B> 12,4 </B>πως εισηλθεν εις τον οικον του θεου και τους αρτους της προθεσεως εφαγον ο ουκ εξον ην αυτω φαγειν ουδε τοις μετ αυτου ει μη τοις ιερευσιν μονοις
			<A NAME=40N_12_5><B> 12,5 </B>η ουκ ανεγνωτε εν τω νομω οτι τοις σαββασιν οι ιερεις εν τω ιερω το σαββατον βεβηλουσιν και αναιτιοι εισιν
			<A NAME=40N_12_6><B> 12,6 </B>λεγω δε υμιν οτι του ιερου μειζον εστιν ωδε
			<A NAME=40N_12_7><B> 12,7 </B>ει δε εγνωκειτε τι εστιν ελεος θελω και ου θυσιαν ουκ αν κατεδικασατε τους αναιτιους
			<A NAME=40N_12_8><B> 12,8 </B>κυριος γαρ εστιν του σαββατου ο υιος του ανθρωπου
			<A NAME=40N_12_9><B> 12,9 </B>και μεταβας εκειθεν ηλθεν εις την συναγωγην αυτων
			<A NAME=40N_12_10><B> 12,10 </B>και ιδου ανθρωπος χειρα εχων ξηραν και επηρωτησαν αυτον λεγοντες ει εξεστιν τοις σαββασιν θεραπευσαι ινα κατηγορησωσιν αυτου
			<A NAME=40N_12_11><B> 12,11 </B>ο δε ειπεν αυτοις τις εσται εξ υμων ανθρωπος ος εξει προβατον εν και εαν εμπεση τουτο τοις σαββασιν εις βοθυνον ουχι κρατησει αυτο και εγερει
			<A NAME=40N_12_12><B> 12,12 </B>ποσω ουν διαφερει ανθρωπος προβατου ωστε εξεστιν τοις σαββασιν καλως ποιειν
			<A NAME=40N_12_13><B> 12,13 </B>τοτε λεγει τω ανθρωπω εκτεινον σου την χειρα και εξετεινεν και απεκατεσταθη υγιης ως η αλλη
			<A NAME=40N_12_14><B> 12,14 </B>εξελθοντες δε οι φαρισαιοι συμβουλιον ελαβον κατ αυτου οπως αυτον απολεσωσιν
			<A NAME=40N_12_15><B> 12,15 </B>ο δε ιησους γνους ανεχωρησεν εκειθεν και ηκολουθησαν αυτω πολλοι και εθεραπευσεν αυτους παντας
			<A NAME=40N_12_16><B> 12,16 </B>και επετιμησεν αυτοις ινα μη φανερον αυτον ποιησωσιν
			<A NAME=40N_12_17><B> 12,17 </B>ινα πληρωθη το ρηθεν δια ησαιου του προφητου λεγοντος
			<A NAME=40N_12_18><B> 12,18 </B>ιδου ο παις μου ον ηρετισα ο αγαπητος μου εις ον ηυδοκησεν η ψυχη μου θησω το πνευμα μου επ αυτον και κρισιν τοις εθνεσιν απαγγελει
			<A NAME=40N_12_19><B> 12,19 </B>ουκ ερισει ουδε κραυγασει ουδε ακουσει τις εν ταις πλατειαις την φωνην αυτου
			<A NAME=40N_12_20><B> 12,20 </B>καλαμον συντετριμμενον ου κατεαξει και λινον τυφομενον ου σβεσει εως αν εκβαλη εις νικος την κρισιν
			<A NAME=40N_12_21><B> 12,21 </B>και τω ονοματι αυτου εθνη ελπιουσιν
			<A NAME=40N_12_22><B> 12,22 </B>τοτε προσηνεχθη αυτω δαιμονιζομενος τυφλος και κωφος και εθεραπευσεν αυτον ωστε τον κωφον λαλειν και βλεπειν
			<A NAME=40N_12_23><B> 12,23 </B>και εξισταντο παντες οι οχλοι και ελεγον μητι ουτος εστιν ο υιος δαυειδ
			<A NAME=40N_12_24><B> 12,24 </B>οι δε φαρισαιοι ακουσαντες ειπον ουτος ουκ εκβαλλει τα δαιμονια ει μη εν τω βεελζεβουλ αρχοντι των δαιμονιων
			<A NAME=40N_12_25><B> 12,25 </B>ειδως δε τας ενθυμησεις αυτων ειπεν αυτοις πασα βασιλεια μερισθεισα καθ εαυτης ερημουται και πασα πολις η οικια μερισθεισα καθ εαυτης ου σταθησεται
			<A NAME=40N_12_26><B> 12,26 </B>και ει ο σατανας τον σαταναν εκβαλλει εφ εαυτον εμερισθη πως ουν σταθησεται η βασιλεια αυτου
			<A NAME=40N_12_27><B> 12,27 </B>και ει εγω εν βεελζεβουλ εκβαλλω τα δαιμονια οι υιοι υμων εν τινι εκβαλλουσιν δια τουτο αυτοι κριται εσονται υμων
			<A NAME=40N_12_28><B> 12,28 </B>ει δε εν πνευματι θεου εγω εκβαλλω τα δαιμονια αρα εφθασεν εφ υμας η βασιλεια του θεου
			<A NAME=40N_12_29><B> 12,29 </B>η πως δυναται τις εισελθειν εις την οικιαν του ισχυρου και τα σκευη αυτου αρπασαι εαν μη πρωτον δηση τον ισχυρον και τοτε την οικιαν αυτου διαρπαση
			<A NAME=40N_12_30><B> 12,30 </B>ο μη ων μετ εμου κατ εμου εστιν και ο μη συναγων μετ εμου σκορπιζει
			<A NAME=40N_12_31><B> 12,31 </B>δια τουτο λεγω υμιν πασα αμαρτια και βλασφημια αφεθησεται τοις ανθρωποις η δε του πνευματος βλασφημια ουκ αφεθησεται
			<A NAME=40N_12_32><B> 12,32 </B>και ος εαν ειπη λογον κατα του υιου του ανθρωπου αφεθησεται αυτω ος δ αν ειπη κατα του πνευματος του αγιου ουκ αφεθησεται αυτω ουτε εν τουτω τω αιωνι ουτε εν τω μελλοντι
			<A NAME=40N_12_33><B> 12,33 </B>η ποιησατε το δενδρον καλον και τον καρπον αυτου καλον η ποιησατε το δενδρον σαπρον και τον καρπον αυτου σαπρον εκ γαρ του καρπου το δενδρον γινωσκεται
			<A NAME=40N_12_34><B> 12,34 </B>γεννηματα εχιδνων πως δυνασθε αγαθα λαλειν πονηροι οντες εκ γαρ του περισσευματος της καρδιας το στομα λαλει
			<A NAME=40N_12_35><B> 12,35 </B>ο αγαθος ανθρωπος εκ του αγαθου θησαυρου εκβαλλει τα αγαθα και ο πονηρος ανθρωπος εκ του πονηρου θησαυρου εκβαλλει πονηρα
			<A NAME=40N_12_36><B> 12,36 </B>λεγω δε υμιν οτι παν ρημα αργον ο λαλησουσιν οι ανθρωποι αποδωσουσιν περι αυτου λογον εν ημερα κρισεως
			<A NAME=40N_12_37><B> 12,37 </B>εκ γαρ των λογων σου δικαιωθηση και εκ των λογων σου καταδικασθηση
			<A NAME=40N_12_38><B> 12,38 </B>τοτε απεκριθησαν αυτω τινες των γραμματεων και φαρισαιων λεγοντες διδασκαλε θελομεν απο σου σημειον ιδειν
			<A NAME=40N_12_39><B> 12,39 </B>ο δε αποκριθεις ειπεν αυτοις γενεα πονηρα και μοιχαλις σημειον επιζητει και σημειον ου δοθησεται αυτη ει μη το σημειον ιωνα του προφητου
			<A NAME=40N_12_40><B> 12,40 </B>ωσπερ γαρ ην ιωνας εν τη κοιλια του κητους τρεις ημερας και τρεις νυκτας ουτως εσται ο υιος του ανθρωπου εν τη καρδια της γης τρεις ημερας και τρεις νυκτας
			<A NAME=40N_12_41><B> 12,41 </B>ανδρες νινευειται αναστησονται εν τη κρισει μετα της γενεας ταυτης και κατακρινουσιν αυτην οτι μετενοησαν εις το κηρυγμα ιωνα και ιδου πλειον ιωνα ωδε
			<A NAME=40N_12_42><B> 12,42 </B>βασιλισσα νοτου εγερθησεται εν τη κρισει μετα της γενεας ταυτης και κατακρινει αυτην οτι ηλθεν εκ των περατων της γης ακουσαι την σοφιαν σολομωνος και ιδου πλειον σολομωνος ωδε
			<A NAME=40N_12_43><B> 12,43 </B>οταν δε το ακαθαρτον πνευμα εξελθη απο του ανθρωπου διερχεται δι ανυδρων τοπων ζητουν αναπαυσιν και ουχ ευρισκει
			<A NAME=40N_12_44><B> 12,44 </B>τοτε λεγει εις τον οικον μου επιστρεψω οθεν εξηλθον και ελθον ευρισκει σχολαζοντα και σεσαρωμενον και κεκοσμημενον
			<A NAME=40N_12_45><B> 12,45 </B>τοτε πορευεται και παραλαμβανει μεθ εαυτου επτα ετερα πνευματα πονηροτερα εαυτου και εισελθοντα κατοικει εκει και γινεται τα εσχατα του ανθρωπου εκεινου χειρονα των πρωτων ουτως εσται και τη γενεα ταυτη τη πονηρα
			<A NAME=40N_12_46><B> 12,46 </B>ετι αυτου λαλουντος τοις οχλοις ιδου η μητηρ και οι αδελφοι αυτου ειστηκεισαν εξω ζητουντες αυτω λαλησαι
			<A NAME=40N_12_47><B> 12,47 </B>αδελφοι σου εξω εστηκασιν ζητουντες σοι λαλησαι]
			<A NAME=40N_12_48><B> 12,48 </B>ο δε αποκριθεις ειπεν τω λεγοντι αυτω τις εστιν η μητηρ μου και τινες εισιν οι αδελφοι μου
			<A NAME=40N_12_49><B> 12,49 </B>και εκτεινας την χειρα επι τους μαθητας αυτου ειπεν ιδου η μητηρ μου και οι αδελφοι μου
			<A NAME=40N_12_50><B> 12,50 </B>οστις γαρ αν ποιηση το θελημα του πατρος μου του εν ουρανοις αυτος μου αδελφος και αδελφη και μητηρ εστιν
			</P><P ALIGN= "CENTER"><A NAME=40N_13><H2>Matthäus<BR>Kapitel 13</H2></P><P ALIGN= "LEFT"><A NAME=40N_13_2><B> 13,2 </B>και συνηχθησαν προς αυτον οχλοι πολλοι ωστε αυτον εις πλοιον εμβαντα καθησθαι και πας ο οχλος επι τον αιγιαλον ειστηκει
			<A NAME=40N_13_3><B> 13,3 </B>και ελαλησεν αυτοις πολλα εν παραβολαις λεγων ιδου εξηλθεν ο σπειρων του σπειρειν
			<A NAME=40N_13_4><B> 13,4 </B>και εν τω σπειρειν αυτον α μεν επεσεν παρα την οδον και ελθεν τα πετεινα και κατεφαγεν αυτα
			<A NAME=40N_13_5><B> 13,5 </B>αλλα δε επεσεν επι τα πετρωδη οπου ουκ ειχεν γην πολλην και ευθεως εξανετειλεν δια το μη εχειν βαθος γης
			<A NAME=40N_13_6><B> 13,6 </B>ηλιου δε ανατειλαντος εκαυματισθη και δια το μη εχειν ριζαν εξηρανθη
			<A NAME=40N_13_7><B> 13,7 </B>αλλα δε επεσεν επι τας ακανθας και ανεβησαν αι ακανθαι και επνιξαν αυτα
			<A NAME=40N_13_8><B> 13,8 </B>αλλα δε επεσεν επι την γην την καλην και εδιδου καρπον ο μεν εκατον ο δε εξηκοντα ο δε τριακοντα
			<A NAME=40N_13_9><B> 13,9 </B>ο εχων ωτα ακουετω
			<A NAME=40N_13_10><B> 13,10 </B>και προσελθοντες οι μαθηται ειπαν αυτω διατι εν παραβολαις λαλεις αυτοις
			<A NAME=40N_13_11><B> 13,11 </B>ο δε αποκριθεις ειπεν οτι υμιν δεδοται γνωναι τα μυστηρια της βασιλειας των ουρανων εκεινοις δε ου δεδοται
			<A NAME=40N_13_12><B> 13,12 </B>οστις γαρ εχει δοθησεται αυτω και περισσευθησεται οστις δε ουκ εχει και ο εχει αρθησεται απ αυτου
			<A NAME=40N_13_13><B> 13,13 </B>δια τουτο εν παραβολαις αυτοις λαλω οτι βλεποντες ου βλεπουσιν και ακουοντες ουκ ακουουσιν ουδε συνιουσιν
			<A NAME=40N_13_14><B> 13,14 </B>και αναπληρουται αυτοις η προφητεια ησαιου η λεγουσα ακοη ακουσετε και ου μη συνητε και βλεποντες βλεψετε και ου μη ιδητε
			<A NAME=40N_13_15><B> 13,15 </B>επαχυνθη γαρ η καρδια του λαου τουτου και τοις ωσιν βαρεως ηκουσαν και τους οφθαλμους αυτων εκαμμυσαν μηποτε ιδωσιν τοις οφθαλμοις και τοις ωσιν ακουσωσιν και τη καρδια συνωσιν και επιστρεψωσιν και ιασομαι αυτους
			<A NAME=40N_13_16><B> 13,16 </B>υμων δε μακαριοι οι οφθαλμοι οτι βλεπουσιν και τα ωτα υμων οτι ακουουσιν
			<A NAME=40N_13_17><B> 13,17 </B>αμην λεγω υμιν οτι πολλοι προφηται και δικαιοι επεθυμησαν ιδειν α βλεπετε και ουκ ιδαν και ακουσαι α ακουετε και ουκ ηκουσαν
			<A NAME=40N_13_18><B> 13,18 </B>υμεις ουν ακουσατε την παραβολην του σπειραντος
			<A NAME=40N_13_19><B> 13,19 </B>παντος ακουοντος τον λογον της βασιλειας και μη συνιεντος ερχεται ο πονηρος και αρπαζει το εσπαρμενον εν τη καρδια αυτου ουτος εστιν ο παρα την οδον σπαρεις
			<A NAME=40N_13_20><B> 13,20 </B>ο δε επι τα πετρωδη σπαρεις ουτος εστιν ο τον λογον ακουων και ευθυς μετα χαρας λαμβανων αυτον
			<A NAME=40N_13_21><B> 13,21 </B>ουκ εχει δε ριζαν εν εαυτω αλλα προσκαιρος εστιν γενομενης δε θλιψεως η διωγμου δια τον λογον ευθυς σκανδαλιζεται
			<A NAME=40N_13_22><B> 13,22 </B>ο δε εις τας ακανθας σπαρεις ουτος εστιν ο τον λογον ακουων και η μεριμνα του αιωνος και η απατη του πλουτου συμπνιγει τον λογον και ακαρπος γινεται
			<A NAME=40N_13_23><B> 13,23 </B>ο δε επι την καλην γην σπαρεις ουτος εστιν ο τον λογον ακουων και συνιεις ος δη καρποφορει και ποιει ο μεν εκατον ο δε εξηκοντα ο δε τριακοντα
			<A NAME=40N_13_24><B> 13,24 </B>αλλην παραβολην παρεθηκεν αυτοις λεγων ωμοιωθη η βασιλεια των ουρανων ανθρωπω σπειραντι καλον σπερμα εν τω αγρω αυτου
			<A NAME=40N_13_25><B> 13,25 </B>εν δε τω καθευδειν τους ανθρωπους ηλθεν αυτου ο εχθρος και επεσπειρεν ζιζανια ανα μεσον του σιτου και απηλθεν
			<A NAME=40N_13_26><B> 13,26 </B>οτε δε εβλαστησεν ο χορτος και καρπον εποιησεν τοτε εφανη και τα ζιζανια
			<A NAME=40N_13_27><B> 13,27 </B>προσελθοντες δε οι δουλοι του οικοδεσποτου ειπον αυτω κυριε ουχι καλον σπερμα εσπειρας εν τω σω αγρω ποθεν ουν εχει ζιζανια
			<A NAME=40N_13_28><B> 13,28 </B>ο δε εφη αυτοις εχθρος ανθρωπος τουτο εποιησεν οι δε δουλοι λεγουσιν αυτω θελεις ουν απελθοντες συλλεξωμεν αυτα
			<A NAME=40N_13_29><B> 13,29 </B>ο δε φησιν ου μηποτε συλλεγοντες τα ζιζανια εκριζωσητε αμα αυτοις τον σιτον
			<A NAME=40N_13_30><B> 13,30 </B>αφετε συναυξανεσθαι αμφοτερα μεχρι του θερισμου και εν καιρω του θερισμου ερω τοις θερισταις συλλεξατε πρωτον τα ζιζανια και δησατε αυτα εις δεσμας προς το κατακαυσαι αυτα τον δε σιτον συναγαγετε εις την αποθηκην μου
			<A NAME=40N_13_31><B> 13,31 </B>αλλην παραβολην παρεθηκεν αυτοις λεγων ομοια εστιν η βασιλεια των ουρανων κοκκω σιναπεως ον λαβων ανθρωπος εσπειρεν εν τω αγρω αυτου
			<A NAME=40N_13_32><B> 13,32 </B>ο μικροτερον μεν εστιν παντων των σπερματων οταν δε αυξηθη μειζον των λαχανων εστιν και γινεται δενδρον ωστε ελθειν τα πετεινα του ουρανου και κατασκηνουν εν τοις κλαδοις αυτου
			<A NAME=40N_13_33><B> 13,33 </B>αλλην παραβολην ελαλησεν αυτοις ομοια εστιν η βασιλεια των ουρανων ζυμη ην λαβουσα γυνη ενεκρυψεν εις αλευρου σατα τρια εως ου εζυμωθη ολον
			<A NAME=40N_13_34><B> 13,34 </B>ταυτα παντα ελαλησεν ο ιησους εν παραβολαις τοις οχλοις και χωρις παραβολης ουδεν ελαλει αυτοις
			<A NAME=40N_13_35><B> 13,35 </B>οπως πληρωθη το ρηθεν δια του προφητου ησαιου λεγοντος ανοιξω εν παραβολαις το στομα μου ερευξομαι κεκρυμμενα απο καταβολης
			<A NAME=40N_13_36><B> 13,36 </B>τοτε αφεις τους οχλους ηλθεν εις την οικιαν και προσηλθον αυτω οι μαθηται αυτου λεγοντες φρασον ημιν την παραβολην των ζιζανιων του αγρου
			<A NAME=40N_13_37><B> 13,37 </B>ο δε αποκριθεις ειπεν ο σπειρων το καλον σπερμα εστιν ο υιος του ανθρωπου
			<A NAME=40N_13_38><B> 13,38 </B>ο δε αγρος εστιν ο κοσμος το δε καλον σπερμα ουτοι εισιν οι υιοι της βασιλειας τα δε ζιζανια εισιν οι υιοι του πονηρου
			<A NAME=40N_13_39><B> 13,39 </B>ο δε εχθρος ο σπειρας αυτα εστιν ο διαβολος ο δε θερισμος συντελεια αιωνος εστιν οι δε θερισται αγγελοι εισιν
			<A NAME=40N_13_40><B> 13,40 </B>ωσπερ ουν συλλεγεται τα ζιζανια και πυρι κατακαιεται ουτως εσται εν τη συντελεια του αιωνος
			<A NAME=40N_13_41><B> 13,41 </B>αποστελει ο υιος του ανθρωπου τους αγγελους αυτου και συλλεξουσιν εκ της βασιλειας αυτου παντα τα σκανδαλα και τους ποιουντας την ανομιαν
			<A NAME=40N_13_42><B> 13,42 </B>και βαλουσιν αυτους εις την καμινον του πυρος εκει εσται ο κλαυθμος και ο βρυγμος των οδοντων
			<A NAME=40N_13_43><B> 13,43 </B>τοτε οι δικαιοι εκλαμψουσιν ως ο ηλιος εν τη βασιλεια του πατρος αυτων ο εχων ωτα ακουετω
			<A NAME=40N_13_44><B> 13,44 </B>ομοια εστιν η βασιλεια των ουρανων θησαυρω κεκρυμμενω εν τω αγρω ον ευρων ανθρωπος εκρυψεν και απο της χαρας αυτου υπαγει και πωλει παντα οσα εχει και αγοραζει τον αγρον εκεινον
			<A NAME=40N_13_45><B> 13,45 </B>παλιν ομοια εστιν η βασιλεια των ουρανων ανθρωπω εμπορω ζητουντι καλους μαργαριτας
			<A NAME=40N_13_46><B> 13,46 </B>ευρων δε ενα πολυτιμον μαργαριτην απελθων πεπρακεν παντα οσα ειχεν και ηγορασεν αυτον
			<A NAME=40N_13_47><B> 13,47 </B>παλιν ομοια εστιν η βασιλεια των ουρανων σαγηνη βληθειση εις την θαλασσαν και εκ παντος γενους συναγαγουση
			<A NAME=40N_13_48><B> 13,48 </B>ην οτε επληρωθη αναβιβασαντες επι τον αιγιαλον και καθισαντες συνελεξαν τα καλα εις αγγη τα δε σαπρα εξω εβαλον
			<A NAME=40N_13_49><B> 13,49 </B>ουτως εσται εν τη συντελεια του αιωνος εξελευσονται οι αγγελοι και αφοριουσιν τους πονηρους εκ μεσου των δικαιων
			<A NAME=40N_13_50><B> 13,50 </B>και βαλουσιν αυτους εις την καμινον του πυρος εκει εσται ο κλαυθμος και ο βρυγμος των οδοντων
			<A NAME=40N_13_51><B> 13,51 </B>συνηκατε ταυτα παντα λεγουσιν αυτω ναι
			<A NAME=40N_13_52><B> 13,52 </B>ο δε ειπεν αυτοις δια τουτο πας γραμματευς μαθητευθεις τη βασιλεια των ουρανων ομοιος εστιν ανθρωπω οικοδεσποτη οστις εκβαλλει εκ του θησαυρου αυτου καινα και παλαια
			<A NAME=40N_13_53><B> 13,53 </B>και εγενετο οτε ετελεσεν ο ιησους τας παραβολας ταυτας μετηρεν εκειθεν
			<A NAME=40N_13_54><B> 13,54 </B>και ελθων εις την πατριδα αυτου εδιδασκεν αυτους εν τη συναγωγη αυτων ωστε εκπλησσεσθαι αυτους και λεγειν ποθεν τουτω η σοφια αυτη και αι δυναμεις
			<A NAME=40N_13_55><B> 13,55 </B>ουχ ουτος εστιν ο του τεκτονος υιος ουχ η μητηρ αυτου λεγεται μαριαμ και οι αδελφοι αυτου ιακωβος και ιωσηφ και σιμων και ιουδας
			<A NAME=40N_13_56><B> 13,56 </B>και αι αδελφαι αυτου ουχι πασαι προς ημας εισιν ποθεν ουν τουτω ταυτα παντα
			<A NAME=40N_13_57><B> 13,57 </B>και εσκανδαλιζοντο εν αυτω ο δε ιησους ειπεν αυτοις ουκ εστιν προφητης ατιμος ει μη εν τη ιδια πατριδι και εν τη οικια αυτου
			<A NAME=40N_13_58><B> 13,58 </B>και ουκ εποιησεν εκει δυναμεις πολλας δια την απιστιαν αυτων
			</P><P ALIGN= "CENTER"><A NAME=40N_14><H2>Matthäus<BR>Kapitel 14</H2></P><P ALIGN= "LEFT"><A NAME=40N_14_2><B> 14,2 </B>και ειπεν τοις παισιν αυτου ουτος εστιν ιωαννης ο βαπτιστης αυτος ηγερθη απο των νεκρων και δια τουτο αι δυναμεις ενεργουσιν εν αυτω
			<A NAME=40N_14_3><B> 14,3 </B>ο γαρ ηρωδης κρατησας τον ιωαννην εδησεν και εν φυλακη απεθετο δια ηρωδιαδα την γυναικα [φιλιππου] του αδελφου αυτου
			<A NAME=40N_14_4><B> 14,4 </B>ελεγεν γαρ ο ιωαννης αυτω ουκ εξεστιν σοι εχειν αυτην
			<A NAME=40N_14_5><B> 14,5 </B>και θελων αυτον αποκτειναι εφοβηθη τον οχλον οτι ως προφητην αυτον ειχον
			<A NAME=40N_14_6><B> 14,6 </B>γενεσιοις δε γενομενοις του ηρωδου ωρχησατο η θυγατηρ της ηρωδιαδος εν τω μεσω και ηρεσεν τω ηρωδη
			<A NAME=40N_14_7><B> 14,7 </B>οθεν μεθ ορκου ωμολογησεν αυτη δουναι ο εαν αιτησηται
			<A NAME=40N_14_8><B> 14,8 </B>η δε προβιβασθεισα υπο της μητρος αυτης δος μοι φησιν ωδε επι πινακι την κεφαλην ιωαννου του βαπτιστου
			<A NAME=40N_14_9><B> 14,9 </B>και λυπηθεις ο βασιλευς δια τους ορκους και τους συνανακειμενους εκελευσεν δοθηναι
			<A NAME=40N_14_10><B> 14,10 </B>και πεμψας απεκεφαλισεν ιωαννην εν τη φυλακη
			<A NAME=40N_14_11><B> 14,11 </B>και ηνεχθη η κεφαλη αυτου επι πινακι και εδοθη τω κορασιω και ηνεγκεν τη μητρι αυτης
			<A NAME=40N_14_12><B> 14,12 </B>και προσελθοντες οι μαθηται αυτου ηραν το πτωμα και εθαψαν αυτον και ελθοντες απηγγειλαν τω ιησου
			<A NAME=40N_14_13><B> 14,13 </B>ακουσας δε ο ιησους ανεχωρησεν εκειθεν εν πλοιω εις ερημον τοπον κατ ιδιαν και ακουσαντες οι οχλοι ηκολουθησαν αυτω πεζοι απο των πολεων
			<A NAME=40N_14_14><B> 14,14 </B>και εξελθων ειδεν πολυν οχλον και εσπλαγχνισθη επ αυτοις και εθεραπευσεν τους αρρωστους αυτων
			<A NAME=40N_14_15><B> 14,15 </B>οψιας δε γενομενης προσηλθον αυτω οι μαθηται λεγοντες ερημος εστιν ο τοπος και η ωρα παρηλθεν ηδη απολυσον ουν τους οχλους ινα απελθοντες εις τας κωμας αγορασωσιν εαυτοις βρωματα
			<A NAME=40N_14_16><B> 14,16 </B>ο δε ειπεν αυτοις ου χρειαν εχουσιν απελθειν δοτε αυτοις υμεις φαγειν
			<A NAME=40N_14_17><B> 14,17 </B>οι δε λεγουσιν αυτω ουκ εχομεν ωδε ει μη πεντε αρτους και δυο ιχθυας
			<A NAME=40N_14_18><B> 14,18 </B>ο δε ειπεν φερετε μοι ωδε αυτους
			<A NAME=40N_14_19><B> 14,19 </B>και κελευσας τους οχλους ανακλιθηναι επι του χορτου λαβων τους πεντε αρτους και τους δυο ιχθυας αναβλεψας εις τον ουρανον ευλογησεν και κλασας εδωκεν τοις μαθηταις τους αρτους οι δε μαθηται τοις οχλοις
			<A NAME=40N_14_20><B> 14,20 </B>και εφαγον παντες και εχορτασθησαν και ηραν το περισσευον των κλασματων δωδεκα κοφινους πληρεις
			<A NAME=40N_14_21><B> 14,21 </B>οι δε εσθιοντες ησαν ανδρες ωσει πεντακισχιλιοι χωρις γυναικων και παιδιων
			<A NAME=40N_14_22><B> 14,22 </B>και ηναγκασεν τους μαθητας εμβηναι εις το πλοιον και προαγειν αυτον εις το περαν εως ου απολυση τους οχλους
			<A NAME=40N_14_23><B> 14,23 </B>και απολυσας τους οχλους ανεβη εις το ορος κατ ιδιαν προσευξασθαι οψιας δε γενομενης μονος ην εκει
			<A NAME=40N_14_24><B> 14,24 </B>το δε πλοιον ηδη μεσον της θαλλασσης ην βασανιζομενον υπο των κυματων ην γαρ εναντιος ο ανεμος
			<A NAME=40N_14_25><B> 14,25 </B>τεταρτη δε φυλακη της νυκτος ηλθεν προς αυτους περιπατων επι την θαλασσαν
			<A NAME=40N_14_26><B> 14,26 </B>ιδοντες δε αυτον επι της θαλασσης περιπατουντα εταραχθησαν λεγοντες οτι φαντασμα εστιν και απο του φοβου εκραξαν
			<A NAME=40N_14_27><B> 14,27 </B>ευθυς δε ελαλησεν αυτοις λεγων θαρσειτε εγω ειμι μη φοβεισθε
			<A NAME=40N_14_28><B> 14,28 </B>αποκριθεις δε αυτω ο πετρος ειπεν κυριε ει συ ει κελευσον με ελθειν προς σε επι τα υδατα
			<A NAME=40N_14_29><B> 14,29 </B>ο δε ειπεν ελθε και καταβας απο του πλοιου πετρος περιεπατησεν επι τα υδατα και ηλθεν προς τον ιησουν
			<A NAME=40N_14_30><B> 14,30 </B>βλεπων δε τον ανεμον εφοβηθη και αρξαμενος καταποντιζεσθαι εκραξεν λεγων κυριε σωσον με
			<A NAME=40N_14_31><B> 14,31 </B>ευθεως δε ο ιησους εκτεινας την χειρα επελαβετο αυτου και λεγει αυτω ολιγοπιστε εις τι εδιστασας
			<A NAME=40N_14_32><B> 14,32 </B>και αναβαντων αυτων εις το πλοιον εκοπασεν ο ανεμος
			<A NAME=40N_14_33><B> 14,33 </B>οι δε εν τω πλοιω προσεκυνησαν αυτω λεγοντες αληθως θεου υιος ει
			<A NAME=40N_14_34><B> 14,34 </B>και διαπερασαντες ηλθον επι την γην εις γεννησαρετ
			<A NAME=40N_14_35><B> 14,35 </B>και επιγνοντες αυτον οι ανδρες του τοπου εκεινου απεστειλαν εις ολην την περιχωρον εκεινην και προσηνεγκαν αυτω παντας τους κακως εχοντας
			<A NAME=40N_14_36><B> 14,36 </B>και παρεκαλουν αυτον ινα μονον αψωνται του κρασπεδου του ιματιου αυτου και οσοι ηψαντο διεσωθησαν
			</P><P ALIGN= "CENTER"><A NAME=40N_15><H2>Matthäus<BR>Kapitel 15</H2></P><P ALIGN= "LEFT"><A NAME=40N_15_2><B> 15,2 </B>διατι οι μαθηται σου παραβαινουσιν την παραδοσιν των πρεσβυτερων ου γαρ νιπτονται τας χειρας οταν αρτον εσθιωσιν
			<A NAME=40N_15_3><B> 15,3 </B>ο δε αποκριθεις ειπεν αυτοις διατι και υμεις παραβαινετε την εντολην του θεου δια την παραδοσιν υμων
			<A NAME=40N_15_4><B> 15,4 </B>ο γαρ θεος εςετειλατο λεγων τιμα τον πατερα και την μητερα και ο κακολογων πατερα η μητερα θανατω τελευτατω
			<A NAME=40N_15_5><B> 15,5 </B>υμεις δε λεγετε ος αν ειπη τω πατρι η τη μητρι δωρον ο εαν εξ εμου ωφεληθης ου μη τιμησει τον πατερα αυτου η την μητερα αυτου
			<A NAME=40N_15_6><B> 15,6 </B>και ηκυρωσατε τον νομον του θεου δια την παραδοσιν υμων
			<A NAME=40N_15_7><B> 15,7 </B>υποκριται καλως επροφητευσεν περι υμων ησαιας λεγων
			<A NAME=40N_15_8><B> 15,8 </B>ο λαος ουτος τοις χειλεσιν με τιμα η δε καρδια αυτων πορρω απεχει απ εμου
			<A NAME=40N_15_9><B> 15,9 </B>ματην δε σεβονται με διδασκοντες διδασκαλιας ενταλματα ανθρωπων
			<A NAME=40N_15_10><B> 15,10 </B>και προσκαλεσαμενος τον οχλον ειπεν αυτοις ακουετε και συνιετε
			<A NAME=40N_15_11><B> 15,11 </B>ου το εισερχομενον εις το στομα κοινοι τον ανθρωπον αλλα το εκπορευομενον εκ του στοματος τουτο κοινοι τον ανθρωπον
			<A NAME=40N_15_12><B> 15,12 </B>τοτε προσελθοντες οι μαθηται λεγουσιν αυτω οιδας οτι οι φαρισαιοι ακουσαντες τον λογον εσκανδαλισθησαν
			<A NAME=40N_15_13><B> 15,13 </B>ο δε αποκριθεις ειπεν πασα φυτεια ην ουκ εφυτευσεν ο πατηρ μου ο ουρανιος εκριζωθησεται
			<A NAME=40N_15_14><B> 15,14 </B>αφετε αυτους οδηγοι εισιν τυφλοι τυφλων τυφλος δε τυφλον εαν οδηγη αμφοτεροι εις βοθυνον πεσουνται
			<A NAME=40N_15_15><B> 15,15 </B>αποκριθεις δε ο πετρος ειπεν αυτω φρασον ημιν την παραβολην
			<A NAME=40N_15_16><B> 15,16 </B>ο δε ειπεν ακμην και υμεις ασυνετοι εστε
			<A NAME=40N_15_17><B> 15,17 </B>ου νοειτε οτι παν το εισπορευομενον εις το στομα εις την κοιλιαν χωρει και εις αφεδρωνα εκβαλλεται
			<A NAME=40N_15_18><B> 15,18 </B>τα δε εκπορευομενα εκ του στοματος εκ της καρδιας εξερχεται κακεινα κοινοι τον ανθρωπον
			<A NAME=40N_15_19><B> 15,19 </B>εκ γαρ της καρδιας εξερχονται διαλογισμοι πονηροι φονοι μοιχειαι πορνειαι κλοπαι ψευδομαρτυριαι βλασφημιαι
			<A NAME=40N_15_20><B> 15,20 </B>ταυτα εστιν τα κοινουντα τον ανθρωπον το δε ανιπτοις χερσιν φαγειν ου κοινοι τον ανθρωπον
			<A NAME=40N_15_21><B> 15,21 </B>και εξελθων εκειθεν ο ιησους ανεχωρησεν εις τα μερη τυρου και σιδωνος
			<A NAME=40N_15_22><B> 15,22 </B>και ιδου γυνη χαναναια απο των οριων εκεινων εξελθουσα εκραζεν λεγουσα ελεησον με κυριε υιος δαυειδ η θυγατηρ μου κακως δαιμονιζεται
			<A NAME=40N_15_23><B> 15,23 </B>ο δε ουκ απεκριθη αυτη λογον και προσελθοντες οι μαθηται αυτου ηρωτουν αυτον λεγοντες απολυσον αυτην οτι κραζει οπισθεν ημων
			<A NAME=40N_15_24><B> 15,24 </B>ο δε αποκριθεις ειπεν ουκ απεσταλην ει μη εις τα προβατα τα απολωλοτα οικου ισραηλ
			<A NAME=40N_15_25><B> 15,25 </B>η δε ελθουσα προσεκυνει αυτω λεγουσα κυριε βοηθει μοι
			<A NAME=40N_15_26><B> 15,26 </B>ο δε αποκριθεις ειπεν ουκ εξεστιν λαβειν τον αρτον των τεκνων και βαλειν τοις κυναριοις
			<A NAME=40N_15_27><B> 15,27 </B>η δε ειπεν ναι κυριε και γαρ τα κυναρια εσθιει απο των ψιχιων των πιπτοντων απο της τραπεζης των κυριων αυτων
			<A NAME=40N_15_28><B> 15,28 </B>τοτε αποκριθεις ο ιησους ειπεν αυτη ω γυναι μεγαλη σου η πιστις γενηθητω σοι ως θελεις και ιαθη η θυγατηρ αυτης απο της ωρας εκεινης
			<A NAME=40N_15_29><B> 15,29 </B>και μεταβας εκειθεν ο ιησους ηλθεν παρα την θαλασσαν της γαλιλαιας και αναβας εις το ορος εκαθητο εκει
			<A NAME=40N_15_30><B> 15,30 </B>και προσηλθον αυτω οχλοι πολλοι εχοντες μεθ εαυτων χωλους τυφλους κωφους κυλλους και ετερους πολλους και εριψαν αυτους παρα τους ποδας αυτου και εθεραπευσεν αυτους
			<A NAME=40N_15_31><B> 15,31 </B>ωστε τον οχλον θαυμασαι βλεποντας κωφους λαλουντας κυλλους υγιεις και χωλους περιπατουντας και τυφλους βλεποντας και εδοξασαν τον θεον ισραηλ
			<A NAME=40N_15_32><B> 15,32 </B>ο δε ιησους προσκαλεσαμενος τους μαθητας αυτου ειπεν σπλαγχνιζομαι επι τον οχλον οτι ηδη ημεραι τρεις προσμενουσιν μοι και ουκ εχουσιν τι φαγωσιν και απολυσαι αυτους νηστεις ου θελω μηποτε εκλυθωσιν εν τη οδω
			<A NAME=40N_15_33><B> 15,33 </B>και λεγουσιν αυτω οι μαθηται ποθεν ημιν εν ερημια αρτοι τοσουτοι ωστε χορτασαι οχλον τοσουτον
			<A NAME=40N_15_34><B> 15,34 </B>και λεγει αυτοις ο ιησους ποσους αρτους εχετε οι δε ειπον επτα και ολιγα ιχθυδια
			<A NAME=40N_15_35><B> 15,35 </B>και παραγγειλας τω οχλω αναπεσειν επι την γην
			<A NAME=40N_15_36><B> 15,36 </B>ελαβεν τους επτα αρτους και τους ιχθυας και ευχαριστησας εκλασεν και εδιδου τοις μαθηταις οι δε μαθηται τοις οχλοις
			<A NAME=40N_15_37><B> 15,37 </B>και εφαγον παντες και εχορτασθησαν και το περισσευον των κλασματων ηραν επτα σπυριδας πληρεις
			<A NAME=40N_15_38><B> 15,38 </B>οι δε εσθιοντες ησαν τετρακισχιλιοι ανδρες χωρις παιδιων και γυναικων
			<A NAME=40N_15_39><B> 15,39 </B>και απολυσας τους οχλους ενεβη εις το πλοιον και ηλθεν εις τα ορια μαγαδαν
			</P><P ALIGN= "CENTER"><A NAME=40N_16><H2>Matthäus<BR>Kapitel 16</H2></P><P ALIGN= "LEFT"><A NAME=40N_16_2><B> 16,2 </B>ο δε αποκριθεις ειπεν αυτοις [οψιας γενομενης λεγετε ευδια πυρραζει γαρ ο ουρανος
			<A NAME=40N_16_3><B> 16,3 </B>και πρωι σημερον χειμων πυρραζει γαρ στυγναζων ο ουρανος το μεν προσωπον του ουρανου γινωσκετε διακρινειν τα δε σημεια των καιρων ου δυνασθε]
			<A NAME=40N_16_4><B> 16,4 </B>γενεα πονηρα και μοιχαλις σημειον επιζητει και σημειον ου δοθησεται αυτη ει μη το σημειον ιωνα και καταλιπων αυτους απηλθεν
			<A NAME=40N_16_5><B> 16,5 </B>και ελθοντες οι μαθηται εις το περαν επελαθοντο αρτους λαβειν
			<A NAME=40N_16_6><B> 16,6 </B>ο δε ιησους ειπεν αυτοις ορατε και προσεχετε απο της ζυμης των φαρισαιων και σαδδουκαιων
			<A NAME=40N_16_7><B> 16,7 </B>οι δε διελογιζοντο εν εαυτοις λεγοντες οτι αρτους ουκ ελαβομεν
			<A NAME=40N_16_8><B> 16,8 </B>γνους δε ο ιησους ειπεν τι διαλογιζεσθε εν εαυτοις ολιγοπιστοι οτι αρτους ουκ ελαβετε
			<A NAME=40N_16_9><B> 16,9 </B>ουπω νοειτε ουδε μνημονευετε τους πεντε αρτους των πεντακισχιλιων και ποσους κοφινους ελαβετε
			<A NAME=40N_16_10><B> 16,10 </B>ουδε τους επτα αρτους των τετρακισχιλιων και ποσας σπυριδας ελαβετε
			<A NAME=40N_16_11><B> 16,11 </B>πως ου νοειτε οτι ου περι αρτων ειπον υμιν προσεχετε δε απο της ζυμης των φαρισαιων και σαδδουκαιων
			<A NAME=40N_16_12><B> 16,12 </B>τοτε συνηκαν οτι ουκ ειπεν προσεχειν απο της ζυμης των φαρισαιων και σαδδουκιαν αλλα απο της διδαχης των φαρισαιων και σαδδουκιαν
			<A NAME=40N_16_13><B> 16,13 </B>ελθων δε ο ιησους εις τα μερη καισαρειας της φιλιππου ηρωτα τους μαθητας αυτου λεγων τινα λεγουσιν οι ανθρωποι ειναι τον υιον του ανθρωπου
			<A NAME=40N_16_14><B> 16,14 </B>οι δε ειπαν οι μεν ιωαννην τον βαπτιστην αλλοι δε ηλειαν ετεροι δε ιερεμιαν η ενα των προφητων
			<A NAME=40N_16_15><B> 16,15 </B>λεγει αυτοις υμεις δε τινα με λεγετε ειναι
			<A NAME=40N_16_16><B> 16,16 </B>αποκριθεις δε σιμων πετρος ειπεν συ ει ο χριστος ο υιος του θεου του ζωντος
			<A NAME=40N_16_17><B> 16,17 </B>αποκριθεις δε ο ιησους ειπεν αυτω μακαριος ει σιμων βαριωνα οτι σαρξ και αιμα ουκ απεκαλυψεν σοι αλλ ο πατηρ μου ο εν τοις ουρανοις
			<A NAME=40N_16_18><B> 16,18 </B>καγω δε σοι λεγω οτι συ ει πετρος και επι ταυτη τη πετρα οικοδομησω μου την εκκλησιαν και πυλαι αδου ου κατισχυσουσιν αυτης
			<A NAME=40N_16_19><B> 16,19 </B>δωσω σοι τας κλειδας της βασιλειας των ουρανων και ο εαν δησης επι της γης εσται δεδεμενον εν τοις ουρανοις και ο εαν λυσης επι της γης εσται λελυμενον εν τοις ουρανοις
			<A NAME=40N_16_20><B> 16,20 </B>τοτε διεστειλατο τοις μαθηταις ινα μηδενι ειπωσιν οτι αυτος εστιν ο χριστος
			<A NAME=40N_16_21><B> 16,21 </B>απο τοτε ηρξατο ο ιησους δεικνυειν τοις μαθηταις αυτου οτι δει αυτον εις ιεροσολυμα απελθειν και πολλα παθειν απο των πρεσβυτερων και αρχιερεων και γραμματεων και αποκτανθηναι και τη τριτη ημερα εγερθηναι
			<A NAME=40N_16_22><B> 16,22 </B>και προσλαβομενος αυτον ο πετρος ηρξατο επιτιμαν αυτω λεγων ιλεως σοι κυριε ου μη εσται σοι τουτο
			<A NAME=40N_16_23><B> 16,23 </B>ο δε στραφεις ειπεν τω πετρω υπαγε οπισω μου σατανα σκανδαλον ει εμου οτι ου φρονεις τα του θεου αλλα τα των ανθρωπων
			<A NAME=40N_16_24><B> 16,24 </B>τοτε ο ιησους ειπεν τοις μαθηταις αυτου ει τις θελει οπισω μου ελθειν απαρνησασθω εαυτον και αρατω τον σταυρον αυτου και ακολουθειτω μοι
			<A NAME=40N_16_25><B> 16,25 </B>ος γαρ εαν θελη την ψυχην αυτου σωσαι απολεσει αυτην ος δ αν απολεση την ψυχην αυτου ενεκεν εμου ευρησει αυτην
			<A NAME=40N_16_26><B> 16,26 </B>τι γαρ ωφεληθησεται ανθρωπος εαν τον κοσμον ολον κερδηση την δε ψυχην αυτου ζημιωθη η τι δωσει ανθρωπος ανταλλαγμα της ψυχης αυτου
			<A NAME=40N_16_27><B> 16,27 </B>μελλει γαρ ο υιος του ανθρωπου ερχεσθαι εν τη δοξη του πατρος αυτου μετα των αγγελων αυτου και τοτε αποδωσει εκαστω κατα την πραξιν αυτου
			<A NAME=40N_16_28><B> 16,28 </B>αμην λεγω υμιν οτι εισιν τινες των ωδε εστωτων οιτινες ου μη γευσωνται θανατου εως αν ιδωσιν τον υιον του ανθρωπου ερχομενον εν τη βασιλεια αυτου
			</P><P ALIGN= "CENTER"><A NAME=40N_17><H2>Matthäus<BR>Kapitel 17</H2></P><P ALIGN= "LEFT"><A NAME=40N_17_2><B> 17,2 </B>και μετεμορφωθη εμπροσθεν αυτων και ελαμψεν το προσωπον αυτου ως ο ηλιος τα δε ιματια αυτου εγενετο λευκα ως το φως
			<A NAME=40N_17_3><B> 17,3 </B>και ιδου ωφθη αυτοις μωυσης και ηλειας συνλαλουντες μετ αυτου
			<A NAME=40N_17_4><B> 17,4 </B>αποκριθεις δε ο πετρος ειπεν τω ιησου κυριε καλον εστιν ημας ωδε ειναι ει θελεις ποιησω ωδε τρεις σκηνας σοι μιαν και μωυσει μιαν και ηλεια μιαν
			<A NAME=40N_17_5><B> 17,5 </B>ετι αυτου λαλουντος ιδου νεφελη φωτεινη επεσκιασεν αυτους και ιδου φωνη εκ της νεφελης λεγουσα ουτος εστιν ο υιος μου ο αγαπητος εν ω ευδοκησα ακουετε αυτου
			<A NAME=40N_17_6><B> 17,6 </B>και ακουσαντες οι μαθηται επεσαν επι προσωπον αυτων και εφοβηθησαν σφοδρα
			<A NAME=40N_17_7><B> 17,7 </B>και προσηλθεν ο ιησους και αψαμενος αυτων ειπεν εγερθητε και μη φοβεισθε
			<A NAME=40N_17_8><B> 17,8 </B>επαραντες δε τους οφθαλμους αυτων ουδενα ειδον ει μη τον ιησουν μονον
			<A NAME=40N_17_9><B> 17,9 </B>και καταβαινοντων αυτων εκ του ορους ενετειλατο αυτοις ο ιησους λεγων μηδενι ειπητε το οραμα εως ου ο υιος του ανθρωπου εκ νεκρων εγερθη
			<A NAME=40N_17_10><B> 17,10 </B>και επηρωτησαν αυτον οι μαθηται λεγοντες τι ουν οι γραμματεις λεγουσιν οτι ηλειαν δει ελθειν πρωτον
			<A NAME=40N_17_11><B> 17,11 </B>ο δε αποκριθεις ειπεν ηλειας μεν ερχεται και αποκαταστησει παντα
			<A NAME=40N_17_12><B> 17,12 </B>λεγω δε υμιν οτι ηλειας ηδη ηλθεν και ουκ επεγνωσαν αυτον αλλα εποιησαν εν αυτω οσα ηθελησαν ουτως και ο υιος του ανθρωπου μελλει πασχειν υπ αυτων
			<A NAME=40N_17_13><B> 17,13 </B>τοτε συνηκαν οι μαθηται οτι περι ιωαννου του βαπτιστου ειπεν αυτοις
			<A NAME=40N_17_14><B> 17,14 </B>και ελθοντων προς τον οχλον προσηλθεν αυτω ανθρωπος γονυπετων αυτον
			<A NAME=40N_17_15><B> 17,15 </B>και λεγων κυριε ελεησον μου τον υιον οτι σεληνιαζεται και κακως πασχει πολλακις γαρ πιπτει εις το πυρ και πολλακις εις το υδωρ
			<A NAME=40N_17_16><B> 17,16 </B>και προσηνεγκα αυτον τοις μαθηταις σου και ουκ ηδυνηθησαν αυτον θεραπευσαι
			<A NAME=40N_17_17><B> 17,17 </B>αποκριθεις δε ο ιησους ειπεν ω γενεα απιστος και διεστραμμενη εως ποτε μεθ υμων εσομαι εως ποτε ανεξομαι υμων φερετε μοι αυτον ωδε
			<A NAME=40N_17_18><B> 17,18 </B>και επετιμησεν αυτω ο ιησους και εξηλθεν απ αυτου το δαιμονιον και εθεραπευθη ο παις απο της ωρας εκεινης
			<A NAME=40N_17_19><B> 17,19 </B>τοτε προσελθοντες οι μαθηται τω ιησου κατ ιδιαν ειπον δια τι ημεις ουκ ηδυνηθημεν εκβαλειν αυτο
			<A NAME=40N_17_20><B> 17,20 </B>ο δε λεγει αυτοις δια την ολιγοπιστιαν υμων αμην γαρ λεγω υμιν εαν εχητε πιστιν ως κοκκον σιναπεως ερειτε τω ορει τουτω μεταβα ενθεν εκει και μεταβησεται και ουδεν αδυνατησει υμιν
			<A NAME=40N_17_21><B> 17,21 </B>
			<A NAME=40N_17_22><B> 17,22 </B>συστρεφομενων δε αυτων εν τη γαλιλαια ειπεν αυτοις ο ιησους μελλει ο υιος του ανθρωπου παραδιδοσθαι εις χειρας ανθρωπων
			<A NAME=40N_17_23><B> 17,23 </B>και αποκτενουσιν αυτον και τη τριτη ημερα εγερθησεται και ελυπηθησαν σφοδρα
			<A NAME=40N_17_24><B> 17,24 </B>ελθοντων δε αυτων εις καφαρναουμ προσηλθον οι τα διδραχμα λαμβανοντες τω πετρω και ειπαν ο διδασκαλος υμων ου τελει διδραχμα
			<A NAME=40N_17_25><B> 17,25 </B>λεγει ναι και εισελθοντα εις την οικιαν προεφθασεν αυτον ο ιησους λεγων τι σοι δοκει σιμων οι βασιλεις της γης απο τινων λαμβανουσιν τελη η κηνσον απο των υιων αυτων η απο των αλλοτριων
			<A NAME=40N_17_26><B> 17,26 </B>ειποντος δε απο των αλλοτριων εφη αυτω ο ιησους αραγε ελευθεροι εισιν οι υιοι
			<A NAME=40N_17_27><B> 17,27 </B>ινα δε μη σκανδαλισωμεν αυτους πορευθεις εις θαλασσαν βαλε αγκιστρον και τον αναβαντα πρωτον ιχθυν αρον και ανοιξας το στομα αυτου ευρησεις στατηρα εκεινον λαβων δος αυτοις αντι εμου και σου
			</P><P ALIGN= "CENTER"><A NAME=40N_18><H2>Matthäus<BR>Kapitel 18</H2></P><P ALIGN= "LEFT"><A NAME=40N_18_2><B> 18,2 </B>και προσκαλεσαμενος παιδιον εστησεν αυτο εν μεσω αυτων
			<A NAME=40N_18_3><B> 18,3 </B>και ειπεν αμην λεγω υμιν εαν μη στραφητε και γενησθε ως τα παιδια ου μη εισελθητε εις την βασιλειαν των ουρανων
			<A NAME=40N_18_4><B> 18,4 </B>οστις ουν ταπεινωσει εαυτον ως το παιδιον τουτο ουτος εστιν ο μειζων εν τη βασιλεια των ουρανων
			<A NAME=40N_18_5><B> 18,5 </B>και ος εαν δεξηται εν παιδιον τοιουτο επι τω ονοματι μου εμε δεχεται
			<A NAME=40N_18_6><B> 18,6 </B>ος δ αν σκανδαλιση ενα των μικρων τουτων των πιστευοντων εις εμε συμφερει αυτω ινα κρεμασθη μυλος ονικος περι τον τραχηλον αυτου και καταποντισθη εν τω πελαγει της θαλασσης
			<A NAME=40N_18_7><B> 18,7 </B>ουαι τω κοσμω απο των σκανδαλων αναγκη γαρ εστιν ελθειν τα σκανδαλα πλην ουαι τω ανθρωπω δι ου το σκανδαλον ερχεται
			<A NAME=40N_18_8><B> 18,8 </B>ει δε η χειρ σου η ο πους σου σκανδαλιζει σε εκκοψον αυτον και βαλε απο σου καλον σοι εστιν εισελθειν εις την ζωην κυλλον η χωλον η δυο χειρας η δυο ποδας εχοντα βληθηναι εις το πυρ το αιωνιον
			<A NAME=40N_18_9><B> 18,9 </B>και ει ο οφθαλμος σου σκανδαλιζει σε εξελε αυτον και βαλε απο σου καλον σοι εστιν μονοφθαλμον εις την ζωην εισελθειν η δυο οφθαλμους εχοντα βληθηναι εις την γεενναν του πυρος
			<A NAME=40N_18_10><B> 18,10 </B>ορατε μη καταφρονησητε ενος των μικρων τουτων λεγω γαρ υμιν οτι οι αγγελοι αυτων εν ουρανοις δια παντος βλεπουσιν το προσωπον του πατρος μου του εν ουρανοις
			<A NAME=40N_18_11><B> 18,11 </B>
			<A NAME=40N_18_12><B> 18,12 </B>τι υμιν δοκει εαν γενηται τινι ανθρωπω εκατον προβατα και πλανηθη εν εξ αυτων ουχι αφησει τα ενενηκοντα εννεα επι τα ορη και πορευθεις ζητει το πλανωμενον
			<A NAME=40N_18_13><B> 18,13 </B>και εαν γενηται ευρειν αυτο αμην λεγω υμιν οτι χαιρει επ αυτω μαλλον η επι τοις ενενηκοντα εννεα τοις μη πεπλανημενοις
			<A NAME=40N_18_14><B> 18,14 </B>ουτως ουκ εστιν θελημα εμπροσθεν του πατρος υμων του εν ουρανοις ινα αποληται εν των μικρων τουτων
			<A NAME=40N_18_15><B> 18,15 </B>εαν δε αμαρτηση ο αδελφος σου υπαγε ελεγξον αυτον μεταξυ σου και αυτου μονου εαν σου ακουση εκερδησας τον αδελφον σου
			<A NAME=40N_18_16><B> 18,16 </B>εαν δε μη ακουση παραλαβε μετα σεατου ετι ενα η δυο ινα επι στοματος δυο μαρτυρων η τριων σταθη παν ρημα
			<A NAME=40N_18_17><B> 18,17 </B>εαν δε παρακουση αυτων ειπον τη εκκλησια εαν δε και της εκκλησιας παρακουση εστω σοι ωσπερ ο εθνικος και ο τελωνης
			<A NAME=40N_18_18><B> 18,18 </B>αμην λεγω υμιν οσα εαν δησητε επι της γης εσται δεδεμενα εν ουρανω και οσα εαν λυσητε επι της γης εσται λελυμενα εν ουρανω
			<A NAME=40N_18_19><B> 18,19 </B>παλιν λεγω υμιν οτι εαν δυο συμφωνησουσιν εξ υμων επι της γης περι παντος πραγματος ου εαν αιτησωνται γενησεται αυτοις παρα του πατρος μου του εν ουρανοις
			<A NAME=40N_18_20><B> 18,20 </B>ου γαρ εισιν δυο η τρεις συνηγμενοι εις το εμον ονομα εκει ειμι εν μεσω αυτων
			<A NAME=40N_18_21><B> 18,21 </B>τοτε προσελθων ο πετρος ειπεν αυτω κυριε ποσακις αμαρτησει εις εμε ο αδελφος μου και αφησω αυτω εως επτακις
			<A NAME=40N_18_22><B> 18,22 </B>λεγει αυτω ο ιησους ου λεγω σοι εως επτακις αλλα εως εβδομηκοντακις επτα
			<A NAME=40N_18_23><B> 18,23 </B>δια τουτο ωμοιωθη η βασιλεια των ουρανων ανθρωπω βασιλει ος ηθελησεν συναραι λογον μετα των δουλων αυτου
			<A NAME=40N_18_24><B> 18,24 </B>αρξαμενου δε αυτου συναιρειν προσηνεχθη εις αυτω οφειλετης μυριων ταλαντων
			<A NAME=40N_18_25><B> 18,25 </B>μη εχοντος δε αυτου αποδουναι εκελευσεν αυτον ο κυριος πραθηναι και την γυναικα και τα τεκνα και παντα οσα ειχεν και αποδοθηναι
			<A NAME=40N_18_26><B> 18,26 </B>πεσων ουν ο δουλος εκεινος προσεκυνει αυτω λεγων μακροθυμησον επ εμοι και παντα αποδωσω σοι
			<A NAME=40N_18_27><B> 18,27 </B>σπλαγχνισθεις δε ο κυριος του δουλου εκεινου απελυσεν αυτον και το δανειον αφηκεν αυτω
			<A NAME=40N_18_28><B> 18,28 </B>εξελθων δε ο δουλος εκεινος ευρεν ενα των συνδουλων αυτου ος ωφειλεν αυτω εκατον δηναρια και κρατησας αυτον επνιγεν λεγων αποδος ει τι οφειλεις
			<A NAME=40N_18_29><B> 18,29 </B>πεσων ουν ο συνδουλος αυτου παρεκαλει αυτον λεγων μακροθυμησον επ εμοι και αποδωσω σοι
			<A NAME=40N_18_30><B> 18,30 </B>ο δε ουκ ηθελεν αλλα απελθων εβαλεν αυτον εις φυλακην εως αποδω το οφειλομενον
			<A NAME=40N_18_31><B> 18,31 </B>ιδοντες ουν οι συνδουλοι αυτου τα γενομενα ελυπηθησαν σφοδρα και ελθοντες διεσαφησαν τω κυριω εαυτων παντα τα γενομενα
			<A NAME=40N_18_32><B> 18,32 </B>τοτε προσκαλεσαμενος αυτον ο κυριος αυτου λεγει αυτω δουλε πονηρε πασαν την οφειλην εκεινην αφηκα σοι επει παρεκαλεσας με
			<A NAME=40N_18_33><B> 18,33 </B>ουκ εδει και σε ελεησαι τον συνδουλον σου ως καγω σε ηλεησα
			<A NAME=40N_18_34><B> 18,34 </B>και οργισθεις ο κυριος αυτου παρεδωκεν αυτον τοις βασανισταις εως ου αποδω παν το οφειλομενον αυτω
			<A NAME=40N_18_35><B> 18,35 </B>ουτως και ο πατηρ μου ο ουρανιος ποιησει υμιν εαν μη αφητε εκαστος τω αδελφω αυτου απο των καρδιων υμων
			</P><P ALIGN= "CENTER"><A NAME=40N_19><H2>Matthäus<BR>Kapitel 19</H2></P><P ALIGN= "LEFT"><A NAME=40N_19_2><B> 19,2 </B>και ηκολουθησαν αυτω οχλοι πολλοι και εθεραπευσεν αυτους εκει
			<A NAME=40N_19_3><B> 19,3 </B>και προσηλθον αυτω οι φαρισαιοι πειραζοντες αυτον και λεγοντες ει εξεστιν απολυσαι την γυναικα αυτου κατα πασαν αιτιαν
			<A NAME=40N_19_4><B> 19,4 </B>ο δε αποκριθεις ειπεν ουκ ανεγνωτε οτι ο ποισας απ αρχης αρσεν και θηλυ εποιησεν αυτους
			<A NAME=40N_19_5><B> 19,5 </B>και ειπεν ενεκα τουτου καταλειψει ανθρωπος τον πατερα και την μητερα και κολληθησεται τη γυναικι αυτου και εσονται οι δυο εις σαρκα μιαν
			<A NAME=40N_19_6><B> 19,6 </B>ωστε ουκετι εισιν δυο αλλα σαρξ μια ο ουν ο θεος συνεζευξεν ανθρωπος μη χωριζετω
			<A NAME=40N_19_7><B> 19,7 </B>λεγουσιν αυτω τι ουν μωυσης ενετειλατο δουναι βιβλιον αποστασιου και απολυσαι
			<A NAME=40N_19_8><B> 19,8 </B>λεγει αυτοις οτι μωυσης προς την σκληροκαρδιαν υμων επετρεψεν υμιν απολυσαι τας γυναικας υμων απ αρχης δε ου γεγονεν ουτως
			<A NAME=40N_19_9><B> 19,9 </B>λεγω δε υμιν οτι ος αν απολυση την γυναικα αυτου μη επι πορνεια και γαμηση αλλην μοιχαται
			<A NAME=40N_19_10><B> 19,10 </B>λεγουσιν αυτω οι μαθηται ει ουτως εστιν η αιτια του ανθρωπου μετα της γυναικος ου συμφερει γαμησαι
			<A NAME=40N_19_11><B> 19,11 </B>ο δε ειπεν αυτοις ου παντες χωρουσιν τον λογον τουτον αλλ οις δεδοται
			<A NAME=40N_19_12><B> 19,12 </B>εισιν γαρ ευνουχοι οιτινες εκ κοιλιας μητρος εγεννηθησαν ουτως και εισιν ευνουχοι οιτινες ευνουχισθησαν υπο των ανθρωπων και εισιν ευνουχοι οιτινες ευνουχισαν εαυτους δια την βασιλειαν των ουρανων ο δυναμενος χωρειν χωρειτω
			<A NAME=40N_19_13><B> 19,13 </B>τοτε προσηνεχθησαν αυτω παιδια ινα τας χειρας επιθη αυτοις και προσευξηται οι δε μαθηται επετιμησαν αυτοις
			<A NAME=40N_19_14><B> 19,14 </B>ο δε ιησους ειπεν αυτοις αφετε τα παιδια και μη κωλυετε αυτα ελθειν προς εμε των γαρ τοιουτων εστιν η βασιλεια των ουρανων
			<A NAME=40N_19_15><B> 19,15 </B>και επιθεις τας χειρας αυτοις επορευθη εκειθεν
			<A NAME=40N_19_16><B> 19,16 </B>και ιδου εις προσελθων αυτω ειπεν διδασκαλε τι αγαθον ποιησω ινα σχω ζωην αιωνιον
			<A NAME=40N_19_17><B> 19,17 </B>ο δε ειπεν αυτω τι με ερωτας περι του αγαθου εις εστιν ο αγαθος ει δε θελεις εις την ζωην εισελθειν τηρησον τας εντολας
			<A NAME=40N_19_18><B> 19,18 </B>ποιας φησιν ο δε ιησους ειπεν το ου φονευσεις ου μοιχευσεις ου κλεψεις ου ψευδομαρτυρησεις
			<A NAME=40N_19_19><B> 19,19 </B>τιμα τον πατερα και την μητερα και αγαπησεις τον πλησιον σου ως σεαυτον
			<A NAME=40N_19_20><B> 19,20 </B>λεγει αυτω ο νεανισκος παντα ταυτα εφυλαξα τι ετι υστερω
			<A NAME=40N_19_21><B> 19,21 </B>εφη αυτω ο ιησους ει θελεις τελειος ειναι υπαγε πωλησον σου τα υπαρχοντα και δος πτωχοις και εξεις θησαυρον εν ουρανοις και δευρο ακολουθει μοι
			<A NAME=40N_19_22><B> 19,22 </B>ακουσας δε ο νεανισκος απηλθεν λυπουμενος ην γαρ εχων κτηματα πολλα
			<A NAME=40N_19_23><B> 19,23 </B>ο δε ιησους ειπεν τοις μαθηταις αυτου αμην λεγω υμιν οτι πλουσιος δυσκολως εισελευσεται εις την βασιλειαν των ουρανων
			<A NAME=40N_19_24><B> 19,24 </B>παλιν δε λεγω υμιν οτι ευκοπωτερον εστιν καμηλον δια τρυπηματος ραφιδος εισελθειν η πλουσιον εις την βασιλειαν των ουρανων
			<A NAME=40N_19_25><B> 19,25 </B>ακουσαντες δε οι μαθηται εξεπλησσοντο σφοδρα λεγοντες τις αρα δυναται σωθηναι
			<A NAME=40N_19_26><B> 19,26 </B>εμβλεψας δε ο ιησους ειπεν αυτοις παρα ανθρωποις τουτο αδυνατον εστιν παρα δε θεω δυνατα παντα
			<A NAME=40N_19_27><B> 19,27 </B>τοτε αποκριθεις ο πετρος ειπεν αυτω ιδου ημεις αφηκαμεν παντα και ηκολουθησαμεν σοι τι αρα εσται ημιν
			<A NAME=40N_19_28><B> 19,28 </B>ο δε ιησους ειπεν αυτοις αμην λεγω υμιν οτι υμεις οι ακολουθησαντες μοι εν τη παλινγενεσια οταν καθιση ο υιος του ανθρωπου επι θρονου δοξης αυτου καθισεσθε και αυτοι επι δωδεκα θρονους κρινοντες τας δωδεκα φυλας του ισραηλ
			<A NAME=40N_19_29><B> 19,29 </B>και πας οστις αφηκεν αδελφους η αδελφας η πατερα η μητερα η τεκνα η αγρους η οικιας ενεκα του εμου ονοματος πολλαπλασιονα λημψεται και ζωην αιωνιον κληρονομησει
			<A NAME=40N_19_30><B> 19,30 </B>πολλοι δε εσονται πρωτοι εσχατοι και εσχατοι πρωτοι
			</P><P ALIGN= "CENTER"><A NAME=40N_20><H2>Matthäus<BR>Kapitel 20</H2></P><P ALIGN= "LEFT"><A NAME=40N_20_2><B> 20,2 </B>συμφωνησας δε μετα των εργατων εκ δηναριου την ημεραν απεστειλεν αυτους εις τον αμπελωνα αυτου
			<A NAME=40N_20_3><B> 20,3 </B>και εξελθων περι τριτην ωραν ειδεν αλλους εστωτας εν τη αγορα αργους
			<A NAME=40N_20_4><B> 20,4 </B>και εκεινοις ειπεν υπαγετε και υμεις εις τον αμπελωνα και ο εαν η δικαιον δωσω υμιν
			<A NAME=40N_20_5><B> 20,5 </B>οι δε απηλθον παλιν δε εξελθων περι εκτην και ενατην ωραν εποιησεν ωσαυτως
			<A NAME=40N_20_6><B> 20,6 </B>περι δε την ενδεκατην εξελθων ευρεν αλλους εστωτας και λεγει αυτοις τι ωδε εστηκατε ολην την ημεραν αργοι
			<A NAME=40N_20_7><B> 20,7 </B>λεγουσιν αυτω οτι ουδεις ημας εμισθωσατο λεγει αυτοις υπαγετε και υμεις εις τον αμπελωνα
			<A NAME=40N_20_8><B> 20,8 </B>οψιας δε γενομενης λεγει ο κυριος του αμπελωνος τω επιτροπω αυτου καλεσον τους εργατας και αποδος τον μισθον αρξαμενος απο των εσχατων εως των πρωτων
			<A NAME=40N_20_9><B> 20,9 </B>και ελθοντες οι περι την ενδεκατην ωραν ελαβον ανα δηναριον
			<A NAME=40N_20_10><B> 20,10 </B>ελθοντες δε οι πρωτοι ενομισαν οτι πλειον λημψονται και ελαβον το ανα δηναριον και αυτοι
			<A NAME=40N_20_11><B> 20,11 </B>λαβοντες δε εγογγυζον κατα του οικοδεσποτου
			<A NAME=40N_20_12><B> 20,12 </B>λεγοντες ουτοι οι εσχατοι μιαν ωραν εποιησαν και ισους αυτους ημιν εποιησας τοις βαστασασι το βαρος της ημερας και τον καυσωνα
			<A NAME=40N_20_13><B> 20,13 </B>ο δε αποκριθεις ενι αυτων ειπεν εταιρε ουκ αδικω σε ουχι δηναριου συνεφωνησας μοι
			<A NAME=40N_20_14><B> 20,14 </B>αρον το σον και υπαγε θελω δε τουτω τω εσχατω δουναι ως και σοι
			<A NAME=40N_20_15><B> 20,15 </B>η ουκ εξεστιν μοι ο θελω ποιησαι εν τοις εμοις η ο οφθαλμος σου πονηρος εστιν οτι εγω αγαθος ειμι
			<A NAME=40N_20_16><B> 20,16 </B>ουτως εσονται οι εσχατοι πρωτοι και οι πρωτοι εσχατοι
			<A NAME=40N_20_17><B> 20,17 </B>και αναβαινων ο ιησους εις ιεροσολυμα παρελαβεν τους δωδεκα κατ ιδιαν και εν τη οδω ειπεν αυτοις
			<A NAME=40N_20_18><B> 20,18 </B>ιδου αναβαινομεν εις ιεροσολυμα και ο υιος του ανθρωπου παραδοθησεται τοις αρχιερευσιν και γραμματευσιν και κατακρινουσιν αυτον εις θανατον
			<A NAME=40N_20_19><B> 20,19 </B>και παραδωσουσιν αυτον τοις εθνεσιν εις το εμπαιξαι και μαστιγωσαι και σταυρωσαι και τη τριτη ημερα εγερθησεται
			<A NAME=40N_20_20><B> 20,20 </B>τοτε προσηλθεν αυτω η μητηρ των υιων ζεβεδαιου μετα των υιων αυτης προσκυνουσα και αιτουσα τι παρ αυτου
			<A NAME=40N_20_21><B> 20,21 </B>ο δε ειπεν αυτη τι θελεις λεγει αυτω ειπε ινα καθισωσιν ουτοι οι δυο υιοι μου εις εκ δεξιων και εις εξ ευωνυμων σου εν τη βασιλεια σου
			<A NAME=40N_20_22><B> 20,22 </B>αποκριθεις δε ο ιησους ειπεν ουκ οιδατε τι αιτεισθε δυνασθε πιειν το ποτηριον ο εγω μελλω πινειν λεγουσιν αυτω δυναμεθα
			<A NAME=40N_20_23><B> 20,23 </B>λεγει αυτοις το μεν ποτηριον μου πιεσθε το δε καθισαι εκ δεξιων μου και εξ ευωνυμων ουκ εστιν εμον τουτο δουναι αλλ οις ητοιμασται υπο του πατρος μου
			<A NAME=40N_20_24><B> 20,24 </B>ακουσαντες δε οι δεκα ηγανακτησαν περι των δυο αδελφων
			<A NAME=40N_20_25><B> 20,25 </B>ο δε ιησους προσκαλεσαμενος αυτους ειπεν οιδατε οτι οι αρχοντες των εθνων κατακυριευουσιν αυτων και οι μεγαλοι κατεξουσιαζουσιν αυτων
			<A NAME=40N_20_26><B> 20,26 </B>ουχ ουτως εσται εν υμιν αλλ ος εαν θελη εν υμιν μεγας γενεσθαι εσται υμων διακονος
			<A NAME=40N_20_27><B> 20,27 </B>και ος αν θελη εν υμιν ειναι πρωτος εσται υμων δουλος
			<A NAME=40N_20_28><B> 20,28 </B>ωσπερ ο υιος του ανθρωπου ουκ ηλθεν διακονηθηναι αλλα διακονησαι και δουναι την ψυχην αυτου λυτρον αντι πολλων
			<A NAME=40N_20_29><B> 20,29 </B>και εκπορευομενων αυτων απο ιερειχω ηκολουθησεν αυτω οχλος πολυς
			<A NAME=40N_20_30><B> 20,30 </B>και ιδου δυο τυφλοι καθημενοι παρα την οδον ακουσαντες οτι ιησους παραγει εκραξαν λεγοντες ελεησον ημας υιε δαυειδ
			<A NAME=40N_20_31><B> 20,31 </B>ο δε οχλος επετιμησεν αυτοις ινα σιωπησωσιν οι δε μειζον εκραξαν λεγοντες κυριε ελεησον ημας υιε δαυειδ
			<A NAME=40N_20_32><B> 20,32 </B>και στας ο ιησους εφωνησεν αυτους και ειπεν τι θελετε ποιησω υμιν
			<A NAME=40N_20_33><B> 20,33 </B>λεγουσιν αυτω κυριε ινα ανοιγωσιν οι οφθαλμοι ημων
			<A NAME=40N_20_34><B> 20,34 </B>σπλαγχνισθεις δε ο ιησους ηψατο των ομματων αυτων και ευθεως ανεβλεψαν και ηκολουθησαν αυτω
			</P><P ALIGN= "CENTER"><A NAME=40N_21><H2>Matthäus<BR>Kapitel 21</H2></P><P ALIGN= "LEFT"><A NAME=40N_21_2><B> 21,2 </B>λεγων αυτοις πορευεσθε εις την κωμην την κατεναντι υμων και ευθυς ευρησετε ονον δεδεμενην και πωλον μετ αυτης λυσαντες αγαγετε μοι
			<A NAME=40N_21_3><B> 21,3 </B>και εαν τις υμιν ειπη τι ερειτε οτι ο κυριος αυτων χρειαν εχει ευθυς δε αποστελει αυτους
			<A NAME=40N_21_4><B> 21,4 </B>τουτο δε γεγονεν ινα πληρωθη το ρηθεν δια του προφητου λεγοντος
			<A NAME=40N_21_5><B> 21,5 </B>ειπατε τη θυγατρι σιων ιδου ο βασιλευς σου ερχεται σοι πραυς και επιβεβηκως επι ονον και επι πωλον υιον υποζυγιου
			<A NAME=40N_21_6><B> 21,6 </B>πορευθεντες δε οι μαθηται και ποιησαντες καθως προσετεαξεν αυτοις ο ιησους
			<A NAME=40N_21_7><B> 21,7 </B>ηγαγον την ονον και τον πωλον και επεθηκαν επ αυτων τα ιματια και επεκαθισεν επανω αυτων
			<A NAME=40N_21_8><B> 21,8 </B>ο δε πλειστος οχλος εστρωσαν εαυτων τα ιματια εν τη οδω αλλοι δε εκοπτον κλαδους απο των δενδρων και εστρωσαν εν τη οδω
			<A NAME=40N_21_9><B> 21,9 </B>οι δε οχλοι οι προαγοντες αυτον και οι ακολουθουντες εκραζον λεγοντες ωσαννα τω υιω δαυειδ ευλογημενος ο ερχομενος εν ονοματι κυριου ωσαννα εν τοις υψιστοις
			<A NAME=40N_21_10><B> 21,10 </B>και εισελθοντος αυτου εις ιεροσολυμα εσεισθη πασα η πολις λεγουσα τις εστιν ουτος
			<A NAME=40N_21_11><B> 21,11 </B>οι δε οχλοι ελεγον ουτος εστιν ο προφητης ιησους ο απο ναζαρεθ της γαλιλαιας
			<A NAME=40N_21_12><B> 21,12 </B>και εισηλθεν ιησους εις το ιερον του θεου και εξεβαλεν παντας τους πωλουντας και αγοραζοντας εν τω ιερω και τας τραπεζας των κολλυβιστων κατεστρεψεν και τας καθεδρας των πωλουντων τας περιστερας
			<A NAME=40N_21_13><B> 21,13 </B>και λεγει αυτοις γεγραπται ο οικος μου οικος προσευχης κληθησεται υμεις δε αυτον ποιειτε σπηλαιον ληστων
			<A NAME=40N_21_14><B> 21,14 </B>και προσηλθον αυτω τυφλοι και χωλοι εν τω ιερω και εθεραπευσεν αυτους
			<A NAME=40N_21_15><B> 21,15 </B>ιδοντες δε οι αρχιερεις και οι γραμματεις τα θαυμασια α εποιησεν και τους παιδας τους κραζοντας εν τω ιερω και λεγοντας ωσαννα τω υιω δαυειδ ηγανακτησαν
			<A NAME=40N_21_16><B> 21,16 </B>και ειπαν αυτω ακουεις τι ουτοι λεγουσιν ο δε ιησους λεγει αυτοις ναι ουδεποτε ανεγνωτε οτι εκ στοματος νηπιων και θηλαζοντων κατηρτισω αινον
			<A NAME=40N_21_17><B> 21,17 </B>και καταλιπων αυτους εξηλθεν εξω της πολεως εις βηθανιαν και ηυλισθη εκει
			<A NAME=40N_21_18><B> 21,18 </B>πρωι δε επαναγαγων εις την πολιν επεινασεν
			<A NAME=40N_21_19><B> 21,19 </B>και ιδων συκην μιαν επι της οδου ηλθεν επ αυτην και ουδεν ευρεν εν αυτη ει μη φυλλα μονον και λεγει αυτη ου μηκετι εκ σου καρπος γενηται εις τον αιωνα και εξηρανθη παραχρημα η συκη
			<A NAME=40N_21_20><B> 21,20 </B>και ιδοντες οι μαθηται εθαυμασαν λεγοντες πως παραχρημα εξηρανθη η συκη
			<A NAME=40N_21_21><B> 21,21 </B>αποκριθεις δε ο ιησους ειπεν αυτοις αμην λεγω υμιν εαν εχητε πιστιν και μη διακριθητε ου μονον το της συκης ποιησετε αλλα καν τω ορει τουτω ειπητε αρθητι και βληθητι εις την θαλασσαν γενησεται
			<A NAME=40N_21_22><B> 21,22 </B>και παντα οσα αν αιτησητε εν τη προσευχη πιστευοντες λημψεσθε
			<A NAME=40N_21_23><B> 21,23 </B>και ελθοντος αυτου εις το ιερον προσηλθον αυτω διδασκοντι οι αρχιερεις και οι πρεσβυτεροι του λαου λεγοντες εν ποια εξουσια ταυτα ποιεις και τις σοι εδωκεν την εξουσιαν ταυτην
			<A NAME=40N_21_24><B> 21,24 </B>αποκριθεις δε ο ιησους ειπεν αυτοις ερωτησω υμας καγω λογον ενα ον εαν ειπητε μοι καγω υμιν ερω εν ποια εξουσια ταυτα ποιω
			<A NAME=40N_21_25><B> 21,25 </B>το βαπτισμα το ιωαννου ποθεν ην εξ ουρανου η εξ ανθρωπων οι δε διελογιζοντο παρ εαυτοις λεγοντες
			<A NAME=40N_21_26><B> 21,26 </B>εαν ειπωμεν εξ ουρανου ερει ημιν δια τι ουν ουκ επιστευσατε αυτω εαν δε ειπωμεν εξ ανθρωπων φοβουμεθα τον οχλον παντες γαρ ως προφητην εχουσιν τον ιωαννην
			<A NAME=40N_21_27><B> 21,27 </B>και αποκριθεντες τω ιησου ειπαν ουκ οιδαμεν εφη αυτοις και αυτος ουδε εγω λεγω υμιν εν ποια εξουσια ταυτα ποιω
			<A NAME=40N_21_28><B> 21,28 </B>τι δε υμιν δοκει ανθρωπος ειχεν τεκνα δυο προσελθων τω πρωτω ειπεν τεκνον υπαγε σημερον εργαζου εν τω αμπελωνι
			<A NAME=40N_21_29><B> 21,29 </B>ο δε αποκριθεις ειπεν ου θελω υστερον δε μεταμεληθεις απηλθεν
			<A NAME=40N_21_30><B> 21,30 </B>προσελθων δε τω ετερω ειπεν ωσαυτως ο δε αποκριθεις ειπεν εγω κυριε και ουκ απηλθεν
			<A NAME=40N_21_31><B> 21,31 </B>τις εκ των δυο εποιησεν το θελημα του πατρος λεγουσιν ο πρωτος λεγει αυτοις ο ιησους αμην λεγω υμιν οτι οι τελωναι και αι πορναι προαγουσιν υμας εις την βασιλειαν του θεου
			<A NAME=40N_21_32><B> 21,32 </B>ηλθεν γαρ ιωαννης προς υμας εν οδω δικαιοσυνης και ουκ επιστευσατε αυτω οι δε τελωναι και αι πορναι επιστευσαν αυτω υμεις δε ιδοντες ου μετεμεληθητε υστερον του πιστευσαι αυτω
			<A NAME=40N_21_33><B> 21,33 </B>αλλην παραβολην ακουσατε ανθρωπος ην οικοδεσποτης οστις εφυτευσεν αμπελωνα και φραγμον αυτω περιεθηκεν και ωρυξεν εν αυτω ληνον και ωκοδομησεν πυργον και εξεδετο αυτον γεωργοις και απεδημησεν
			<A NAME=40N_21_34><B> 21,34 </B>οτε δε ηγγισεν ο καιρος των καρπων απεστειλεν τους δουλους αυτου προς τους γεωργους λαβειν τους καρπους αυτου
			<A NAME=40N_21_35><B> 21,35 </B>και λαβοντες οι γεωργοι τους δουλους αυτου ον μεν εδειραν ον δε απεκτειναν ον δε ελιθοβολησαν
			<A NAME=40N_21_36><B> 21,36 </B>παλιν απεστειλεν αλλους δουλους πλειονας των πρωτων και εποιησαν αυτοις ωσαυτως
			<A NAME=40N_21_37><B> 21,37 </B>υστερον δε απεστειλεν προς αυτους τον υιον αυτου λεγων εντραπησονται τον υιον μου
			<A NAME=40N_21_38><B> 21,38 </B>οι δε γεωργοι ιδοντες τον υιον ειπον εν εαυτοις ουτος εστιν ο κληρονομος δευτε αποκτεινωμεν αυτον και σχωμεν την κληρονομιαν αυτου
			<A NAME=40N_21_39><B> 21,39 </B>και λαβοντες αυτον εξεβαλον εξω του αμπελωνος και απεκτειναν
			<A NAME=40N_21_40><B> 21,40 </B>οταν ουν ελθη ο κυριος του αμπελωνος τι ποιησει τοις γεωργοις εκεινοις
			<A NAME=40N_21_41><B> 21,41 </B>λεγουσιν αυτω κακους κακως απολεσει αυτους και τον αμπελωνα εκδωσεται αλλοις γεωργοις οιτινες αποδωσουσιν αυτω τους καρπους εν τοις καιροις αυτων
			<A NAME=40N_21_42><B> 21,42 </B>λεγει αυτοις ο ιησους ουδεποτε ανεγνωτε εν ταις γραφαις λιθον ον απεδοκιμασαν οι οικοδομουντες ουτος εγενηθη εις κεφαλην γωνιας παρα κυριου εγενετο αυτη και εστιν θαυμαστη εν οφθαλμοις ημων
			<A NAME=40N_21_43><B> 21,43 </B>δια τουτο λεγω υμιν οτι αρθησεται αφ υμων η βασιλεια του θεου και δοθησεται εθνει ποιουντι τους καρπους αυτης
			<A NAME=40N_21_44><B> 21,44 </B>
			<A NAME=40N_21_45><B> 21,45 </B>ακουσαντες δε οι αρχιερεις και οι φαρισαιοι τας παραβολας αυτου εγνωσαν οτι περι αυτων λεγει
			<A NAME=40N_21_46><B> 21,46 </B>και ζητουντες αυτον κρατησαι εφοβηθησαν τους οχλους επει εις προφητην αυτον ειχον
			</P><P ALIGN= "CENTER"><A NAME=40N_22><H2>Matthäus<BR>Kapitel 22</H2></P><P ALIGN= "LEFT"><A NAME=40N_22_2><B> 22,2 </B>ωμοιωθη η βασιλεια των ουρανων ανθρωπω βασιλει οστις εποιησεν γαμους τω υιω αυτου
			<A NAME=40N_22_3><B> 22,3 </B>και απεστειλεν τους δουλους αυτου καλεσαι τους κεκλημενους εις τους γαμους και ουκ ηθελον ελθειν
			<A NAME=40N_22_4><B> 22,4 </B>παλιν απεστειλεν αλλους δουλους λεγων ειπατε τοις κεκλημενοις ιδου το αριστον μου ητοιμακα οι ταυροι μου και τα σιτιστα τεθυμενα και παντα ετοιμα δευτε εις τους γαμους
			<A NAME=40N_22_5><B> 22,5 </B>οι δε αμελησαντες απηλθον ος μεν εις τον ιδιον αγρον ος δε επι την εμποριαν αυτου
			<A NAME=40N_22_6><B> 22,6 </B>οι δε λοιποι κρατησαντες τους δουλους αυτου υβρισαν και απεκτειναν
			<A NAME=40N_22_7><B> 22,7 </B>ο δε βασιλευς ωργισθη και πεμψας τα στρατευματα αυτου απωλεσεν τους φονεις εκεινους και την πολιν αυτων ενεπρησεν
			<A NAME=40N_22_8><B> 22,8 </B>τοτε λεγει τοις δουλοις αυτου ο μεν γαμος ετοιμος εστιν οι δε κεκλημενοι ουκ ησαν αξιοι
			<A NAME=40N_22_9><B> 22,9 </B>πορευεσθε ουν επι τας διεξοδους των οδων και οσους εαν ευρητε καλεσατε εις τους γαμους
			<A NAME=40N_22_10><B> 22,10 </B>και εξελθοντες οι δουλοι εκεινοι εις τας οδους συνηγαγον παντας οσους ευρον πονηρους τε και αγαθους και επλησθη ο νυμφων ανακειμενων
			<A NAME=40N_22_11><B> 22,11 </B>εισελθων δε ο βασιλευς θεασασθαι τους ανακειμενους ειδεν εκει ανθρωπον ουκ ενδεδυμενον ενδυμα γαμου
			<A NAME=40N_22_12><B> 22,12 </B>και λεγει αυτω εταιρε πως εισηλθες ωδε μη εχων ενδυμα γαμου ο δε εφιμωθη
			<A NAME=40N_22_13><B> 22,13 </B>τοτε ο βασιλευς ειπεν τοις διακονοις δησαντες αυτου ποδας και χειρας εκβαλετε αυτον εις το σκοτος το εξωτερον εκει εσται ο κλαυθμος και ο βρυγμος των οδοντων
			<A NAME=40N_22_14><B> 22,14 </B>πολλοι γαρ εισιν κλητοι ολιγοι δε εκλεκτοι
			<A NAME=40N_22_15><B> 22,15 </B>τοτε πορευθεντες οι φαρισαιοι συμβουλιον ελαβον οπως αυτον παγιδευσωσιν εν λογω
			<A NAME=40N_22_16><B> 22,16 </B>και αποστελλουσιν αυτω τους μαθητας αυτων μετα των ηρωδιανων λεγοντας διδασκαλε οιδαμεν οτι αληθης ει και την οδον του θεου εν αληθεια διδασκεις και ου μελει σοι περι ουδενος ου γαρ βλεπεις εις προσωπον ανθρωπων
			<A NAME=40N_22_17><B> 22,17 </B>ειπον ουν ημιν τι σοι δοκει εξεστιν δουναι κηνσον καισαρι η ου
			<A NAME=40N_22_18><B> 22,18 </B>γνους δε ο ιησους την πονηριαν αυτων ειπεν τι με πειραζετε υποκριται
			<A NAME=40N_22_19><B> 22,19 </B>επιδειξατε μοι το νομισμα του κηνσου οι δε προσηνεγκαν αυτω δηναριον
			<A NAME=40N_22_20><B> 22,20 </B>και λεγει αυτοις ο ιησους τινος η εικων αυτη και η επιγραφη
			<A NAME=40N_22_21><B> 22,21 </B>λεγουσιν καισαρος τοτε λεγει αυτοις αποδοτε ουν τα καισαρος καισαρι και τα του θεου τω θεω
			<A NAME=40N_22_22><B> 22,22 </B>και ακουσαντες εθαυμασαν και αφεντες αυτον απηλθαν
			<A NAME=40N_22_23><B> 22,23 </B>εν εκεινη τη ημερα προσηλθον αυτω σαδδουκαιοι λεγοντες μη ειναι αναστασιν και επηρωτησαν αυτον
			<A NAME=40N_22_24><B> 22,24 </B>λεγοντες διδασκαλε μωυσης ειπεν εαν τις αποθανη μη εχων τεκνα επιγαμβρευσει ο αδελφος αυτου την γυναικα αυτου και αναστησει σπερμα τω αδελφω αυτου
			<A NAME=40N_22_25><B> 22,25 </B>ησαν δε παρ ημιν επτα αδελφοι και ο πρωτος γημας ετελευτησεν και μη εχων σπερμα αφηκεν την γυναικα αυτου τω αδελφω αυτου
			<A NAME=40N_22_26><B> 22,26 </B>ομοιως και ο δευτερος και ο τριτος εως των επτα
			<A NAME=40N_22_27><B> 22,27 </B>υστερον δε παντων απεθανεν η γυνη
			<A NAME=40N_22_28><B> 22,28 </B>εν τη αναστασει ουν τινος των επτα εσται γυνη παντες γαρ εσχον αυτην
			<A NAME=40N_22_29><B> 22,29 </B>αποκριθεις δε ο ιησους ειπεν αυτοις πλανασθε μη ειδοτες τας γραφας μηδε την δυναμιν του θεου
			<A NAME=40N_22_30><B> 22,30 </B>εν γαρ τη αναστασει ουτε γαμουσιν ουτε γαμιζονται αλλ ως αγγελοι θεου εν τω ουρανω εισιν
			<A NAME=40N_22_31><B> 22,31 </B>περι δε της αναστασεως των νεκρων ουκ ανεγνωτε το ρηθεν υμιν υπο του θεου λεγοντος
			<A NAME=40N_22_32><B> 22,32 </B>εγω ειμι ο θεος αβρααμ και ο θεος ισαακ και ο θεος ιακωβ ουκ εστιν θεος νεκρων αλλα ζωντων
			<A NAME=40N_22_33><B> 22,33 </B>και ακουσαντες οι οχλοι εξεπλησσοντο επι τη διδαχη αυτου
			<A NAME=40N_22_34><B> 22,34 </B>οι δε φαρισαιοι ακουσαντες οτι εφιμωσεν τους σαδδουκαιους συνηχθησαν επι το αυτο
			<A NAME=40N_22_35><B> 22,35 </B>και επηρωτησεν εις εξ αυτων νομικος πειραζων αυτον
			<A NAME=40N_22_36><B> 22,36 </B>διδασκαλε ποια εντολη μεγαλη εν τω νομω
			<A NAME=40N_22_37><B> 22,37 </B>ο δε εφη αυτω αγαπησεις κυριον τον θεον σου εν ολη τη καρδια σου και εν ολη τη ψυχη σου και εν ολη τη διανοια σου
			<A NAME=40N_22_38><B> 22,38 </B>αυτη εστιν η μεγαλη και πρωτη εντολη
			<A NAME=40N_22_39><B> 22,39 </B>δευτερα ομοια αυτη αγαπησεις τον πλησιον σου ως σεαυτον
			<A NAME=40N_22_40><B> 22,40 </B>εν ταυταις ταις δυσιν εντολαις ολος ο νομος κρεμαται και οι προφηται
			<A NAME=40N_22_41><B> 22,41 </B>συνηγμενων δε των φαρισαιων επηρωτησεν αυτους ο ιησους
			<A NAME=40N_22_42><B> 22,42 </B>λεγων τι υμιν δοκει περι του χριστου τινος υιος εστιν λεγουσιν αυτω του δαυειδ
			<A NAME=40N_22_43><B> 22,43 </B>λεγει αυτοις πως ουν δαυειδ εν πνευματι καλει κυριον αυτον λεγων
			<A NAME=40N_22_44><B> 22,44 </B>ειπεν κυριος τω κυριω μου καθου εκ δεξιων μου εως αν θω τους εχθρους σου υποκατω των ποδων σου
			<A NAME=40N_22_45><B> 22,45 </B>ει ουν δαυειδ καλει αυτον κυριον πως υιος αυτου εστιν
			<A NAME=40N_22_46><B> 22,46 </B>και ουδεις εδυνατο αποκριθηναι αυτω λογον ουδε ετολμησεν τις απ εκεινης της ημερας επερωτησαι αυτον ουκετι
			</P><P ALIGN= "CENTER"><A NAME=40N_23><H2>Matthäus<BR>Kapitel 23</H2></P><P ALIGN= "LEFT"><A NAME=40N_23_2><B> 23,2 </B>λεγων επι της μωυσεως καθεδρας εκαθισαν οι γραμματεις και οι φαρισαιοι
			<A NAME=40N_23_3><B> 23,3 </B>παντα ουν οσα εαν ειπωσιν υμιν ποιησατε και τηρειτε κατα δε τα εργα αυτων μη ποιειτε λεγουσιν γαρ και ου ποιουσιν
			<A NAME=40N_23_4><B> 23,4 </B>δεσμευουσιν δε φορτια βαρεα και επιτιθεασιν επι τους ωμους των ανθρωπων αυτοι δε τω δακτυλω αυτων ου θελουσιν κινησαι αυτα
			<A NAME=40N_23_5><B> 23,5 </B>παντα δε τα εργα αυτων ποιουσιν προς το θεαθηναι τοις ανθρωποις πλατυνουσιν γαρ τα φυλακτηρια αυτων και μεγαλυνουσιν τα κρασπεδα
			<A NAME=40N_23_6><B> 23,6 </B>φιλουσιν δε την πρωτοκλισιαν εν τοις δειπνοις και τας πρωτοκαθεδριας εν ταις συναγωγαις
			<A NAME=40N_23_7><B> 23,7 </B>και τους ασπασμους εν ταις αγοραις και καλεισθαι υπο των ανθρωπων ραββει
			<A NAME=40N_23_8><B> 23,8 </B>υμεις δε μη κληθητε ραββει εις γαρ εστιν υμων ο διδασκαλος παντες δε υμεις αδελφοι εστε
			<A NAME=40N_23_9><B> 23,9 </B>και πατερα μη καλεσητε υμων επι της γης εις γαρ εστιν υμων ο πατηρ ο ουρανιος
			<A NAME=40N_23_10><B> 23,10 </B>μηδε κληθητε καθηγηται οτι καθηγητης υμων εστιν εις ο χριστος
			<A NAME=40N_23_11><B> 23,11 </B>ο δε μειζων υμων εσται υμων διακονος
			<A NAME=40N_23_12><B> 23,12 </B>οστις δε υψωσει εαυτον ταπεινωθησεται και οστις ταπεινωσει εαυτον υψωθησεται
			<A NAME=40N_23_13><B> 23,13 </B>ουαι δε υμιν γραμματεις και φαρισαιοι υποκριται οτι κλειετε την βασιλειαν των ουρανων εμπροσθεν των ανθρωπων υμεις γαρ ουκ εισερχεσθε ουδε τους εισερχομενους αφιετε εισελθειν
			<A NAME=40N_23_14><B> 23,14 </B>
			<A NAME=40N_23_15><B> 23,15 </B>ουαι υμιν γραμματεις και φαρισαιοι υποκριται οτι περιαγετε την θαλασσαν και την ξηραν ποιησαι ενα προσηλυτον και οταν γενηται ποιειτε αυτον υιον γεεννης διπλοτερον υμων
			<A NAME=40N_23_16><B> 23,16 </B>ουαι υμιν οδηγοι τυφλοι οι λεγοντες ος αν ομοση εν τω ναω ουδεν εστιν ος δ αν ομοση εν τω χρυσω του ναου οφειλει
			<A NAME=40N_23_17><B> 23,17 </B>μωροι και τυφλοι τις γαρ μειζων εστιν ο χρυσος η ο ναος ο αγιασας τον χρυσον
			<A NAME=40N_23_18><B> 23,18 </B>και ος αν ομοση εν τω θυσιαστηριω ουδεν εστιν ος δ αν ομοση εν τω δωρω τω επανω αυτου οφειλει
			<A NAME=40N_23_19><B> 23,19 </B>τυφλοι τι γαρ μειζον το δωρον η το θυσιαστηριον το αγιαζον το δωρον
			<A NAME=40N_23_20><B> 23,20 </B>ο ουν ομοσας εν τω θυσιαστηριω ομνυει εν αυτω και εν πασιν τοις επανω αυτου
			<A NAME=40N_23_21><B> 23,21 </B>και ο ομοσας εν τω ναω ομνυει εν αυτω και εν τω κατοικουντι αυτον
			<A NAME=40N_23_22><B> 23,22 </B>και ο ομοσας εν τω ουρανω ομνυει εν τω θρονω του θεου και εν τω καθημενω επανω αυτου
			<A NAME=40N_23_23><B> 23,23 </B>ουαι υμιν γραμματεις και φαρισαιοι υποκριται οτι αποδεκατουτε το ηδυοσμον και το ανηθον και το κυμινον και αφηκατε τα βαρυτερα του νομου την κρισιν και το ελεος και την πιστιν ταυτα εδει ποιησαι κακεινα μη αφιεναι
			<A NAME=40N_23_24><B> 23,24 </B>οδηγοι τυφλοι οι διυλιζοντες τον κωνωπα την δε καμηλον καταπινοντες
			<A NAME=40N_23_25><B> 23,25 </B>ουαι υμιν γραμματεις και φαρισαιοι υποκριται οτι καθαριζετε το εξωθεν του ποτηριου και της παροψιδος εσωθεν δε γεμουσιν εξ αρπαγης και ακρασιας
			<A NAME=40N_23_26><B> 23,26 </B>φαρισαιε τυφλε καθαρισον πρωτον το εντος του ποτηριου ινα γενηται και το εκτος αυτου καθαρον
			<A NAME=40N_23_27><B> 23,27 </B>ουαι υμιν γραμματεις και φαρισαιοι υποκριται οτι παρομοιαζετε ταφοις κεκονιαμενοις οιτινες εξωθεν μεν φαινονται ωραιοι εσωθεν δε γεμουσιν οστεων νεκρων και πασης ακαθαρσιας
			<A NAME=40N_23_28><B> 23,28 </B>ουτως και υμεις εξωθεν μεν φαινεσθε τοις ανθρωποις δικαιοι εσωθεν δε εστε μεστοι υποκρισεως και ανομιας
			<A NAME=40N_23_29><B> 23,29 </B>ουαι υμιν γραμματεις και φαρισαιοι υποκριται οτι οικοδομειτε τους ταφους των προφητων και κοσμειτε τα μνημεια των δικαιων
			<A NAME=40N_23_30><B> 23,30 </B>και λεγετε ει ημεθα εν ταις ημεραις των πατερων ημων ουκ αν ημεθα κοινωνοι αυτων εν τω αιματι των προφητων
			<A NAME=40N_23_31><B> 23,31 </B>ωστε μαρτυρειτε εαυτοις οτι υιοι εστε των φονευσαντων τους προφητας
			<A NAME=40N_23_32><B> 23,32 </B>και υμεις πληρωσατε το μετρον των πατερων υμων
			<A NAME=40N_23_33><B> 23,33 </B>οφεις γεννηματα εχιδνων πως φυγητε απο της κρισεως της γεεννης
			<A NAME=40N_23_34><B> 23,34 </B>δια τουτο ιδου εγω αποστελλω προς υμας προφητας και σοφους και γραμματεις εξ αυτων αποκτενειτε και σταυρωσετε και εξ αυτων μαστιγωσετε εν ταις συναγωγαις υμων και διωξετε απο πολεως εις πολιν
			<A NAME=40N_23_35><B> 23,35 </B>οπως ελθη εφ υμας παν αιμα δικαιον εκχυννομενον επι της γης απο του αιματος αβελ του δικαιου εως του αιματος ζαχαριου υιου βαραχιου ον εφονευσατε μεταξυ του ναου και του θυσιαστηριου
			<A NAME=40N_23_36><B> 23,36 </B>αμην λεγω υμιν ηξει ταυτα παντα επι την γενεαν ταυτην
			<A NAME=40N_23_37><B> 23,37 </B>ιερουσαλημ ιερουσαλημ η αποκτεινουσα τους προφητας και λιθοβολουσα τους απεσταλμενους προς αυτην ποσακις ηθελησα επισυναγαγειν τα τεκνα σου ον τροπον ορνις επισυναγει τα νοσσια αυτης υπο τας πτερυγας και ουκ ηθελησατε
			<A NAME=40N_23_38><B> 23,38 </B>ιδου αφιεται υμιν ο οικος υμων ερημος
			<A NAME=40N_23_39><B> 23,39 </B>λεγω γαρ υμιν ου μη με ιδητε απ αρτι εως αν ειπητε ευλογημενος ο ερχομενος εν ονοματι κυριου
			</P><P ALIGN= "CENTER"><A NAME=40N_24><H2>Matthäus<BR>Kapitel 24</H2></P><P ALIGN= "LEFT"><A NAME=40N_24_2><B> 24,2 </B>ο δε αποκριθεις ειπεν αυτοις ου βλεπετε ταυτα παντα αμην λεγω υμιν ου μη αφεθη ωδε λιθος επι λιθον ος ου καταλυθησεται
			<A NAME=40N_24_3><B> 24,3 </B>καθημενου δε αυτου επι του ορους των ελαιων προσηλθον αυτω οι μαθηται κατ ιδιαν λεγοντες ειπε ημιν ποτε ταυτα εσται και τι το σημειον της σης παρουσιας και συντελειας του αιωνος
			<A NAME=40N_24_4><B> 24,4 </B>και αποκριθεις ο ιησους ειπεν αυτοις βλεπετε μη τις υμας πλανηση
			<A NAME=40N_24_5><B> 24,5 </B>πολλοι γαρ ελευσονται επι τω ονοματι μου λεγοντες εγω ειμι ο χριστος και πολλους πλανησουσιν
			<A NAME=40N_24_6><B> 24,6 </B>μελλησετε δε ακουειν πολεμους και ακοας πολεμων ορατε μη θροεισθε δει γαρ γενεσθαι αλλ ουπω εστιν το τελος
			<A NAME=40N_24_7><B> 24,7 </B>εγερθησεται γαρ εθνος επ εθνος και βασιλεια επι βασιλειαν και εσονται λιμοι και σεισμοι κατα τοπους
			<A NAME=40N_24_8><B> 24,8 </B>παντα δε ταυτα αρχη ωδινων
			<A NAME=40N_24_9><B> 24,9 </B>τοτε παραδωσουσιν υμας εις θλιψιν και αποκτενουσιν υμας και εσεσθε μισουμενοι υπο παντων των εθνων δια το ονομα μου
			<A NAME=40N_24_10><B> 24,10 </B>και τοτε σκανδαλισθησονται πολλοι και αλληλους παραδωσουσιν και μισησουσιν αλληλους
			<A NAME=40N_24_11><B> 24,11 </B>και πολλοι ψευδοπροφηται εγερθησονται και πλανησουσιν πολλους
			<A NAME=40N_24_12><B> 24,12 </B>και δια το πληθυνθηναι την ανομιαν ψυγησεται η αγαπη των πολλων
			<A NAME=40N_24_13><B> 24,13 </B>ο δε υπομεινας εις τελος ουτος σωθησεται
			<A NAME=40N_24_14><B> 24,14 </B>και κηρυχθησεται τουτο το ευαγγελιον της βασιλειας εν ολη τη οικουμενη εις μαρτυριον πασιν τοις εθνεσιν και τοτε ηξει το τελος
			<A NAME=40N_24_15><B> 24,15 </B>οταν ουν ιδητε το βδελυγμα της ερημωσεως το ρηθεν δια δανιηλ του προφητου εστος εν τοπω αγιω ο αναγινωσκων νοειτω
			<A NAME=40N_24_16><B> 24,16 </B>τοτε οι εν τη ιουδαια φευγετωσαν επι τα ορη
			<A NAME=40N_24_17><B> 24,17 </B>ο επι του δωματος μη καταβατω αραι τα εκ της οικιας αυτου
			<A NAME=40N_24_18><B> 24,18 </B>και ο εν τω αγρω μη επιστρεψατω οπισω αραι το ιματιον αυτου
			<A NAME=40N_24_19><B> 24,19 </B>ουαι δε ταις εν γαστρι εχουσαις και ταις θηλαζουσαις εν εκειναις ταις ημεραις
			<A NAME=40N_24_20><B> 24,20 </B>προσευχεσθε δε ινα μη γενηται η φυγη υμων χειμωνος μηδε σαββατω
			<A NAME=40N_24_21><B> 24,21 </B>εσται γαρ τοτε θλιψις μεγαλη οια ουκ εγενετο απ αρχης κοσμου εως του νυν ουδ ου μη γενηται
			<A NAME=40N_24_22><B> 24,22 </B>και ει μη εκολοβωθησαν αι ημεραι εκειναι ουκ αν εσωθη πασα σαρξ δια δε τους εκλεκτους κολοβωθησονται αι ημεραι εκειναι
			<A NAME=40N_24_23><B> 24,23 </B>τοτε εαν τις υμιν ειπη ιδου ωδε ο χριστος η ωδε μη πιστευσητε
			<A NAME=40N_24_24><B> 24,24 </B>εγερθησονται γαρ ψευδοχριστοι και ψευδοπροφηται και δωσουσιν σημεια μεγαλα και τερατα ωστε πλανασθαι ει δυνατον και τους εκλεκτους
			<A NAME=40N_24_25><B> 24,25 </B>ιδου προειρηκα υμιν
			<A NAME=40N_24_26><B> 24,26 </B>εαν ουν ειπωσιν υμιν ιδου εν τη ερημω εστιν μη εξελθητε ιδου εν τοις ταμειοις μη πιστευσητε
			<A NAME=40N_24_27><B> 24,27 </B>ωσπερ γαρ η αστραπη εξερχεται απο ανατολων και φαινεται εως δυσμων ουτως εσται η παρουσια του υιου του ανθρωπου
			<A NAME=40N_24_28><B> 24,28 </B>οπου εαν η το πτωμα εκει συναχθησονται οι αετοι
			<A NAME=40N_24_29><B> 24,29 </B>ευθεως δε μετα την θλιψιν των ημερων εκεινων ο ηλιος σκοτισθησεται και η σεληνη ου δωσει το φεγγος αυτης και οι αστερες πεσουνται εκ του ουρανου και αι δυναμεις των ουρανων σαλευθησονται
			<A NAME=40N_24_30><B> 24,30 </B>και τοτε φανησεται το σημειον του υιου του ανθρωπου εν ουρανω και κοψονται πασαι αι φυλαι της γης και οψονται τον υιον του ανθρωπου ερχομενον επι των νεφελων του ουρανου μετα δυναμεως και δοξης πολλης
			<A NAME=40N_24_31><B> 24,31 </B>και αποστελει τους αγγελους αυτου μετα σαλπιγγος μεγαλης και επισυναξουσιν τους εκλεκτους αυτου εκ των τεσσαρων ανεμων απ ακρων ουρανων εως ακρων αυτων
			<A NAME=40N_24_32><B> 24,32 </B>απο δε της συκης μαθετε την παραβολην οταν ηδη ο κλαδος αυτης γενηται απαλος και τα φυλλα εκφυη γινωσκετε οτι εγγυς το θερος
			<A NAME=40N_24_33><B> 24,33 </B>ουτως και υμεις οταν ιδητε ταυτα παντα γινωσκετε οτι εγγυς εστιν επι θυραις
			<A NAME=40N_24_34><B> 24,34 </B>αμην λεγω υμιν οτι ου μη παρελθη η γενεα αυτη εως αν παντα ταυτα γενηται
			<A NAME=40N_24_35><B> 24,35 </B>ο ουρανος και η γη παρελευσεται οι δε λογοι μου ου μη παρελθωσιν
			<A NAME=40N_24_36><B> 24,36 </B>περι δε της ημερας εκεινης και ωρας ουδεις οιδεν ουδε οι αγγελοι των ουρανων ουδε ο υιος ει μη ο πατηρ μονος
			<A NAME=40N_24_37><B> 24,37 </B>ωσπερ δε αι ημεραι του νωε ουτως εσται η παρουσια του υιου του ανθρωπου
			<A NAME=40N_24_38><B> 24,38 </B>ως γαρ ησαν εν ταις ημεραις ταις προ του κατακλυσμου τρωγοντες και πινοντες γαμουντες και γαμιζοντες αχρι ης ημερας εισηλθεν νωε εις την κιβωτον
			<A NAME=40N_24_39><B> 24,39 </B>και ουκ εγνωσαν εως ηλθεν ο κατακλυσμος και ηρεν απαντας ουτως εσται και η παρουσια του υιου του ανθρωπου
			<A NAME=40N_24_40><B> 24,40 </B>τοτε εσονται δυο εν τω αγρω εις παραλαμβανεται και εις αφιεται
			<A NAME=40N_24_41><B> 24,41 </B>δυο αληθουσαι εν τω μυλω μια παραλαμβανεται και μια αφιεται
			<A NAME=40N_24_42><B> 24,42 </B>γρηγορειτε ουν οτι ουκ οιδατε ποια ημερα ο κυριος υμων ερχεται
			<A NAME=40N_24_43><B> 24,43 </B>εκεινο δε γινωσκετε οτι ει ηδει ο οικοδεσποτης ποια φυλακη ο κλεπτης ερχεται εγρηγορησεν αν και ουκ αν ειασεν διορυχθηναι την οικιαν αυτου
			<A NAME=40N_24_44><B> 24,44 </B>δια τουτο και υμεις γινεσθε ετοιμοι οτι η ου δοκειτε ωρα ο υιος του ανθρωπου ερχεται
			<A NAME=40N_24_45><B> 24,45 </B>τις αρα εστιν ο πιστος δουλος και φρονιμος ον κατεστησεν ο κυριος επι της οικετειας αυτου του δουναι αυτοις την τροφην εν καιρω
			<A NAME=40N_24_46><B> 24,46 </B>μακαριος ο δουλος εκεινος ον ελθων ο κυριος αυτου ευρησει ουτως ποιουντα
			<A NAME=40N_24_47><B> 24,47 </B>αμην λεγω υμιν οτι επι πασιν τοις υπαρχουσιν αυτου καταστησει αυτον
			<A NAME=40N_24_48><B> 24,48 </B>εαν δε ειπη ο κακος δουλος εν τη καρδια αυτου χρονιζει μου ο κυριος
			<A NAME=40N_24_49><B> 24,49 </B>και αρξηται τυπτειν τους συνδουλους αυτου εσθιη δε και πινη μετα των μεθυοντων
			<A NAME=40N_24_50><B> 24,50 </B>ηξει ο κυριος του δουλου εκεινου εν ημερα η ου προσδοκα και εν ωρα η ου γινωσκει
			<A NAME=40N_24_51><B> 24,51 </B>και διχοτομησει αυτον και το μερος αυτου μετα των υποκριτων θησει εκει εσται ο κλαυθμος και ο βρυγμος των οδοντων
			</P><P ALIGN= "CENTER"><A NAME=40N_25><H2>Matthäus<BR>Kapitel 25</H2></P><P ALIGN= "LEFT"><A NAME=40N_25_2><B> 25,2 </B>πεντε δε εξ αυτων ησαν μωραι και πεντε φρονιμοι
			<A NAME=40N_25_3><B> 25,3 </B>αι γαρ μωραι λαβουσαι τας λαμπαδας ουκ ελαβον μεθ εαυτων ελαιον
			<A NAME=40N_25_4><B> 25,4 </B>αι δε φρονιμοι ελαβον ελαιον εν τοις αγγειοις μετα των λαμπαδων εαυτων
			<A NAME=40N_25_5><B> 25,5 </B>χρονιζοντος δε του νυμφιου ενυσταξαν πασαι και εκαθευδον
			<A NAME=40N_25_6><B> 25,6 </B>μεσης δε νυκτος κραυγη γεγονεν ιδου ο νυμφιος εξερχεσθε εις απαντησιν
			<A NAME=40N_25_7><B> 25,7 </B>τοτε ηγερθησαν πασαι αι παρθενοι εκειναι και εκοσμησαν τας λαμπαδας εαυτων
			<A NAME=40N_25_8><B> 25,8 </B>αι δε μωραι ταις φρονιμοις ειπαν δοτε ημιν εκ του ελαιου υμων οτι αι λαμπαδες ημων σβεννυνται
			<A NAME=40N_25_9><B> 25,9 </B>απεκριθησαν δε αι φρονιμοι λεγουσαι μηποτε ουκ αρκεση ημιν και υμιν πορευεσθε μαλλον προς τους πωλουντας και αγορασατε εαυταις
			<A NAME=40N_25_10><B> 25,10 </B>απερχομενων δε αυτων αγορασαι ηλθεν ο νυμφιος και αι ετοιμοι εισηλθον μετ αυτου εις τους γαμους και εκλεισθη η θυρα
			<A NAME=40N_25_11><B> 25,11 </B>υστερον δε ερχονται και αι λοιπαι παρθενοι λεγουσαι κυριε κυριε ανοιξον ημιν
			<A NAME=40N_25_12><B> 25,12 </B>ο δε αποκριθεις ειπεν αμην λεγω υμιν ουκ οιδα υμας
			<A NAME=40N_25_13><B> 25,13 </B>γρηγορειτε ουν οτι ουκ οιδατε την ημεραν ουδε την ωραν
			<A NAME=40N_25_14><B> 25,14 </B>ωσπερ γαρ ανθρωπος αποδημων εκαλεσεν τους ιδιους δουλους και παρεδωκεν αυτοις τα υπαρχοντα αυτου
			<A NAME=40N_25_15><B> 25,15 </B>και ω μεν εδωκεν πεντε ταλαντα ω δε δυο ω δε εν εκαστω κατα την ιδιαν δυναμιν και απεδημησεν ευθεως
			<A NAME=40N_25_16><B> 25,16 </B>πορευθεις ο τα πεντε ταλαντα λαβων ηργασατο εν αυτοις και εποιησεν αλλα πεντε ταλαντα
			<A NAME=40N_25_17><B> 25,17 </B>ωσαυτως ο τα δυο εκερδησεν αλλα δυο
			<A NAME=40N_25_18><B> 25,18 </B>ο δε το εν λαβων απελθων ωρυξεν γην και εκρυψεν το αργυριον του κυριου αυτου
			<A NAME=40N_25_19><B> 25,19 </B>μετα δε πολυν χρονον ερχεται ο κυριος των δουλων εκεινων και συναιρει λογον μετ αυτων
			<A NAME=40N_25_20><B> 25,20 </B>και προσελθων ο τα πεντε ταλαντα λαβων προσηνεγκεν αλλα πεντε ταλαντα λεγων κυριε πεντε ταλαντα μοι παρεδωκας ιδε αλλα πεντε ταλαντα εκερδησα
			<A NAME=40N_25_21><B> 25,21 </B>εφη αυτω ο κυριος αυτου ευ δουλε αγαθε και πιστε επι ολιγα ης πιστος επι πολλων σε καταστησω εισελθε εις την χαραν του κυριου σου
			<A NAME=40N_25_22><B> 25,22 </B>προσελθων και ο τα δυο ταλαντα ειπεν κυριε δυο ταλαντα μοι παρεδωκας ιδε αλλα δυο ταλαντα εκερδησα
			<A NAME=40N_25_23><B> 25,23 </B>εφη αυτω ο κυριος αυτου ευ δουλε αγαθε και πιστε επι ολιγα ης πιστος επι πολλων σε καταστησω εισελθε εις την χαραν του κυριου σου
			<A NAME=40N_25_24><B> 25,24 </B>προσελθων δε και ο το εν ταλαντον ειληφως ειπεν κυριε εγνων σε οτι σκληρος ει ανθρωπος θεριζων οπου ουκ εσπειρας και συναγων οθεν ου διεσκορπισας
			<A NAME=40N_25_25><B> 25,25 </B>και φοβηθεις απελθων εκρυψα το ταλαντον σου εν τη γη ιδε εχεις το σον
			<A NAME=40N_25_26><B> 25,26 </B>αποκριθεις δε ο κυριος αυτου ειπεν αυτω πονηρε δουλε και οκνηρε ηδεις οτι θεριζω οπου ουκ εσπειρα και συναγω οθεν ου διεσκορπισα
			<A NAME=40N_25_27><B> 25,27 </B>εδει σε ουν βαλειν τα αργυρια μου τοις τραπεζιταις και ελθων εγω εκομισαμην αν το εμον συν τοκω
			<A NAME=40N_25_28><B> 25,28 </B>αρατε ουν απ αυτου το ταλαντον και δοτε τω εχοντι τα δεκα ταλαντα
			<A NAME=40N_25_29><B> 25,29 </B>τω γαρ εχοντι παντι δοθησεται και περισσευθησεται του δε μη εχοντος και ο εχει αρθησεται απ αυτου
			<A NAME=40N_25_30><B> 25,30 </B>και τον αχρειον δουλον εκβαλετε εις το σκοτος το εξωτερον εκει εσται ο κλαυθμος και ο βρυγμος των οδοντων
			<A NAME=40N_25_31><B> 25,31 </B>οταν δε ελθη ο υιος του ανθρωπου εν τη δοξη αυτου και παντες οι αγγελοι μετ αυτου τοτε καθισει επι θρονου δοξης αυτου
			<A NAME=40N_25_32><B> 25,32 </B>και συναχθησονται εμπροσθεν αυτου παντα τα εθνη και αφορισει αυτους απ αλληλων ωσπερ ο ποιμην αφοριζει τα προβατα απο των εριφων
			<A NAME=40N_25_33><B> 25,33 </B>και στησει τα μεν προβατα εκ δεξιων αυτου τα δε εριφια εξ ευωνυμων
			<A NAME=40N_25_34><B> 25,34 </B>τοτε ερει ο βασιλευς τοις εκ δεξιων αυτου δευτε οι ευλογημενοι του πατρος μου κληρονομησατε την ητοιμασμενην υμιν βασιλειαν απο καταβολης κοσμου
			<A NAME=40N_25_35><B> 25,35 </B>επεινασα γαρ και εδωκατε μοι φαγειν εδιψησα και εποτισατε με ξενος ημην και συνηγαγετε με
			<A NAME=40N_25_36><B> 25,36 </B>γυμνος και περιεβαλετε με ησθενησα και επεσκεψασθε με εν φυλακη ημην και ηλθατε προς με
			<A NAME=40N_25_37><B> 25,37 </B>τοτε αποκριθησονται αυτω οι δικαιοι λεγοντες κυριε ποτε σε ειδομεν πεινωντα και εθρεψαμεν η διψωντα και εποτισαμεν
			<A NAME=40N_25_38><B> 25,38 </B>ποτε δε σε ειδομεν ξενον και συνηγαγομεν η γυμνον και περιεβαλομεν
			<A NAME=40N_25_39><B> 25,39 </B>ποτε δε σε ειδομεν ασθενουντα η εν φυλακη και ηλθομεν προς σε
			<A NAME=40N_25_40><B> 25,40 </B>και αποκριθεις ο βασιλευς ερει αυτοις αμην λεγω υμιν εφ οσον εποιησατε ενι τουτων των αδελφων μου των ελαχιστων εμοι εποιησατε
			<A NAME=40N_25_41><B> 25,41 </B>τοτε ερει και τοις εξ ευωνυμων πορευεσθε απ εμου κατηραμενοι εις το πυρ το αιωνιον το ητοιμασμενον τω διαβολω και τοις αγγελοις αυτου
			<A NAME=40N_25_42><B> 25,42 </B>επεινασα γαρ και ουκ εδωκατε μοι φαγειν εδιψησα και ουκ εποτισατε με
			<A NAME=40N_25_43><B> 25,43 </B>ξενος ημην και ου συνηγαγετε με γυμνος και ου περιεβαλετε με ασθενης και εν φυλακη και ουκ επεσκεψασθε με
			<A NAME=40N_25_44><B> 25,44 </B>τοτε αποκριθησονται και αυτοι λεγοντες κυριε ποτε σε ειδομεν πεινωντα η διψωντα η ξενον η γυμνον η ασθενη η εν φυλακη και ου διηκονησαμεν σοι
			<A NAME=40N_25_45><B> 25,45 </B>τοτε αποκριθησεται αυτοις λεγων αμην λεγω υμιν εφ οσον ουκ εποιησατε ενι τουτων των ελαχιστων ουδε εμοι εποιησατε
			<A NAME=40N_25_46><B> 25,46 </B>και απελευσονται ουτοι εις κολασιν αιωνιον οι δε δικαιοι εις ζωην αιωνιον
			</P><P ALIGN= "CENTER"><A NAME=40N_26><H2>Matthäus<BR>Kapitel 26</H2></P><P ALIGN= "LEFT"><A NAME=40N_26_2><B> 26,2 </B>οιδατε οτι μετα δυο ημερας το πασχα γινεται και ο υιος του ανθρωπου παραδιδοται εις το σταυρωθηναι
			<A NAME=40N_26_3><B> 26,3 </B>τοτε συνηχθησαν οι αρχιερεις και οι πρεσβυτεροι του λαου εις την αυλην του αρχιερεως του λεγομενου καιαφα
			<A NAME=40N_26_4><B> 26,4 </B>και συνεβουλευσαντο ινα τον ιησουν δολω κρατησωσιν και αποκτεινωσιν
			<A NAME=40N_26_5><B> 26,5 </B>ελεγον δε μη εν τη εορτη ινα μη θορυβος γενηται εν τω λαω
			<A NAME=40N_26_6><B> 26,6 </B>του δε ιησου γενομενου εν βηθανια εν οικια σιμωνος του λεπρου
			<A NAME=40N_26_7><B> 26,7 </B>προσηλθεν αυτω γυνη εχουσα αλαβαστρον μυρου πολυτιμου και κατεχεεν επι της κεφαλης αυτου ανακειμενου
			<A NAME=40N_26_8><B> 26,8 </B>ιδοντες δε οι μαθηται ηγανακτησαν λεγοντες εις τι η απωλεια αυτη
			<A NAME=40N_26_9><B> 26,9 </B>εδυνατο γαρ τουτο πραθηναι πολλου και δοθηναι πτωχοις
			<A NAME=40N_26_10><B> 26,10 </B>γνους δε ο ιησους ειπεν αυτοις τι κοπους παρεχετε τη γυναικι εργον γαρ καλον ηργασατο εις εμε
			<A NAME=40N_26_11><B> 26,11 </B>παντοτε γαρ τους πτωχους εχετε μεθ εαυτων εμε δε ου παντοτε εχετε
			<A NAME=40N_26_12><B> 26,12 </B>βαλουσα γαρ αυτη το μυρον τουτο επι του σωματος μου προς το ενταφιασαι με εποιησεν
			<A NAME=40N_26_13><B> 26,13 </B>αμην λεγω υμιν οπου εαν κηρυχθη το ευαγγελιον τουτο εν ολω τω κοσμω λαληθησεται και ο εποιησεν αυτη εις μνημοσυνον αυτης
			<A NAME=40N_26_14><B> 26,14 </B>τοτε πορευθεις εις των δωδεκα ο λεγομενος ιουδας ισκαριωτης προς τους αρχιερεις
			<A NAME=40N_26_15><B> 26,15 </B>ειπεν τι θελετε μοι δουναι και εγω υμιν παραδωσω αυτον οι δε εστησαν αυτω τριακοντα αργυρια
			<A NAME=40N_26_16><B> 26,16 </B>και απο τοτε εζητει ευκαιριαν ινα αυτον παραδω
			<A NAME=40N_26_17><B> 26,17 </B>τη δε πρωτη των αζυμων προσηλθον οι μαθηται τω ιησου λεγοντες που θελεις ετοιμασωμεν σοι φαγειν το πασχα
			<A NAME=40N_26_18><B> 26,18 </B>ο δε ειπεν υπαγετε εις την πολιν προς τον δεινα και ειπατε αυτω ο διδασκαλος λεγει ο καιρος μου εγγυς εστιν προς σε ποιω το πασχα μετα των μαθητων μου
			<A NAME=40N_26_19><B> 26,19 </B>και εποιησαν οι μαθηται ως συνεταξεν αυτοις ο ιησους και ητοιμασαν το πασχα
			<A NAME=40N_26_20><B> 26,20 </B>οψιας δε γενομενης ανεκειτο μετα των δωδεκα μαθητων
			<A NAME=40N_26_21><B> 26,21 </B>και εσθιοντων αυτων ειπεν αμην λεγω υμιν οτι εις εξ υμων παραδωσει με
			<A NAME=40N_26_22><B> 26,22 </B>και λυπουμενοι σφοδρα ηρξαντο λεγειν αυτω εις εκαστος μητι εγω ειμι κυριε
			<A NAME=40N_26_23><B> 26,23 </B>ο δε αποκριθεις ειπεν ο εμβαψας μετ εμου την χειρα εν τω τρυβλιω ουτος με παραδωσει
			<A NAME=40N_26_24><B> 26,24 </B>ο μεν υιος του ανθρωπου υπαγει καθως γεγραπται περι αυτου ουαι δε τω ανθρωπω εκεινω δι ου ο υιος του ανθρωπου παραδιδοται καλον ην αυτω ει ουκ εγεννηθη ο ανθρωπος εκεινος
			<A NAME=40N_26_25><B> 26,25 </B>αποκριθεις δε ιουδας ο παραδιδους αυτον ειπεν μητι εγω ειμι ραββει λεγει αυτω συ ειπας
			<A NAME=40N_26_26><B> 26,26 </B>εσθιοντων δε αυτων λαβων ο ιησους αρτον και ευλογησας εκλασεν και δους τοις μαθηταις ειπεν λαβετε φαγετε τουτο εστιν το σωμα μου
			<A NAME=40N_26_27><B> 26,27 </B>και λαβων ποτηριον και ευχαριστησας εδωκεν αυτοις λεγων πιετε εξ αυτου παντες
			<A NAME=40N_26_28><B> 26,28 </B>τουτο γαρ εστιν το αιμα μου της διαθηκης το περι πολλων εκχυννομενον εις αφεσιν αμαρτιων
			<A NAME=40N_26_29><B> 26,29 </B>λεγω δε υμιν ου μη πιω απ αρτι εκ τουτου του γενηματος της αμπελου εως της ημερας εκεινης οταν αυτο πινω μεθ υμων καινον εν τη βασιλεια του πατρος μου
			<A NAME=40N_26_30><B> 26,30 </B>και υμνησαντες εξηλθον εις το ορος των ελαιων
			<A NAME=40N_26_31><B> 26,31 </B>τοτε λεγει αυτοις ο ιησους παντες υμεις σκανδαλισθησεσθε εν εμοι εν τη νυκτι ταυτη γεγραπται γαρ παταξω τον ποιμενα και διασκορπισθησονται τα προβατα της ποιμνης
			<A NAME=40N_26_32><B> 26,32 </B>μετα δε το εγερθηναι με προαξω υμας εις την γαλιλαιαν
			<A NAME=40N_26_33><B> 26,33 </B>αποκριθεις δε ο πετρος ειπεν αυτω ει παντες σκανδαλισθησονται εν σοι εγω ουδεποτε σκανδαλισθησομαι
			<A NAME=40N_26_34><B> 26,34 </B>εφη αυτω ο ιησους αμην λεγω σοι οτι εν ταυτη τη νυκτι πριν αλεκτορα φωνησαι τρις απαρνηση με
			<A NAME=40N_26_35><B> 26,35 </B>λεγει αυτω ο πετρος καν δεη με συν σοι αποθανειν ου μη σε απαρνησομαι ομοιως και παντες οι μαθηται ειπον
			<A NAME=40N_26_36><B> 26,36 </B>τοτε ερχεται μετ αυτων ο ιησους εις χωριον λεγομενον γεθσημανει και λεγει τοις μαθηταις καθισατε αυτου εως ου απελθων εκει προσευξωμαι
			<A NAME=40N_26_37><B> 26,37 </B>και παραλαβων τον πετρον και τους δυο υιους ζεβεδαιου ηρξατο λυπεισθαι και αδημονειν
			<A NAME=40N_26_38><B> 26,38 </B>τοτε λεγει αυτοις περιλυπος εστιν η ψυχη μου εως θανατου μεινατε ωδε και γρηγορειτε μετ εμου
			<A NAME=40N_26_39><B> 26,39 </B>και προσελθων μικρον επεσεν επι προσωπον αυτου προσευχομενος και λεγων πατερ ει δυνατον εστιν παρελθατω απ εμου το ποτηριον τουτο πλην ουχ ως εγω θελω αλλ ως συ
			<A NAME=40N_26_40><B> 26,40 </B>και ερχεται προς τους μαθητας και ευρισκει αυτους καθευδοντας και λεγει τω πετρω ουτως ουκ ισχυσατε μιαν ωραν γρηγορησαι μετ εμου
			<A NAME=40N_26_41><B> 26,41 </B>γρηγορειτε και προσευχεσθε ινα μη εισελθητε εις πειρασμον το μεν πνευμα προθυμον η δε σαρξ ασθενης
			<A NAME=40N_26_42><B> 26,42 </B>παλιν εκ δευτερου απελθων προσηυξατο λεγων πατερ μου ει ου δυναται τουτο παρελθειν εαν μη αυτο πιω γενηθητω το θελημα σου
			<A NAME=40N_26_43><B> 26,43 </B>και ελθων παλιν ευρεν αυτους καθευδοντας ησαν γαρ αυτων οι οφθαλμοι βεβαρημενοι
			<A NAME=40N_26_44><B> 26,44 </B>και αφεις αυτους παλιν απελθων προσηυξατο εκ τριτου τον αυτον λογον ειπων παλιν
			<A NAME=40N_26_45><B> 26,45 </B>τοτε ερχεται προς τους μαθητας και λεγει αυτοις καθευδετε το λοιπον και αναπαυεσθε ιδου ηγγικεν η ωρα και ο υιος του ανθρωπου παραδιδοται εις χειρας αμαρτωλων
			<A NAME=40N_26_46><B> 26,46 </B>εγειρεσθε αγωμεν ιδου ηγγικεν ο παραδιδους με
			<A NAME=40N_26_47><B> 26,47 </B>και ετι αυτου λαλουντος ιδου ιουδας εις των δωδεκα ηλθεν και μετ αυτου οχλος πολυς μετα μαχαιρων και ξυλων απο των αρχιερεων και πρεσβυτερων του λαου
			<A NAME=40N_26_48><B> 26,48 </B>ο δε παραδιδους αυτον εδωκεν αυτοις σημειον λεγων ον εαν φιλησω αυτος εστιν κρατησατε αυτον
			<A NAME=40N_26_49><B> 26,49 </B>και ευθεως προσελθων τω ιησου ειπεν χαιρε ραββει και κατεφιλησεν αυτον
			<A NAME=40N_26_50><B> 26,50 </B>ο δε ιησους ειπεν αυτω εταιρε εφ ο παρει τοτε προσελθοντες επεβαλον τας χειρας επι τον ιησουν και εκρατησαν αυτον
			<A NAME=40N_26_51><B> 26,51 </B>και ιδου εις των μετα ιησου εκτεινας την χειρα απεσπασεν την μαχαιραν αυτου και παταξας τον δουλον του αρχιερεως αφειλεν αυτου το ωτιον
			<A NAME=40N_26_52><B> 26,52 </B>τοτε λεγει αυτω ο ιησους αποστρεψον την μαχαιραν σου εις τον τοπον αυτης παντες γαρ οι λαβοντες μαχαιραν εν μαχαιρη απολουνται
			<A NAME=40N_26_53><B> 26,53 </B>η δοκεις οτι ου δυναμαι παρακαλεσαι τον πατερα μου και παραστησει μοι αρτι πλειω δωδεκα λεγιωνων αγγελων
			<A NAME=40N_26_54><B> 26,54 </B>πως ουν πληρωθωσιν αι γραφαι οτι ουτως δει γενεσθαι
			<A NAME=40N_26_55><B> 26,55 </B>εν εκεινη τη ωρα ειπεν ο ιησους τοις οχλοις ως επι ληστην εξηλθατε μετα μαχαιρων και ξυλων συλλαβειν με καθ ημεραν εν τω ιερω εκαθεζομην διδασκων και ουκ εκρατησατε με
			<A NAME=40N_26_56><B> 26,56 </B>τουτο δε ολον γεγονεν ινα πληρωθωσιν αι γραφαι των προφητων τοτε οι μαθηται παντες αφεντες αυτον εφυγον
			<A NAME=40N_26_57><B> 26,57 </B>οι δε κρατησαντες τον ιησουν απηγαγον προς καιαφαν τον αρχιερεα οπου οι γραμματεις και οι πρεσβυτεροι συνηχθησαν
			<A NAME=40N_26_58><B> 26,58 </B>ο δε πετρος ηκολουθει αυτω μακροθεν εως της αυλης του αρχιερεως και εισελθων εσω εκαθητο μετα των υπηρετων ιδειν το τελος
			<A NAME=40N_26_59><B> 26,59 </B>οι δε αρχιερεις και το συνεδριον ολον εζητουν ψευδομαρτυριαν κατα του ιησου οπως αυτον θανατωσουσιν
			<A NAME=40N_26_60><B> 26,60 </B>και ουχ ευρον πολλων προσελθοντων ψευδομαρτυρων υστερον δε προσελθοντες δυο
			<A NAME=40N_26_61><B> 26,61 </B>ειπον ουτος εφη δυναμαι καταλυσαι τον ναον του θεου και δια τριων ημερων αυτον οικοδομησαι
			<A NAME=40N_26_62><B> 26,62 </B>και αναστας ο αρχιερευς ειπεν αυτω ουδεν αποκρινη τι ουτοι σου καταμαρτυρουσιν
			<A NAME=40N_26_63><B> 26,63 </B>ο δε ιησους εσιωπα και αποκριθεις ο αρχιερευς ειπεν αυτω εξορκιζω σε κατα του θεου του ζωντος ινα ημιν ειπης ει συ ει ο χριστος ο υιος του θεου
			<A NAME=40N_26_64><B> 26,64 </B>λεγει αυτω ο ιησους συ ειπας πλην λεγω υμιν απ αρτι οψεσθε τον υιον του ανθρωπου καθημενον εκ δεξιων της δυναμεως και ερχομενον επι των νεφελων του ουρανου
			<A NAME=40N_26_65><B> 26,65 </B>τοτε ο αρχιερευς διερρηξεν τα ιματια αυτου λεγων εβλασφημησεν τι ετι χρειαν εχομεν μαρτυρων ιδε νυν ηκουσατε την βλασφημιαν
			<A NAME=40N_26_66><B> 26,66 </B>τι υμιν δοκει οι δε αποκριθεντες ειπον ενοχος θανατου εστιν
			<A NAME=40N_26_67><B> 26,67 </B>τοτε ενεπτυσαν εις το προσωπον αυτου και εκολαφισαν αυτον οι δε εραπισαν
			<A NAME=40N_26_68><B> 26,68 </B>λεγοντες προφητευσον ημιν χριστε τις εστιν ο παισας σε
			<A NAME=40N_26_69><B> 26,69 </B>ο δε πετρος εκαθητο εξω εν τη αυλη και προσηλθεν αυτω μια παιδισκη λεγουσα και συ ησθα μετα ιησου του γαλιλαιου
			<A NAME=40N_26_70><B> 26,70 </B>ο δε ηρνησατο εμπροσθεν παντων λεγων ουκ οιδα τι λεγεις
			<A NAME=40N_26_71><B> 26,71 </B>εξελθοντα δε αυτον εις τον πυλωνα ειδεν αυτον αλλη και λεγει τοις εκει ουτος ην μετα ιησου του ναζωραιου
			<A NAME=40N_26_72><B> 26,72 </B>και παλιν ηρνησατο μετα ορκου οτι ουκ οιδα τον ανθρωπον
			<A NAME=40N_26_73><B> 26,73 </B>μετα μικρον δε προσελθοντες οι εστωτες ειπον τω πετρω αληθως και συ εξ αυτων ει και γαρ η λαλια σου δηλον σε ποιει
			<A NAME=40N_26_74><B> 26,74 </B>τοτε ηρξατο καταθεματιζειν και ομνυειν οτι ουκ οιδα τον ανθρωπον και ευθεως αλεκτωρ εφωνησεν
			<A NAME=40N_26_75><B> 26,75 </B>και εμνησθη ο πετρος του ρηματος ιησου ειρηκοτος οτι πριν αλεκτορα φωνησαι τρις απαρνηση με και εξελθων εξω εκλαυσεν πικρως
			</P><P ALIGN= "CENTER"><A NAME=40N_27><H2>Matthäus<BR>Kapitel 27</H2></P><P ALIGN= "LEFT"><A NAME=40N_27_2><B> 27,2 </B>και δησαντες αυτον απηγαγον και παρεδωκαν πειλατω τω ηγεμονι
			<A NAME=40N_27_3><B> 27,3 </B>τοτε ιδων ιουδας ο παραδιδους αυτον οτι κατεκριθη μεταμεληθεις εστρεψεν τα τριακοντα αργυρια τοις αρχιερευσιν και πρεσβυτεροις
			<A NAME=40N_27_4><B> 27,4 </B>λεγων ημαρτον παραδους αιμα αθωον οι δε ειπον τι προς ημας συ οψη
			<A NAME=40N_27_5><B> 27,5 </B>και ριψας τα αργυρια εις τον ναον ανεχωρησεν και απελθων απηγξατο
			<A NAME=40N_27_6><B> 27,6 </B>οι δε αρχιερεις λαβοντες τα αργυρια ειπαν ουκ εξεστιν βαλειν αυτα εις τον κορβαναν επει τιμη αιματος εστιν
			<A NAME=40N_27_7><B> 27,7 </B>συμβουλιον δε λαβοντες ηγορασαν εξ αυτων τον αγρον του κεραμεως εις ταφην τοις ξενοις
			<A NAME=40N_27_8><B> 27,8 </B>διο εκληθη ο αγρος εκεινος αγρος αιματος εως της σημερον
			<A NAME=40N_27_9><B> 27,9 </B>τοτε επληρωθη το ρηθεν δια ιερεμιου του προφητου λεγοντος και ελαβον τα τριακοντα αργυρια την τιμην του τετιμημενου ον ετιμησαντο απο υιων ισραηλ
			<A NAME=40N_27_10><B> 27,10 </B>και εδωκαν αυτα εις τον αγρον του κεραμεως καθα συνεταξεν μοι κυριος
			<A NAME=40N_27_11><B> 27,11 </B>ο δε ιησους εσταθη εμπροσθεν του ηγεμονος και επηρωτησεν αυτον ο ηγεμων λεγων συ ει ο βασιλευς των ιουδαιων ο δε ιησους εφη συ λεγεις
			<A NAME=40N_27_12><B> 27,12 </B>και εν τω κατηγορεισθαι αυτον υπο των αρχιερεων και πρεσβυτερων ουδεν απεκρινατο
			<A NAME=40N_27_13><B> 27,13 </B>τοτε λεγει αυτω ο πειλατος ουκ ακουεις ποσα σου καταμαρτυρουσιν
			<A NAME=40N_27_14><B> 27,14 </B>και ουκ απεκριθη αυτω προς ουδε εν ρημα ωστε θαυμαζειν τον ηγεμονα λιαν
			<A NAME=40N_27_15><B> 27,15 </B>κατα δε εορτην ειωθει ο ηγεμων απολυειν ενα τω οχλω δεσμιον ον ηθελον
			<A NAME=40N_27_16><B> 27,16 </B>ειχον δε τοτε δεσμιον επισημον λεγομενον βαραββαν
			<A NAME=40N_27_17><B> 27,17 </B>συνηγμενων ουν αυτων ειπεν αυτοις ο πειλατος τινα θελετε απολυσω υμιν βαραββαν η ιησουν τον λεγομενον χριστον
			<A NAME=40N_27_18><B> 27,18 </B>ηδει γαρ οτι δια φθονον παρεδωκαν αυτον
			<A NAME=40N_27_19><B> 27,19 </B>καθημενου δε αυτου επι του βηματος απεστειλεν προς αυτον η γυνη αυτου λεγουσα μηδεν σοι και τω δικαιω εκεινω πολλα γαρ επαθον σημερον κατ οναρ δι αυτον
			<A NAME=40N_27_20><B> 27,20 </B>οι δε αρχιερεις και οι πρεσβυτεροι επεισαν τους οχλους ινα αιτησωνται τον βαραββαν τον δε ιησουν απολεσωσιν
			<A NAME=40N_27_21><B> 27,21 </B>αποκριθεις δε ο ηγεμων ειπεν αυτοις τινα θελετε απο των δυο απολυσω υμιν οι δε ειπαν τον βαραββαν
			<A NAME=40N_27_22><B> 27,22 </B>λεγει αυτοις ο πειλατος τι ουν ποιησω ιησουν τον λεγομενον χριστον λεγουσιν παντες σταυρωθητω
			<A NAME=40N_27_23><B> 27,23 </B>ο δε εφη τι γαρ κακον εποιησεν οι δε περισσως εκραζον λεγοντες σταυρωθητω
			<A NAME=40N_27_24><B> 27,24 </B>ιδων δε ο πειλατος οτι ουδεν ωφελει αλλα μαλλον θορυβος γινεται λαβων υδωρ απενιψατο τας χειρας απεναντι του οχλου λεγων αθωος ειμι απο του αιματος τουτου υμεις οψεσθε
			<A NAME=40N_27_25><B> 27,25 </B>και αποκριθεις πας ο λαος ειπεν το αιμα αυτου εφ ημας και επι τα τεκνα ημων
			<A NAME=40N_27_26><B> 27,26 </B>τοτε απελυσεν αυτοις τον βαραββαν τον δε ιησουν φραγελλωσας παρεδωκεν ινα σταυρωθη
			<A NAME=40N_27_27><B> 27,27 </B>τοτε οι στρατιωται του ηγεμονος παραλαβοντες τον ιησουν εις το πραιτωριον συνηγαγον επ αυτον ολην την σπειραν
			<A NAME=40N_27_28><B> 27,28 </B>και εκδυσαντες αυτον χλαμυδα κοκκινην περιεθηκαν αυτω
			<A NAME=40N_27_29><B> 27,29 </B>και πλεξαντες στεφανον εξ ακανθων επεθηκαν επι της κεφαλης αυτου και καλαμον εν τη δεξια αυτου και γονυπετησαντες εμπροσθεν αυτου ενεπαιξαν αυτω λεγοντες χαιρε βασιλευς των ιουδαιων
			<A NAME=40N_27_30><B> 27,30 </B>και εμπτυσαντες εις αυτον ελαβον τον καλαμον και ετυπτον εις την κεφαλην αυτου
			<A NAME=40N_27_31><B> 27,31 </B>και οτε ενεπαιξαν αυτω εξεδυσαντες αυτον την χλαμυδα ενεδυσαν αυτον τα ιματια αυτου και απηγαγον αυτον εις το σταυρωσαι
			<A NAME=40N_27_32><B> 27,32 </B>εξερχομενοι δε ευρον ανθρωπον κυρηναιον ονοματι σιμωνα τουτον ηγγαρευσαν ινα αρη τον σταυρον αυτου
			<A NAME=40N_27_33><B> 27,33 </B>και ελθοντες εις τοπον λεγομενον γολγοθα ο εστιν κρανιου τοπος λεγομενος
			<A NAME=40N_27_34><B> 27,34 </B>εδωκαν αυτω πειν οινον μετα χολης μεμιγμενον και γευσαμενος ουκ ηθελησεν πειν
			<A NAME=40N_27_35><B> 27,35 </B>σταυρωσαντες δε αυτον διεμερισαντο τα ιματια αυτου βαλοντες κληρον
			<A NAME=40N_27_36><B> 27,36 </B>και καθημενοι ετηρουν αυτον εκει
			<A NAME=40N_27_37><B> 27,37 </B>και επεθηκαν επανω της κεφαλης αυτου την αιτιαν αυτου γεγραμμενην ουτος εστιν ιησους ο βασιλευς των ιουδαιων
			<A NAME=40N_27_38><B> 27,38 </B>τοτε σταυρουνται συν αυτω δυο λησται εις εκ δεξιων και εις εξ ευωνυμων
			<A NAME=40N_27_39><B> 27,39 </B>οι δε παραπορευομενοι εβλασφημουν αυτον κινουντες τας κεφαλας αυτων
			<A NAME=40N_27_40><B> 27,40 </B>και λεγοντες ο καταλυων τον ναον και εν τρισιν ημεραις οικοδομων σωσον σεαυτον ει υιος ει του θεου και καταβηθι απο του σταυρου
			<A NAME=40N_27_41><B> 27,41 </B>ομοιως οι αρχιερεις εμπαιζοντες μετα των γραμματεων και πρεσβυτερων ελεγον
			<A NAME=40N_27_42><B> 27,42 </B>αλλους εσωσεν εαυτον ου δυναται σωσαι βασιλευς ισραηλ εστιν καταβατω νυν απο του σταυρου και πιστευσωμεν επ αυτον
			<A NAME=40N_27_43><B> 27,43 </B>πεποιθεν επι τον θεον ρυσασθω νυν ει θελει αυτον ειπεν γαρ οτι θεου ειμι υιος
			<A NAME=40N_27_44><B> 27,44 </B>το δ αυτο και οι λησται οι συσταυρωθεντες συν αυτω ωνειδιζον αυτον
			<A NAME=40N_27_45><B> 27,45 </B>απο δε εκτης ωρας σκοτος εγενετο επι πασαν την γην εως ωρας ενατης
			<A NAME=40N_27_46><B> 27,46 </B>περι δε την ενατην ωραν ανεβοησεν ο ιησους φωνη μεγαλη λεγων ηλει ηλει λεμα σαβαχθανει τουτ εστιν θεε μου θεε μου ινατι με εγκατελιπες
			<A NAME=40N_27_47><B> 27,47 </B>τινες δε των εκει εστηκοτων ακουσαντες ελεγον οτι ηλειαν φωνει ουτος
			<A NAME=40N_27_48><B> 27,48 </B>και ευθεως δραμων εις εξ αυτων και λαβων σπογγον πλησας τε οξους και περιθεις καλαμω εποτιζεν αυτον
			<A NAME=40N_27_49><B> 27,49 </B>οι δε λοιποι ελεγον αφες ιδωμεν ει ερχεται ηλειας σωσων αυτον
			<A NAME=40N_27_50><B> 27,50 </B>ο δε ιησους παλιν κραξας φωνη μεγαλη αφηκεν το πνευμα
			<A NAME=40N_27_51><B> 27,51 </B>και ιδου το καταπετασμα του ναου εσχισθη ανωθεν εως κατω εις δυο και η γη εσεισθη και αι πετραι εσχισθησαν
			<A NAME=40N_27_52><B> 27,52 </B>και τα μνημεια ανεωχθησαν και πολλα σωματα των κεκοιμημενων αγιων ηγερθησαν
			<A NAME=40N_27_53><B> 27,53 </B>και εξελθοντες εκ των μνημειων μετα την εγερσιν αυτου εισηλθον εις την αγιαν πολιν και ενεφανισθησαν πολλοις
			<A NAME=40N_27_54><B> 27,54 </B>ο δε εκατονταρχης και οι μετ αυτου τηρουντες τον ιησουν ιδοντες τον σεισμον και τα γινομενα εφοβηθησαν σφοδρα λεγοντες αληθως θεου υιος ην ουτος
			<A NAME=40N_27_55><B> 27,55 </B>ησαν δε εκει γυναικες πολλαι απο μακροθεν θεωρουσαι αιτινες ηκολουθησαν τω ιησου απο της γαλιλαιας διακονουσαι αυτω
			<A NAME=40N_27_56><B> 27,56 </B>εν αις ην μαρια η μαγδαληνη και μαρια η του ιακωβου και ιωσηφ μητηρ και η μητηρ των υιων ζεβεδαιου
			<A NAME=40N_27_57><B> 27,57 </B>οψιας δε γενομενης ηλθεν ανθρωπος πλουσιος απο αριμαθαιας τουνομα ιωσηφ ος και αυτος εμαθητευθη τω ιησου
			<A NAME=40N_27_58><B> 27,58 </B>ουτος προσελθων τω πειλατω ητησατο το σωμα του ιησου τοτε ο πειλατος εκελευσεν αποδοθηναι
			<A NAME=40N_27_59><B> 27,59 </B>και λαβων το σωμα ο ιωσηφ ενετυλιξεν αυτο σινδονι καθαρα
			<A NAME=40N_27_60><B> 27,60 </B>και εθηκεν αυτο εν τω καινω αυτου μνημειω ο ελατομησεν εν τη πετρα και προσκυλισας λιθον μεγαν τη θυρα του μνημειου απηλθεν
			<A NAME=40N_27_61><B> 27,61 </B>ην δε εκει μαριαμ η μαγδαληνη και η αλλη μαρια καθημεναι απεναντι του ταφου
			<A NAME=40N_27_62><B> 27,62 </B>τη δε επαυριον ητις εστιν μετα την παρασκευην συνηχθησαν οι αρχιερεις και οι φαρισαιοι προς πειλατον
			<A NAME=40N_27_63><B> 27,63 </B>λεγοντες κυριε εμνησθημεν οτι εκεινος ο πλανος ειπεν ετι ζων μετα τρεις ημερας εγειρομαι
			<A NAME=40N_27_64><B> 27,64 </B>κελευσον ουν ασφαλισθηναι τον ταφον εως της τριτης ημερας μηποτε ελθοντες οι μαθηται κλεψωσιν αυτον και ειπωσιν τω λαω ηγερθη απο των νεκρων και εσται η εσχατη πλανη χειρων της πρωτης
			<A NAME=40N_27_65><B> 27,65 </B>εφη αυτοις ο πειλατος εχετε κουστωδιαν υπαγετε ασφαλισασθε ως οιδατε
			<A NAME=40N_27_66><B> 27,66 </B>οι δε πορευθεντες ησφαλισαντο τον ταφον σφραγισαντες τον λιθον μετα της κουστωδιας
			</P><P ALIGN= "CENTER"><A NAME=40N_28><H2>Matthäus<BR>Kapitel 28</H2></P><P ALIGN= "LEFT"><A NAME=40N_28_2><B> 28,2 </B>και ιδου σεισμος εγενετο μεγας αγγελος γαρ κυριου καταβας εξ ουρανου και προσελθων απεκυλισεν τον λιθον και εκαθητο επανω αυτου
			<A NAME=40N_28_3><B> 28,3 </B>ην δε η ειδεα αυτου ως αστραπη και το ενδυμα αυτου λευκον ως χιων
			<A NAME=40N_28_4><B> 28,4 </B>απο δε του φοβου αυτου εσεισθησαν οι τηρουντες και εγενηθησαν ως νεκροι
			<A NAME=40N_28_5><B> 28,5 </B>αποκριθεις δε ο αγγελος ειπεν ταις γυναιξιν μη φοβεισθε υμεις οιδα γαρ οτι ιησουν τον εσταυρωμενον ζητειτε
			<A NAME=40N_28_6><B> 28,6 </B>ουκ εστιν ωδε ηγερθη γαρ καθως ειπεν δευτε ιδετε τον τοπον οπου εκειτο
			<A NAME=40N_28_7><B> 28,7 </B>και ταχυ πορευθεισαι ειπατε τοις μαθηταις αυτου οτι ηγερθη απο των νεκρων και ιδου προαγει υμας εις την γαλιλαιαν εκει αυτον οψεσθε ιδου ειπον υμιν
			<A NAME=40N_28_8><B> 28,8 </B>και απελθουσαι ταχυ απο του μνημειου μετα φοβου και χαρας μεγαλης εδραμον απαγγειλαι τοις μαθηταις αυτου
			<A NAME=40N_28_9><B> 28,9 </B>και ιδου ιησους υπηντησεν αυταις λεγων χαιρετε αι δε προσελθουσαι εκρατησαν αυτου τους ποδας και προσεκυνησαν αυτω
			<A NAME=40N_28_10><B> 28,10 </B>τοτε λεγει αυταις ο ιησους μη φοβεισθε υπαγετε απαγγειλατε τοις αδελφοις μου ινα απελθωσιν εις την γαλιλαιαν και εκει με οψονται
			<A NAME=40N_28_11><B> 28,11 </B>πορευομενων δε αυτων ιδου τινες της κουστωδιας ελθοντες εις την πολιν απηγγειλαν τοις αρχιερευσιν απαντα τα γενομενα
			<A NAME=40N_28_12><B> 28,12 </B>και συναχθεντες μετα των πρεσβυτερων συμβουλιον τε λαβοντες αργυρια ικανα εδωκαν τοις στρατιωταις
			<A NAME=40N_28_13><B> 28,13 </B>λεγοντες ειπατε οτι οι μαθηται αυτου νυκτος ελθοντες εκλεψαν αυτον ημων κοιμωμενων
			<A NAME=40N_28_14><B> 28,14 </B>και εαν ακουσθη τουτο επι του ηγεμονος ημεις πεισομεν και υμας αμεριμνους ποιησομεν
			<A NAME=40N_28_15><B> 28,15 </B>οι δε λαβοντες τα αργυρια εποιησαν ως εδιδαχθησαν και εφημισθη ο λογος ουτος παρα ιουδαιοις μεχρι της σημερον
			<A NAME=40N_28_16><B> 28,16 </B>οι δε ενδεκα μαθηται επορευθησαν εις την γαλιλαιαν εις το ορος ου εταξατο αυτοις ο ιησους
			<A NAME=40N_28_17><B> 28,17 </B>και ιδοντες αυτον προσεκυνησαν οι δε εδιστασαν
			<A NAME=40N_28_18><B> 28,18 </B>και προσελθων ο ιησους ελαλησεν αυτοις λεγων εδοθη μοι πασα εξουσια εν ουρανω και επι γης
			<A NAME=40N_28_19><B> 28,19 </B>πορευθεντες μαθητευσατε παντα τα εθνη βαπτιζοντες αυτους εις το ονομα του πατρος και του υιου και του αγιου πνευματος
			<A NAME=40N_28_20><B> 28,20 </B>διδασκοντες αυτους τηρειν παντα οσα ενετειλαμην υμιν και ιδου εγω μεθ υμων ειμι πασας τας ημερας εως της συντελειας του αιωνοςοι δε πορευθεντες ησφαλισαντο τον ταφον σφραγισαντες τον λιθον μετα της κουστωδιας
			"""
		self.SetPage(default)